/*
 * GraphBase.cpp
 *
 *  Created on: 5.1.2013
 *      Author: petr
 */

#include "GraphBase.h"

namespace Graph {

GraphBase::GraphBase(const Graph* graph) {
	listCnt = 0;
	allocatedCnt = 0;
	lists = NULL;
	parentGraph = graph;

}

GraphBase::GraphBase(const Graph* graph, int cnt) {
	listCnt = 0;
	allocatedCnt = 0;
	lists = NULL;
	createLists(cnt);
	parentGraph = graph;
}

GraphBase::~GraphBase() {
	// odebirani dat
	for (int i = listCnt; i > 0; i--) {
		deleteList(i - 1);
	}

	// dealokace bloku pameti s polem
	allocator<BaseList*> locator;

	locator.deallocate(lists, allocatedCnt);
}

int GraphBase::createList() {
	// kontrola, jestli je dost mista
	int neededSlots = listCnt + 1;

	if (neededSlots > allocatedCnt) {
		allocate(neededSlots);
	}

	// vytvoreni noveho seznamu na dane pozici
	int index = listCnt;

	lists[listCnt] = new BaseList;

	// zvetseni poctu seznamu a vraceni indexu
	listCnt = neededSlots;

	return index;
}

int GraphBase::createLists(int cnt) {
	// kontrola, jestli je dostatek mista
	int neededSlots = cnt + listCnt;

	if (neededSlots > allocatedCnt) {
		allocate(neededSlots);
	}

	// zapis novych seznamu
	for (int i = 0; i < cnt; i++) {
		createList();
	}

	return neededSlots;
}

void GraphBase::deleteList(int index) {
	BaseList* toDelete = getList(index);

	if (toDelete == NULL)
		return;

	// odebrani dat
	toDelete->clear();
	delete toDelete;
	toDelete = NULL;

	// posunuti zbytku poli
	for (int i = index + 1; i < listCnt; i++) {
		lists[i - 1] = lists[i];
	}
}

int GraphBase::getCount() {
	return listCnt;
}

void GraphBase::allocate(int n) {
	// neni dostatek mistak - musime zaalokovat dalsi
	allocator<BaseList*> locator;
	BaseList** tmp;

	int newCount = allocatedCnt + n;

	tmp = locator.allocate(newCount);

	// prekopirovani stare pameti
	for (int i = 0; i < listCnt; i++) {
		tmp[i] = lists[i];
	}

	// dealokace a swap
	locator.deallocate(lists, allocatedCnt);
	lists = tmp;
	allocatedCnt = newCount;
}

GraphBase::BaseList* GraphBase::getList(int index) const {
	if (index > listCnt)
		return NULL;

	return lists[index];
}

void GraphBase::addToToOtherList(GraphBase* other, int index) {
	BaseList* otherList = other->lists[index];

	BaseList::iterator pos = otherList->begin(), end = otherList->end();

	while (pos != end) {
		// kontrola, jestli souhlasi adresy
		// pokud ano, nebude se prvek pridavat znovu
		if ((*pos) == this) return;

		pos++;
	}

	otherList->push_back(this);
}

void GraphBase::removeFromOtherList(GraphBase* other, int index) {
	BaseList* otherList = other->lists[index];
	BaseList::iterator pos = otherList->begin(), end = otherList->end(), tmp;

	while (pos != end) {
		// pokud ukazatel odpovida tomuto prvku, pak se odebere
		if ((*pos) == this) {
			pos = otherList->erase(pos);
		} else {
			pos++;
		}
	}
}

void GraphBase::removeFromOtherLists(GraphBase* other) {
	// odebirani postupne ze vsech seznamu
	for (int i = 0; i < other->listCnt; i++) {
		removeFromOtherList(other, i);
	}
}

const Graph* GraphBase::graph() {
	return parentGraph;
}

GraphBase& GraphBase::operator =(const GraphBase& o) {

	return *this;
}

} /* namespace SQLRepresenter */
