/*
 * Loader.h
 *
 *  Created on: 27.2.2013
 *      Author: petr
 */

#ifndef LOADER_H_
#define LOADER_H_

#include <string>
#include <cstring>
#include "ParamLoadable.h"
#include "InputData.h"
#include "Transition.h"

namespace FiniteAutomat {

#define WRITE_LOAD_HEADER_BASE std::string load(InputData& data, Transition* transition)
#define WRITE_LOAD_HEADER virtual WRITE_LOAD_HEADER_BASE;
#define WRITE_LOAD_HEADER_VIRTUAL virtual WRITE_LOAD_HEADER_BASE = 0;
#define WRITE_LOAD_IMPL(CLASS) std::string CLASS::load(InputData& data, Transition* transition)

#define WRITE_LOADERCLONE_HEADER virtual Loader* clone();
#define WRITE_LOADERCLONE_IMPL(CLASS) Loader* CLASS::clone() { return new CLASS(*this); }

#ifndef CLASS_TRANSITION
#define CLASS_TRANSITION
class Transition;
#endif

#define CLASS_LOADER

class Loader: public FiniteAutomat::ParamLoadable {
public:

	/**
	 * seznam bílých znaků
	 *
	 * @var const char*
	 */
	static const char* WHITE_CHARS;

	/**
	 * jmeno parametru pro preskakovani pocatecnich bilych znaku
	 *
	 * @var std::stirng
	 */
	static const std::string PARAM_SKIPWHITES;

	/**
	 * pokud je nastavno TRUE, pak se automaticky preskakuji pocatecni bile znaky
	 *
	 * @var bool
	 */
	bool skipStartWhites;

	Loader();
	Loader(const Loader& o);
	virtual ~Loader();

	/**
	 * naklonuje instanci loaderu
	 * vraci ukazatel na novou instanci
	 *
	 * @return Loader*
	 */
	virtual Loader* clone() = 0;

	/*
	 * provede nacteni dat ze vstupnich dat a posune ukazatel pozice
	 * v zavislosti na zpusovu provadeni naciteni je mozne, ze bude provadet sam filtraci a validaci
	 * vraci nactena data v surove podobe
	 *
	 * @param InputData& data reference na predavana data
	 * @param Transition* transition prechod, ze ktereho je nacitani volano
	 * @return std::string
	 */
	WRITE_LOAD_HEADER_VIRTUAL

	/**
	 * preskoci bile znaky od pozice az k prvnimu nebilemu znaku
	 * vraci novou instanci InputData s ukazatelem nastavenym na prvny nebily znak od puvodni pozice
	 *
	 * @param const InputData& vstupni data
	 * @return InputData
	 */
	InputData skipWhites(const InputData& data);

	/**
	 * nastaveni parametru
	 * podporovany jsou parametry
	 * - skipwhites -> preskakovani pocatecnich bilych znaku
	 *
	 * @param std::string name jmeno parametru
	 * @param std::string value hodnota parametru reprezentaovana jako retezec
	 */
	WRITE_SETPARAM_HEADER
};

} /* namespace FiniteAutomat */
#endif /* LOADER_H_ */
