/*
 * State.h
 *
 *  Created on: 26.2.2013
 *      Author: petr
 */

#ifndef STATE_H_
#define STATE_H_

#include "GraphNode.h"
#include "Machine.h"
#include "Transition.h"
#include "StateInstance.h"
#include "ExceptionNotFinalState.h"
#include "Processor.h"

namespace FiniteAutomat {

#define CLASS_STATE

#ifndef TYPE_TRANSITIONLIST
#define TYPE_TRANSITIONLIST
class Transition;
typedef list<Transition*> TransitionList;
#endif

#ifndef CLASS_MACHINE
class Machine;
#endif

#ifndef CLASS_STATEINSTANCE
class StateInstance;
#endif

class State: public Graph::GraphNode {

	/**
	 * prepinac, jetli je stav koncovy
	 *
	 * @property bool
	 */
	bool isFinal;

	/**
	 * instance generatoru dat
	 */
	Processor* processor;

protected:

	/**
	 * porovna prioritu dvou prechodu
	 * vraci TRUE, pokud
	 *
	 * @param Transition* t1 prvni prechod
	 * @param Transition* t2 druhy prechod
	 * @return bool
	 */
	static bool compareTransitions(Graph::GraphBase* t1, Graph::GraphBase* t2);

	/**
	 * vybere prechod k provedeni a aktualizuje parametr nextTransition
	 * pokud neni zadna hrana nalezena, nextTransition ukazuje jeden prvek za konec seznamu
	 *
	 * @param TransitionList::iterator& nextTransition iterator na dalsi hranu
	 * @param const InputData& data vstupni data
	 */
	void chooseTransition(TransitionList::const_iterator& nextTransition, InputData& data) const;

public:
	State(Machine* machine);
	State(const State& o);
	virtual ~State();

	/**
	 * prida novy prechod a vraci ukazatel na nej
	 *
	 * @param State* to stav, do ktereho prechod smeruje
	 * @param unsigned char priority priorita prechodu
	 * @return Transition*
	 */
	Transition* addTransition(State* to, unsigned char priority);

	/**
	 * vytvori instanci stavu
	 */
	virtual StateInstance* createInstance(InputData data, StateInstance* prev, void* processData, TRANSIT_RESULT readData);

	/**
	 * vraci nastaveni prepinace vystupniho stavu
	 *
	 * @return bool
	 */
	bool getFinal();

	/**
	 * vraci aktualne nastaveny procesor
	 *
	 * @return Processor*
	 */
	Processor* getProcesor() const;

	/**
	 * seradi prechody dle priority
	 */
	void orderTransitions();

	/**
	 * odebere prechod
	 *
	 * @param Transition* transition prechod k odebrani
	 */
	void removeTransition(Transition* transiton);

	/**
	 * nastavi hodnotu prepinace vystupniho stavu
	 */
	void setFinal(bool final);

	/**
	 * nastavi novy procesor
	 *
	 * @param Processor* processor novy procesor
	 */
	virtual void setProcessor(Processor* processor);

	/**
	 * provede prechod na úrechodu danem iteratorem
	 * vraci ukazatel na instanci stavu, do ktereho se dostane
	 *
	 * @param Graph::EdgeList::iterator iterator ukazujici na prechod, ktery ma byt proveden
	 * @return StateInstance*
	 */
	virtual StateInstance* transit(const InputData data, StateInstance* prev, TransitionList::const_iterator& nextTransition, void* processData) const;

	/**
	 * vraci seznam prechodu
	 *
	 * @return TransitionList&
	 */
	const TransitionList& transitions() const;
};

} /* namespace FiniteAutomat */
#endif /* STATE_H_ */
