/*
 * StateInstance.h
 *
 *  Created on: 27.2.2013
 *      Author: petr
 */

#ifndef STATEINSTANCE_H_
#define STATEINSTANCE_H_

#include "State.h"
#include "Graph.h"
#include "InputData.h"
#include "Transition.h"

namespace FiniteAutomat {

#ifndef TYPE_TRANSITIONLIST
#define TYPE_TRANSITIONLIST
class Transition;
typedef list<Transition*> TransitionList;
#endif

#ifndef CLASS_STATE
class State;
#endif

#define CLASS_STATEINSTANCE

class StateInstance {

	InputData inputData;

	TransitionList::const_iterator nextTransition;

	StateInstance* next;

	StateInstance* prev;

	TRANSIT_RESULT readedData;

	const State* state;

protected:

	void* processData;

	/**
	 * vrátí data do původního stavu
	 */
	virtual void revertData();

public:
	StateInstance(const State* state, InputData data, StateInstance* prev, void* processData, TRANSIT_RESULT readData);

	/*
	 * destruktor likviduje i nasledujici instanci (
	 */
	virtual ~StateInstance();

	/*
	 * pripoji nasledujici stav
	 * pokud je uz nejaky stav pripojen, je volana metoda revert()
	 */
	void append(StateInstance* instance);

	/**
	 * vraci ukazatel na konecou podobu zpracovanych dat
	 *
	 * @return void*
	 */
	void* getFinalData();

	/**
	 * vraci ukazatel na posledni instanci v retezu
	 *
	 * @return StateInstance*
	 */
	StateInstance* getFinalInstance();

	/*
	 * vraci ukazatel na nasledujici instanci
	 * pokud je tato instance posledni, vraci NULL
	 */
	StateInstance* getNext();

	/*
	 * vraci ukazatel na prechozi instanci
	 * pokud je tato instance prvni, vraci NULL
	 */
	StateInstance* getPrev();

	/**
	 * vraci ukazatel na generovana data
	 *
	 * @return void*
	 */
	void* getProcessData();

	/**
	 * vraci stav, ke kteremu je instance vazana
	 *
	 * @return State*
	 */
	const State* getState();

	/**
	 * provede revert a nastavi
	 * nasledujici stav na prvni v seznamu
	 */
	void reset();

	/**
	 * odrizne a znici nasledujici prvek
	 */
	virtual void revert();

	/**
	 * provede prechod do dalsiho stavu
	 * testovani zacina u nextTransit
	 * vraci ukazatel na novou instanci stavu
	 *
	 * @return StateInstance*
	 */
	virtual StateInstance* transit();
};

} /* namespace FiniteAutomat */
#endif /* STATEINSTANCE_H_ */
