/*
 * Transition.h
 *
 *  Created on: 27.2.2013
 *      Author: petr
 */

#ifndef TRANSITION_H_
#define TRANSITION_H_

#include <string>
#include <list>
#include <algorithm>
#include "config.h"
#include "GraphEdge.h"
#include "TestList.h"
#include "FilterList.h"
#include "State.h"
#include "InputData.h"
#include "Loader.h"
#include "ExceptionTransitionUnavailable.h"

namespace FiniteAutomat {

#ifndef CLASS_STATE
class State;
#endif

#ifndef TYPE_TRANSITIONLIST
#define TYPE_TRANSITIONLIST
class Transition;
typedef list<Transition*> TransitionList;
#endif

#ifndef CLASS_LOADER
#define CLASS_LOADER
class Loader;
#endif

#define TRANSITION_EDGE_TYPE 1

#define CLASS_TRANSITION

class Transition: public Graph::GraphEdge {

	/*
	 * seznam filtru
	 * filtry se provadeji pred testovanim
	 */
	FilterList filters;

	/*
	 * seznam testu
	 * testy kontroluji, zda je mozne prechod provest
	 */
	TestList tests;

	/**
	 * nacitac dat
	 */
	Loader* loader;

	/*
	 * priorita provedeni prechodu
	 */
	unsigned char priority;

public:
	Transition(State* from, State* to, unsigned char priority);
	virtual ~Transition();

	/*
	 * vraci referenci na seznam filtru
	 *
	 * @reutrn FilterList&
	 */
	FilterList& getFilters();

	/**
	 * vraci nastaveny loader
	 *
	 * @return Loader*
	 */
	Loader* getLoader();

	/*
	 * vraci prioritu provedeni
	 *
	 * @return unsigned char
	 */
	unsigned char getPriority();

	/*
	 * vraci seznam testu
	 *
	 * @return TestList&
	 */
	TestList& getTests();

	/*
	 * provede testy a vraci TRUE, pokud vsechny testy probehly spravne
	 *
	 * @param const InputData& data kontrolovana data
	 * @return bool
	 */
	bool isAvailable(const InputData& data);

	/**
	 * nastavi novy loader
	 *
	 * @param Loader* loader novy loader
	 */
	void setLoader(Loader* loader);

	/*
	 * nastavi novou prioritu
	 *
	 * @param unsigned char p nova priorita
	 */
	void setPriority(unsigned char p);

	/*
	 * realizuje prechod
	 * do ukazatele v druhem parametru zapise nactena data v prefiltrovane podobe
	 * vraci nova InputData s posunutym ukazatelem
	 *
	 * @param const InputData& data vstupni data ke zpracovani
	 * @param std::string* readData reference na retezec, kam se budou zapisovat nactena data
	 * @return InputData
	 */
	virtual InputData transit(const InputData& data, TRANSIT_RESULT* readData);
};

} /* namespace FiniteAutomat */
#endif /* TRANSITION_H_ */
