/*
 * XMLLoader.h
 *
 *  Created on: 12.3.2013
 *      Author: petr
 */

#ifndef XMLLOADER_H_
#define XMLLOADER_H_

#include <string>
#include <map>
#include "Machine.h"
#include "StateAgregated.h"
#include "pugixml.hpp"
#include "FactoryAssembler.h"
#include "FactoryFilter.h"
#include "FactoryLoader.h"
#include "FactoryProcessor.h"
#include "FactoryTest.h"

namespace FiniteAutomat {

class XMLLoader {

	/**
	 * typ uchovavajici informace o jmenech prvku
	 */
	typedef std::map<std::string, Graph::GraphBase*> NIM;

	/**
	 * seznam existujicich automatu
	 */
	NIM machines;

	/**
	 * vytvori a nacte definici automatu
	 *
	 * @param pugi::xml_node node uzel s definici automatu
	 * @param Machine* machine automat, do ktereho se bude zapisovat
	 * @return Machine*
	 */
	Machine* loadMachine(pugi::xml_node node, Machine* machine);

	/**
	 * nacte stavy a nastavi je
	 *
	 * @param pugi::xml_node node uzel s definici
	 * @param Machine* machine automat, ke kteremu uzel nalezi
	 * @param NIM &nim seznam pojmenovanych prvku automatu, kam bude prvek pridan
	 * @return State*
	 */
	Machine* loadSates(pugi::xml_node node, Machine* machine, NIM& nim);

	/**
	 * nacte stavy se zanorenym automatem a nastavi je
	 *
	 * @param pugi::xml_node node uzel s definicemi
	 * @param Machine* machine automat, do ktereho se bude stav zapisovat
	 * @param NIM& nim seznam pojmenovanych stavu
	 * @return StateAgregated*
	 */
	Machine* loadStatesAgregated(pugi::xml_node node, Machine* machine, NIM& nim);

	/**
	 * nacte prechod
	 *
	 * @param pugi::xml_node node uzel s definici prechodu
	 * @param Machine* machine automat, ke kteremu prechod patri
	 * @param NIM& nim pojmenovane prvky
	 */
	Transition* loadTransition(pugi::xml_node node, Machine* machine, NIM& nim);

	/**
	 * nacte filtry a zapise je do prechodu
	 *
	 * @param pugi::xml_node node uzel se zdrojovymi daty
	 * @param Transition* transition prechod do ktereho se bude zapisovat
	 * @return Transition*
	 */
	Transition* loadFilters(pugi::xml_node node, Transition* transition);

	/**
	 * nacte testy a zapise je do prechodu
	 *
	 * @param pugi::xml_node node uzel se zdrojovymi informacemi
	 * @param Transition* transition prechod do ktereho se bude zapisovat
	 * @return Transition*
	 */
	Transition* loadTests(pugi::xml_node node, Transition* transition);

	void setStdStateValues(pugi::xml_node stateNode, State* state, NIM& nim);

	FactoryAssembler *fAss;

	FactoryFilter *fFil;

	FactoryProcessor *fProc;

	FactoryTest *fTest;

	FactoryLoader *fLoad;

protected:

	/**
	 * zapise parametry z uzlu do objektu
	 *
	 * @param pugi::xml_node node uzel XML, ktery obsahuje definci objektu
	 * @param PAramLoadable &item reference na parametrizovany objekt
	 */
	void setParameters(pugi::xml_node node, ParamLoadable &item);

public:

	static const pugi::char_t* ELEMENT_ROOT;
	static const pugi::char_t* ELEMENT_MACHINE;
	static const pugi::char_t* ELEMENT_STATE;
	static const pugi::char_t* ELEMENT_STATEAGREGATED;
	static const pugi::char_t* ELEMENT_TRANSITION;
	static const pugi::char_t* ELEMENT_PROCESSOR;
	static const pugi::char_t* ELEMENT_ASSEMBLER;
	static const pugi::char_t* ELEMENT_TEST;
	static const pugi::char_t* ELEMENT_FILTER;
	static const pugi::char_t* ELEMENT_LOADER;
	static const pugi::char_t* ELEMENT_PARAM;

	const static pugi::char_t* ATTR_NAME;
	const static pugi::char_t* ATTR_VALUE;
	const static pugi::char_t* ATTR_CNAME;
	const static pugi::char_t* ATTR_FROM;
	const static pugi::char_t* ATTR_TO;
	const static pugi::char_t* ATTR_STARTER;
	const static pugi::char_t* ATTR_FINAL;
	const static pugi::char_t* ATTR_AGREGATEDNAME;
	const static pugi::char_t* ATTR_PRIORITY;

	XMLLoader();
	virtual ~XMLLoader();

	Machine* load(std::string fileName);
};

} /* namespace FiniteAutomat */
#endif /* XMLLOADER_H_ */
