/*
 * InputData.cpp
 *
 *  Created on: 26.2.2013
 *      Author: petr
 */

#include "InputData.h"

namespace FiniteAutomat {

InputData::InputData() : InputData("", 0) {

}

InputData::InputData(const std::string& data, std::size_t position = 0) {
	this->data = new std::string(data);
	length = this->data->size();

	// vytvoreni a nastaveni pocitadla instanci
	instanceCounter = new std::size_t;
	(*instanceCounter) = 1;
	setPosition(position);
}

InputData::InputData(const InputData& o) {
	data = o.data;
	position = o.position;
	length = o.length;

	// kopirovani a pricteni pocitadla instanci
	instanceCounter = o.instanceCounter;
	(*instanceCounter)++;
}

InputData::~InputData() {
	--(*instanceCounter);

	if ((*instanceCounter) == 0) {
		// pocet instanci klesl na nulu, znici se retezec
		delete this->data;
	}
}

std::string InputData::getData() const {
	return std::string(*data);
}

std::size_t InputData::getLength() const {
	return length;
}

std::size_t InputData::getPosition() const {
	return position;
}

std::string InputData::getSubstr(std::size_t n, bool move) {
	// ziskani navratoveho podretezce
	std::string retVal;
	retVal = data->substr(position, n);

	if (move) {
		// move == TRUE -> posune se ukazatel position
		position += n;

		if (position > length) {
			// nova pozice se nachazi za koncem dat - modifikuje se
			position = length;
		}
	}

	return retVal;
}

void InputData::setPosition(std::size_t p) {
	if (p > length) throw ExceptionOutOfBounds();

	position = p;
}

InputData& InputData::operator =(const InputData& o) {
	// dekrementace ukazatele
	(*instanceCounter)--;

	if (instanceCounter == 0) {
		// zadna dalsi nstance neexistuje, znici se string
		delete data;
	}

	// prirazeni
	instanceCounter = o.instanceCounter;
	(*instanceCounter)++;
	data = o.data;
	length = o.length;
	position = o.position;

	return *this;
}

} /* namespace FiniteAutomat */
