/*
 * State.cpp
 *
 *  Created on: 26.2.2013
 *      Author: petr
 */

#include "State.h"

namespace FiniteAutomat {

State::State(Machine* machine) :
		GraphNode(machine) {
	isFinal = false;
	processor = NULL;
}

State::State(const State& o) :
		Graph::GraphNode(o) {
	isFinal = o.isFinal;

	if (o.processor == NULL) {
		// procesor puvodni instance neni nastaven
		processor = NULL;
	} else {
		// procesor puvodni instance je nastaven a naklonuje se
		processor = o.processor->clone();
	}
}

State::~State() {
	if (processor != NULL) delete processor;
}

Transition* State::addTransition(State* to, unsigned char priority) {
	return new Transition(this, to, priority);
}

void State::removeTransition(Transition* transition) {
	removeEdge(transition);
}

StateInstance* State::transit(InputData data, StateInstance* prev,
		TransitionList::const_iterator& nextTransition,
		void* processData) const {

	chooseTransition(nextTransition, data);

	if (nextTransition == transitions().end()) {
		// zadny dostupny prechod nebyl nalezen
		if (isFinal) {
			// tento stav je vystupni
			prev->append(NULL);
			return NULL;
		} else {
			// CHYBA - tento stav neni vystupni a neni kam jit
			throw ExceptionNotFinalState();
		}

	}

	// priprava dat na prechod
	Transition *trans = (Transition*) *nextTransition;
	TRANSIT_RESULT readData;
	InputData newData;

	// prechod do noveho stavu a posun iteratoru o jednicku
	newData = trans->transit(data, &readData);
	nextTransition++;

	StateInstance* retVal = ((State*) trans->to())->createInstance(newData, prev, processData, readData);
	prev->append(retVal);

	return retVal;
	// vygenerovani vysledku
}

bool State::getFinal() {
	return isFinal;
}

void State::setFinal(bool final) {
	isFinal = final;
}

void State::orderTransitions() {
	getList(LIST_CHILDS)->sort(&compareTransitions);
}

bool State::compareTransitions(Graph::GraphBase* t1, Graph::GraphBase* t2) {
	return ((Transition*) t1)->getPriority() < ((Transition*) t2)->getPriority();
}

Processor* State::getProcesor() const {
	return processor;
}

void State::setProcessor(Processor* processor) {
	this->processor = processor->clone();
}

void State::chooseTransition(TransitionList::const_iterator& nextTransition,
		InputData& data) const {

	TransitionList *edges = reinterpret_cast<TransitionList*>(getList(
			LIST_CHILDS));

	TransitionList::const_iterator end = edges->end();
	bool found = false;

	while (!found && nextTransition != end) {
		Transition* trans = (Transition*) *nextTransition;

		if (trans->isAvailable(data)) {
			// prechod je dostupny
			found = true;
		} else {
			// prechod neni dostupny, pokracuje se na dalsi
			nextTransition++;
		}
	}
}

StateInstance* State::createInstance(InputData data, StateInstance* prev,
		void* processData, TRANSIT_RESULT readData) {


	return new StateInstance(this, data,
			prev, processData, readData);
}

const TransitionList& State::transitions() const {
	Graph::GraphBase::BaseList* edges = getList(LIST_CHILDS);

	return *reinterpret_cast<TransitionList*>(edges);
}

} /* namespace FiniteAutomat */
