/*
 * StateInstance.cpp
 *
 *  Created on: 27.2.2013
 *      Author: petr
 */

#include "StateInstance.h"

namespace FiniteAutomat {


StateInstance::StateInstance(const State* state, InputData data,
		StateInstance* prev, void* processData, TRANSIT_RESULT readData) {

	next = NULL;
	this->prev = prev;
	this->state = state;
	inputData = data;
	this->nextTransition = state->transitions().begin();
	this->readedData = readedData;

	if (state->getProcesor() == NULL) {
		this->processData = processData;
	} else {
		this->processData = state->getProcesor()->process(processData, readData);
	}

}

StateInstance::~StateInstance() {
	// musime zrusit nasledujici zretezene instance, aby nezustaly v pameti
	if (next != NULL) delete next;
}

void StateInstance::append(StateInstance* instance) {
	// musime zrusit nasledujici instance
	revert();

	next = instance;

	if (instance != NULL) instance->prev = this;
}

StateInstance* StateInstance::getNext() {
	return next;
}

StateInstance* StateInstance::getPrev() {
	return prev;
}



void StateInstance::reset() {
	revert();
	nextTransition = state->transitions().begin();
}

void StateInstance::revert() {
	if (next != NULL) {
		// tento clanek neni poslednim clankem retezce
		// revertuji se data a nasledujici instance se znici

		next->revertData();		// tato akce revertuje data celym nasledujicim retezcem
		delete next;			// destruktor nasledujiciho prvku bude mit za nasledek zniceni celeho nasledujiciho retezce
	}

	next = NULL;
}

void* StateInstance::getProcessData() {
	return processData;
}

void* StateInstance::getFinalData() {
	if (next == NULL) {
		return processData;
	}

	return next->getFinalData();
}

StateInstance* StateInstance::getFinalInstance() {
	if (next == NULL) return this;

	return next->getFinalInstance();
}

void StateInstance::revertData() {
	Processor* proc = state->getProcesor();

	// vraceni retezce dat
	if (next != NULL) {
		// tato instance neni poslednim clankem retezce
		// musime provest revert dat i u nasledujiciho clanku, aby nedoslo k nekonzistenci dat
		next->revertData();
	}

	if (proc != NULL) {
		// procesor je nastaven, provede se revert dat
		proc->revert(processData, readedData);
	}
}

const State* StateInstance::getState() {
	return state;
}

StateInstance* StateInstance::transit() {
	if (next != NULL) {
		// existuje dalsi clanek retezu - musime ho odriznout
		revert();
	}

	state->transit(inputData, this, nextTransition, processData);
	return (next == NULL) ? NULL : this->getFinalInstance();
}

} /* namespace FiniteAutomat */
