/*
 * Transition.cpp
 *
 *  Created on: 27.2.2013
 *      Author: petr
 */

#include "Transition.h"

namespace FiniteAutomat {

Transition::Transition(State* from, State* to, unsigned char priority) : Graph::GraphEdge(from, to, TRANSITION_EDGE_TYPE) {
	this->priority = priority;
	loader = NULL;
}

Transition::~Transition() {
	if (loader != NULL) delete loader;
}

FilterList& Transition::getFilters() {
	return filters;
}

unsigned char Transition::getPriority() {
	return priority;
}

TestList& Transition::getTests() {
	return tests;
}

bool Transition::isAvailable(const InputData& data) {
	// profiltrovani a otestovani dat
	InputData dataCloned = data;
	std::string filtered = loader->load(dataCloned, this);
	filtered = filters.doFilters(filtered);

	return tests.doTests(filtered);
}

void Transition::setPriority(unsigned char p) {
	priority = p;
}

Loader* Transition::getLoader() {
	return loader;
}

void Transition::setLoader(Loader* loader) {
	this->loader = loader->clone();
}

InputData Transition::transit(const InputData& data, TRANSIT_RESULT* readData) {
	// nacteni dat
	InputData retVal(data);
	std::string procString = loader->load(retVal, this);		// retezec, ktery bude dale zpracovavan
	readData->raw = procString;

	// zpracovani dat a provedeni testu
	procString = filters.doFilters(procString);

	if (!tests.doTests(procString)) {
		// nactena data nevyhovela testum
		throw ExceptionTransitionUnavailable();
	}

	readData->filtered = procString;

	return retVal;
}

} /* namespace FiniteAutomat */
