/*
 * GraphNode.cpp
 *
 *  Created on: 6.1.2013
 *      Author: petr
 */

#include "GraphNode.h"

namespace Graph {

GraphNode::GraphNode(Graph* graph) :
		GraphBase(graph, 2) {

	nodeType = 0;
	addToToOtherList(graph, Graph::LIST_NODES);
}

void GraphNode::filter(BaseList* sourceList, NodeList* targetList,
		long int nodeType) {

	// vygenerovani iteratoru
	BaseList::iterator pos = sourceList->begin(), end = sourceList->end();
	GraphNode* node;
	GraphEdge* edge;

	while (pos != end) {
		// vyhodnoceni jestli typ sedi
		edge = (GraphEdge*) (*pos);

		// nacteni strany hrany
		if (edge->side(this) == 1) {
			node = edge->from();
		} else {
			node = edge->to();
		}

		if ((node->nodeType & nodeType) > 0) {
			targetList->push_back(node);
		}

		pos++;
	}
}

void GraphNode::filter(BaseList* sourceList, EdgeList* targetList,
		long int rel) {

	// vygenerovani iteratoru
	BaseList::iterator pos = sourceList->begin(), end = sourceList->end();
	GraphEdge* edge;

	while (pos != end) {
		// vyhodnoceni jestli typ sedi
		edge = (GraphEdge*) (*pos);

		if ((edge->rel & rel) > 0) {
			targetList->push_back(edge);
		}

		pos++;
	}
}

GraphNode::GraphNode(Graph* graph, GraphNode* parent) :
		GraphBase(graph, 2) {
	parent->addChild(this, 0);
	nodeType = 0;
}

GraphNode::~GraphNode() {
	deleteEdges(getList(LIST_CHILDS));
	deleteEdges(getList(LIST_PARENTS));

	removeFromOtherLists((GraphBase*) graph());
}

GraphEdge* GraphNode::addChild(GraphNode* child, long int rel) {
	// vytvoreni hrany
	GraphEdge* retVal = new GraphEdge(this, child, rel);

	return retVal;
}

EdgeList GraphNode::edges(long int rel) {
	// vygenerovani seznamu
	EdgeList retVal;

	filter(getList(LIST_PARENTS), &retVal, rel);
	filter(getList(LIST_CHILDS), &retVal, rel);

	return retVal;
}

EdgeList GraphNode::childEdges(long int rel) {
	EdgeList retVal;

	filter(getList(LIST_CHILDS), &retVal, rel);

	return retVal;
}

NodeList GraphNode::childNodes(long int nodeTypes) {
	NodeList retVal;

	filter(getList(LIST_CHILDS), &retVal, nodeTypes);

	return retVal;
}

NodeList GraphNode::nodes(long int nodeTypes) {
	// vygenerovani seznamu
	NodeList retVal;

	filter(getList(LIST_PARENTS), &retVal, nodeTypes);
	filter(getList(LIST_CHILDS), &retVal, nodeTypes);

	return retVal;
}

EdgeList GraphNode::parentEdges(long int rel) {
	EdgeList retVal;

	filter(getList(LIST_PARENTS), &retVal, rel);

	return retVal;
}

NodeList GraphNode::parentNodes(long int nodeTypes) {
	NodeList retVal;

	filter(getList(LIST_PARENTS), &retVal, nodeTypes);

	return retVal;
}

void GraphNode::removeEdge(GraphEdge* edge) {
	delete edge;
}

void GraphNode::removeChild(GraphNode* child) {
	// zjisteni hrany
	BaseList* list = getList(LIST_CHILDS);
	BaseList::iterator pos = list->begin(), end = list->end();

	GraphEdge* edge;

	while (pos != end) {
		edge = (GraphEdge*) (*pos);

		if (edge->to() == child) {
			// pred smazanim se musi iterovat, aby nedoslo ke zneplatneni iteratoru
			pos++;
			delete edge;
		} else {
			pos++;
		}
	}
}

void GraphNode::deleteEdges(BaseList* edges) {
	BaseList::iterator pos = edges->begin(), end = edges->end();
	GraphEdge* edge;

	while (pos != end) {
		edge = (GraphEdge*) *pos;
		pos++;

		delete edge;
	}
}

void GraphNode::removeParent(GraphNode* parent) {
	// zjisteni hrany
	BaseList* list = getList(LIST_CHILDS);
	BaseList::iterator pos = list->begin(), end = list->end();

	GraphEdge* edge;

	while (pos != end) {
		edge = (GraphEdge*) (*pos);

		if (edge->from() == parent) {
			// pred smazanim se musi iterovat, aby nedoslo ke zneplatneni iteratoru
			pos++;
			delete edge;
		} else {
			pos++;
		}
	}
}

} /* namespace SQLRepresenter */
