/*
 * Adapter.h
 *
 *  Created on: 6.2.2013
 *      Author: petr
 */

#ifndef ADAPTER_H_
#define ADAPTER_H_

#include "Database.h"
#include <vector>
#include "../ThirthParty/pugixml/pugixml.hpp"

class Database;


namespace SQLRepresenter {

class Adapter {



public:

	static const char *NODE_PARAM;
	static const char *ATTR_NAME;
	static const char *ATTR_VALUE;

	enum SORTED_LIST_INDEX {
		LIST_ADD = 0,
		LIST_DELETE = 1,
		LIST_CHANGE = 2,
		LIST_MODIFY = 3,
		LIST_LOGIC = 4,
		LIST_NO = 5
	};

	Adapter();

	virtual ~Adapter();

	virtual Database* readData(Database* database) = 0;

	virtual void setConfig(pugi::xml_node config) = 0;

	virtual void writeData(Database* database) = 0;

	virtual void writeDiff(Database* database) = 0;

	template <class T>
	std::vector<T> sortItems(T oldList);
};

template<class T>
inline std::vector<T> SQLRepresenter::Adapter::sortItems(T oldList) {
	typename T::iterator pos = oldList.begin(), end = oldList.end();
	std::vector<T> retVal(6);


	while (pos != end) {
		switch ((*pos)->compareResult) {
		case SQLBase::CR_ADDED:
			retVal[LIST_ADD].push_back(*pos);
			break;

		case SQLBase::CR_CHANGED:
			retVal[LIST_CHANGE].push_back(*pos);
			break;

		case SQLBase::CR_DELETED:
			retVal[LIST_DELETE].push_back(*pos);
			break;

		case SQLBase::CR_LOGIC:
			retVal[LIST_LOGIC].push_back(*pos);
			break;

		case SQLBase::CR_MODIFIED:
			retVal[LIST_MODIFY].push_back(*pos);
			break;

		case SQLBase::CR_NOCOMPARE:
			retVal[LIST_NO].push_back(*pos);
			break;
		}

		pos++;
	}

	return retVal;
}

}

#endif /* ADAPTER_H_ */
