/*
 * Comparator.cpp
 *
 *  Created on: 24.3.2013
 *      Author: petr
 */

#include "Comparator.h"

namespace SQLRepresenter {

Comparator::Comparator() {
	// TODO Auto-generated constructor stub

}

void Comparator::compareTables(COMPARE_HEADER) {

	// nacteni tabulek stare a nove verze
	NodeList oldTables = oldDatabase->getNodes(NODE_TABLE);
	NodeList newTables = newDatabase->getNodes(NODE_TABLE);
	NodeList missing, newbies, stays;

	// porovnani dat
	getDifference(oldTables, newTables, missing, newbies, stays);

	// zapis tabulek ktere chybi (byly smazany)
	NodeList::iterator pos = missing.begin(), end = missing.end();

	while (pos != end) {
		Table *table = (Table*) *pos;
		Table* toDelete = substract->createTable(table->name);
		toDelete->compareResult = SQLBase::CR_DELETED;

		// kontrola cizich klicu, ktere musi byt take smazane
		NodeList deleteFK = table->parentNodes(NODE_FOREIGN_KEY);

		// zapis klicu, ktere je potreba smazat
		NodeList::iterator fkPos = deleteFK.begin(), fkEnd = deleteFK.end();

		while (fkPos != fkEnd) {
			// puvodni FK
			ForeignKey* fk = (ForeignKey*) *fkPos;

			// kontrola jestli existuje referencni tabulka v rozdilu
			Table* tarTable = substract->table(fk->getTable()->name);

			if (tarTable == NULL) {
				// tabulka nebyla nalezena -> vytvori se logicka tabulka
				tarTable = substract->createTable(fk->getTable()->name);
				tarTable->compareResult = SQLBase::CR_LOGIC;
			}

			// zapis mazaneho ciziho klice
			ForeignKey *delFK = tarTable->createForeignKey(fk->name, ColumnList(), table, ColumnList());
			delFK->compareResult = SQLBase::CR_DELETED;

			fkPos++;
		}

		pos++;
	}

	// zpracovani novych tabulek
	pos = newbies.begin();
	end = newbies.end();

	while (pos != end) {
		Table* table = (Table*) *pos;

		// kontrola, jestli tabulka uz existuje jako LOGIC -> pripadne se vytvori nova
		Table* newTable = substract->table(table->name);

		if (newTable == NULL) {
			newTable = substract->createTable(table->name);
		}

		*newTable = *table;
		newTable->compareResult = SQLBase::CR_ADDED;

		// zapis sloupcu
		ColumnList columns = table->getColumns();
		ColumnList::iterator cPos = columns.begin(), cEnd = columns.end();

		while (cPos != cEnd) {
			Column *column = *cPos;
			Column *newColumn = newTable->createColumn(column->name);

			// nastaveni dat
			*newColumn = *column;
			newColumn->compareResult = SQLBase::CR_ADDED;

			cPos++;
		}

		// zapis checku
		CheckList checks = table->getChecks();
		CheckList::iterator chPos = checks.begin(), chEnd = checks.end();

		while (chPos != chEnd) {
			Check *check = *chPos;
			Check *newCheck = newTable->createCheck(check->name);

			*newCheck = *check;
			newCheck->compareResult = SQLBase::CR_ADDED;

			chPos++;
		}

		// zapis indexu
		IndexList indexes = table->getIndexes();
		IndexList::iterator inPos = indexes.begin(), inEnd = indexes.end();

		while (inPos != inEnd) {
			Index* index = (Index*) *inPos;

			// vytvoreni seznamu sloupcu
			ColumnList newColumns, columns = index->getColumns();
			cPos = columns.begin();
			cEnd = columns.end();

			while (cPos != cEnd) {
				Column *localColumn = getOrCreateColumn(newTable, (*cPos)->name);
				newColumns.push_back(localColumn);

				cPos++;
			}

			// vytvoreni indexu
			Index* newIndex = newTable->createIndex(index->name, newColumns);
			*newIndex = *index;
			newIndex->compareResult = SQLBase::CR_ADDED;

			inPos++;
		}

		// zapis cizich klicu
		ForeignList foreigns = table->getForeignKeys();
		ForeignList::iterator fkPos = foreigns.begin(), fkEnd = foreigns.end();

		while (fkPos != fkEnd) {
			ForeignKey *fk = (ForeignKey*) *fkPos;

			// nacteni referencni tabulky a pokud nenexistuje, vytvori se logicka
			Table *refTable = substract->table(fk->getReferencedTable()->name);

			if (refTable == NULL) {
				refTable = substract->createTable(fk->getReferencedTable()->name);
				refTable->compareResult = SQLBase::CR_LOGIC;
			}

			// nacteni seznamu mistnich a vzdalenych sloupcu
			ColumnList locals = copyColumns(newTable, fk->getColumns()), refs = copyColumns(refTable, fk->getReferencedColumns());

			// vytvoreni ciziho klice
			ForeignKey *newFk = newTable->createForeignKey(fk->name, locals, refTable, refs);
			*newFk = *fk;
			newFk->compareResult = SQLBase::CR_ADDED;

			fkPos++;
		}

		pos++;
	}

	// zpracovani stavajicich tabulek
	pos = stays.begin();
	end = stays.end();

	while (pos != end) {
		// nacteni a priprava ukazatelu
		Table *oldTable = (Table*) *pos;
		Table *newTable = newDatabase->table(oldTable->name), *subTable = NULL;

		// porovnani tabulek
		if (!(*oldTable == *newTable)) {
			// zakladni udaje nesouhlasi, vytvori se nova tabulka porovnani a priradi se nove hodnoty
			subTable = substract->table(newTable->name);

			if (subTable == NULL) {
				// tabulka jeste neexistuje, musi se vytvorit nova
				subTable = substract->createTable(newTable->name);
			}

			*subTable = *newTable;
		}

		// provedeni porovnani podcasti tabulek
		compareColumns(oldTable, newTable, &subTable, substract);
		compareIndexes(oldTable, newTable, &subTable, substract);
		compareForeigns(oldTable, newTable, &subTable, substract);
		compareChecks(oldTable, newTable, &subTable, substract);

		pos++;
	}
}

Comparator::~Comparator() {
	// TODO Auto-generated destructor stub
}

void Comparator::getDifference(NodeList oldNodes, NodeList newNodes,
		NodeList& missing, NodeList& newbies, NodeList& stays) {

	// zjisteni tabulek, ktere byly smazany
	NodeList::iterator pos = oldNodes.begin(), end = oldNodes.end();

	while (pos != end) {
		if (isInList(*pos, newNodes)) {
			stays.push_back(*pos);
		} else {
			missing.push_back(*pos);
		}

		pos++;
	}

	// zjisteni uzlu, ktere byly vytvoreny
	pos = newNodes.begin();
	end = newNodes.end();

	while (pos != end) {
		if (!isInList(*pos, oldNodes))
			newbies.push_back(*pos);

		pos++;
	}
}

bool Comparator::isInList(GraphNode* node, NodeList& otherNodes) {
	NodeList::iterator pos = otherNodes.begin(), end = otherNodes.end();
	bool retVal = false;
	SQLBase* item = (SQLBase*) node;

	while (pos != end && !retVal) {
		retVal = ((SQLBase*) *pos)->name == item->name;

		pos++;
	}

	return retVal;
}

void Comparator::compareTriggers(COMPARE_HEADER) {
	// nacteni seznamu triggeru
	PREPARE_COMPARATION(NODE_TRIGGER)

	// prochazeni triggeru ktere jsou smazany a jejich zapis
	NodeList::iterator pos = missing.begin(), end = missing.end();

	while (pos != end) {
		// pretypovani triggeru a kontrola existence tabulky
		Trigger* trigger = (Trigger*) *pos;
		createTriggerByOrig(substract, trigger, SQLBase::CR_DELETED);

		pos++;
	}

	// prochazeni triggeru, ktere jsou vytvoreny a jejich zapis
	pos = newbies.begin();
	end = newbies.end();

	while (pos != end) {
		Trigger* trigger = (Trigger*) *pos;
		Trigger* newTrigger = createTriggerByOrig(substract, trigger,
				SQLBase::CR_ADDED);

		// nastaveni hodnot
		(*newTrigger) = *trigger;
		newTrigger->compareResult = SQLBase::CR_ADDED;

		pos++;
	}

	// prochazeni existujicich triggeru a jejich provnani
	pos = stays.begin();
	end = stays.end();

	while (pos != end) {
		// nacteni triggeru
		Trigger *oldTrigger = (Trigger*) *pos, *newTrigger = newDatabase->table(
				oldTrigger->getTable()->name)->getTrigger(oldTrigger->name);

		// porovnani dat
		if (!(*oldTrigger == *newTrigger)) {
			// triggery se nerovnaji -> stary se smaze a novy vytvori
			createTriggerByOrig(substract, oldTrigger, SQLBase::CR_DELETED);

			Trigger *newCopy = createTriggerByOrig(substract, newTrigger,
					SQLBase::CR_ADDED);
			*newCopy = *newTrigger;
			newCopy->compareResult = SQLBase::CR_ADDED;
		}

		pos++;
	}
}

Trigger* Comparator::createTriggerByOrig(Database* substract, Trigger* original,
		SQLBase::COMPARE_RESULT cr) {

	Table* triggedTable = substract->table(original->getTable()->name);

	if (triggedTable == NULL) {
		// vytvori se logicka tabulka
		triggedTable = substract->createTable(original->getTable()->name);
		triggedTable->compareResult = SQLBase::CR_LOGIC;
	}

	// vytvoreni triggeru pro smazani
	Trigger *retVal = triggedTable->createTrigger(original->name);
	retVal->compareResult = cr;

	return retVal;
}

void Comparator::compareFunctions(COMPARE_HEADER) {
	// nacteni seznamu
	PREPARE_COMPARATION(NODE_FUNCTION)

	// zpracovani smazani
	NodeList::iterator pos = missing.begin(), end = missing.end();

	while (pos != end) {
		Function *fn = (Function*) *pos;
		Function *delFn = substract->createFunction(fn->name, fn->returnType);
		delFn->compareResult = SQLBase::CR_DELETED;

		pos++;
	}

	// zpracovani vytvoreni
	pos = newbies.begin();
	end = newbies.end();

	while (pos != end) {
		Function *fn = (Function*) *pos;
		Function *newFn = substract->createFunction(fn->name, fn->returnType);

		*newFn = *fn;
		newFn->compareResult = SQLBase::CR_ADDED;

		pos++;
	}

	// porovnání existujících
	pos = stays.begin();
	end = stays.end();

	while (pos != end) {
		Function *oldFn = (Function*) *pos;
		Function *newFn = newDatabase->function(oldFn->name);

		if (!(*oldFn == *newFn)) {
			// funkce se nerovnaji -> stara se smaze a nova vytvori
			Function *fn = substract->createFunction(oldFn->name,
					oldFn->returnType);
			fn->compareResult = SQLBase::CR_DELETED;

			fn = substract->createFunction(newFn->name, newFn->returnType);
			*fn = *newFn;
			fn->compareResult = SQLBase::CR_ADDED;
		}

		pos++;
	}
}

void Comparator::compareProcedures(COMPARE_HEADER) {
	// nacteni seznamu
	PREPARE_COMPARATION(NODE_PROCEDURE)

	// zpracovani smazani
	NodeList::iterator pos = missing.begin(), end = missing.end();

	while (pos != end) {
		Procedure *proc = (Procedure*) *pos;
		Procedure *delProc = substract->createProcedure(proc->name);
		delProc->compareResult = SQLBase::CR_DELETED;

		pos++;
	}

	// zpracovani vytvoreni
	pos = newbies.begin();
	end = newbies.end();

	while (pos != end) {
		Procedure *proc = (Procedure*) *pos;
		Procedure *newProc = substract->createProcedure(proc->name);

		*newProc = *proc;
		newProc->compareResult = SQLBase::CR_ADDED;

		pos++;
	}

	// porovnání existujících
	pos = stays.begin();
	end = stays.end();

	while (pos != end) {
		Procedure *oldProc = (Procedure*) *pos;
		Procedure *newProc = newDatabase->procedure(oldProc->name);

		if (!(*oldProc == *newProc)) {
			// funkce se nerovnaji -> stara se smaze a nova vytvori
			Procedure *proc = substract->createProcedure(oldProc->name);
			proc->compareResult = SQLBase::CR_DELETED;

			proc = substract->createProcedure(newProc->name);
			*proc = *newProc;
			proc->compareResult = SQLBase::CR_ADDED;
		}

		pos++;
	}
}

void Comparator::compareViews(COMPARE_HEADER) {
	PREPARE_COMPARATION(NODE_VIEW);

	// smazani neeixstujicich
	NodeList::iterator pos = missing.begin(), end = missing.end();

	while (pos != end) {
		View* view = (View*) *pos;
		View* delView = substract->createView(view->name);
		delView->compareResult = SQLBase::CR_DELETED;

		pos++;
	}

	// vytvoreni novych
	pos = newbies.begin();
	end = newbies.end();

	while (pos != end) {
		View* view = (View*) *pos;
		View* newView = substract->createView(view->name);

		*newView = *view;
		newView->compareResult = SQLBase::CR_ADDED;

		pos++;
	}

	// kontrola existujicich
	pos = stays.begin();
	end = stays.end();

	while (pos != end) {
		View *oldView = (View*) *pos;
		View *newView = newDatabase->view(oldView->name);

		if (!(*oldView == *newView)) {
			// definice pohledu nesouhlasi -> stary se smaze a vytvori se novy
			View *view = substract->createView(oldView->name);
			view->compareResult = SQLBase::CR_DELETED;

			view = substract->createView(newView->name);
			*view = *newView;
			view->compareResult = SQLBase::CR_ADDED;
		}

		pos++;
	}
}

Column* Comparator::getOrCreateColumn(Table* table, std::string name) {
	Column *retVal = table->getColumn(name);

	if (retVal == NULL) {
		retVal = table->createColumn(name);
		retVal->compareResult = SQLBase::CR_LOGIC;
	}

	return retVal;
}

ColumnList Comparator::copyColumns(Table* table, ColumnList otherColumns) {
	ColumnList retVal;
	ColumnList::iterator pos = otherColumns.begin(), end = otherColumns.end();

	while (pos != end) {
		retVal.push_back(getOrCreateColumn(table, (*pos)->name));

		pos++;
	}


	return retVal;
}

void Comparator::compareColumns(COMPARE_TABLE_HEADER) {
	PREPARE_TABLE_COMPARATOR(NODE_COLUMN)

	NodeList::iterator pos = missing.begin(), end = missing.end();

	while (pos != end) {
		Column *column = getOrCreateColumn(*subTable, ((Column*) (*pos))->name);
		column->compareResult = SQLBase::CR_DELETED;

		pos++;
	}

	pos = newbies.begin();
	end = newbies.end();

	while (pos != end) {
		Column *column = (Column*) *pos;
		Column *newColumn = getOrCreateColumn(*subTable, column->name);
		*newColumn = *column;
		newColumn->compareResult = SQLBase::CR_ADDED;

		pos++;
	}

	// zpracovani stavajicich sloupcu
	pos = stays.begin();
	end = stays.end();

	while (pos != end) {
		Column *oldColumn = (Column*) *pos;
		Column *newColumn = newTable->getColumn(oldColumn->name);

		if (!(*oldColumn == *newColumn)) {
			createTableIfNeeds(subTable, newTable, substract);

			// sloupce se nerovnaji -> musi se vytvorit delta sloupec
			Column *column = getOrCreateColumn(*subTable, newColumn->name);
			*column = *newColumn;
			column->compareResult = SQLBase::CR_CHANGED;
		}

		pos++;
	}
}

void Comparator::compareIndexes(COMPARE_TABLE_HEADER) {
	PREPARE_TABLE_COMPARATOR(NODE_INDEX)

	// zpracovani smazanycj
	NodeList::iterator pos = missing.begin(), end = missing.end();

	while (pos != end) {
		Index *index = (Index*) *pos;
		Index *delIndex = getOrCreateIndex(*subTable, index->name, ColumnList());

		delIndex->compareResult = SQLBase::CR_DELETED;

		pos++;
	}

	// zpracovani novych indexu
	pos = newbies.begin();
	end = newbies.end();

	while (pos != end) {
		Index *index = (Index*) *pos;
		Index *newIndex = getOrCreateIndex(*subTable, index->name, copyColumns(*subTable, index->getColumns()));
		*newIndex = *index;
		newIndex->compareResult = SQLBase::CR_ADDED;

		pos++;
	}

	// zpracovani zmenenych indexu
	pos = stays.begin();
	end = stays.end();

	while (pos != end) {
		// vyhodnoceni jestli indexy souhlasi
		Index *oldIndex = (Index*) *pos;
		Index *newIndex = newTable->getIndex(oldIndex->name);

		if (!(*oldIndex == *newIndex)) {
			// indexy nesohlasi -> stary se znici a novy se vytvori
			createTableIfNeeds(subTable, newTable, substract);
			checkTableChangeStatus(*subTable);

			Index *delIndex = getOrCreateIndex(*subTable, oldIndex->name, ColumnList()),
					*creIndex = (*subTable)->createIndex(newIndex->name, copyColumns(*subTable, newIndex->getColumns()));

			delIndex->compareResult = SQLBase::CR_DELETED;
			creIndex->compareResult = SQLBase::CR_ADDED;
		}

		pos++;
	}
}

void Comparator::compareChecks(COMPARE_TABLE_HEADER) {
	PREPARE_TABLE_COMPARATOR(NODE_CHECK)

	// zpracovani smazanych
	NodeList::iterator pos = missing.begin(), end = missing.end();

	while (pos != end) {
		Check* check = (Check*) *pos;
		Check* delCheck = getOrCreateCheck(*subTable, check->name, std::string());
		delCheck->compareResult = SQLBase::CR_DELETED;

		pos++;
	}

	// zpracovani novych
	pos = newbies.begin();
	end = newbies.end();

	while (pos != end) {
		Check *check = (Check*) *pos;
		Check *newCheck = getOrCreateCheck(*subTable, check->name, check->code);

		*newCheck = *check;
		newCheck->compareResult = SQLBase::CR_ADDED;

		pos++;
	}

	// zpracovani stavajicich
	pos = stays.begin();
	end = stays.end();

	while (pos != end) {
		Check *oldCheck = (Check*) *pos;
		Check *newCheck = newTable->getCheck(oldCheck->name);

		if (!(*oldCheck == *newCheck)) {
			// doslo ke zemne -> staryc check se smaze a vytvori se novy
			createTableIfNeeds(subTable, newTable, substract);
			checkTableChangeStatus(*subTable);

			Check* check = getOrCreateCheck(*subTable, oldCheck->name, newCheck->code);
			check->compareResult = SQLBase::CR_DELETED;

			check = (*subTable)->createCheck(newCheck->name);
			*check = *newCheck;
			check->compareResult = SQLBase::CR_ADDED;
		}

		pos++;
	}
}

void Comparator::compareForeigns(COMPARE_TABLE_HEADER) {
	PREPARE_TABLE_COMPARATOR(NODE_FOREIGN_KEY)

	// zpracovani smazanych klicu
	NodeList::iterator pos = missing.begin(), end = missing.end();

	while (pos != end) {
		ForeignKey *fk = (ForeignKey*) *pos;

		// nacteni referencni tabulky
		Table *refTable = substract->table(fk->getReferencedTable()->name);
		if (refTable == NULL) {
			refTable = substract->createTable(fk->getReferencedTable()->name);
			refTable->compareResult = SQLBase::CR_LOGIC;
		}

		ForeignKey *delFk = (*subTable)->getForeignKey(fk->name);
		if (delFk == NULL) delFk = (*subTable)->createForeignKey(
				fk->name,
				ColumnList(),
				refTable,
				ColumnList()
		);

		delFk->compareResult = SQLBase::CR_DELETED;

		pos++;
	}

	// zpracovani novych klicu
	pos = newbies.begin();
	end = newbies.end();

	while (pos != end) {
		ForeignKey *fk = (ForeignKey*) *pos;
		ForeignKey *newFk = (*subTable)->getForeignKey(fk->name);

		if (newFk == NULL) {
			Table *refTable = substract->table(fk->getReferencedTable()->name);

			if (refTable == NULL) {
				refTable = substract->createTable(fk->getReferencedTable()->name);
				refTable->compareResult = SQLBase::CR_LOGIC;
			}

			newFk = (*subTable)->createForeignKey(
					fk->name,
					copyColumns((*subTable), fk->getColumns()),
					refTable,
					copyColumns(refTable, fk->getReferencedColumns())
			);
		}

		newFk->compareResult = SQLBase::CR_ADDED;

		pos++;
	}

	// zpracovani stavajicich cizich klicu
	pos = stays.begin();
	end = stays.end();

	while (pos != end) {
		// nacteni dat
		ForeignKey *oldFk = (ForeignKey*) *pos;
		ForeignKey *newFk = newTable->getForeignKey(oldFk->name);

		if (!(*oldFk == *newFk)) {
			createTableIfNeeds(subTable, newTable, substract);

			// doslo ke zmene -> stary FK se znici a vytvori se novy
			ForeignKey *delFk = (*subTable)->getForeignKey(oldFk->name);
			Table *refTable = substract->table(oldFk->getReferencedTable()->name);

			if (refTable == NULL) {
				refTable = substract->createTable(oldFk->getReferencedTable()->name);
				refTable->compareResult = SQLBase::CR_LOGIC;
			}

			if (delFk == NULL) {
				delFk = (*subTable)->createForeignKey(
						oldFk->name,
						ColumnList(),
						refTable,
						ColumnList()
				);
			}

			delFk->compareResult = SQLBase::CR_DELETED;

			// vytvoreni noveho FK
			refTable = substract->table(newFk->getReferencedTable()->name);

			if (refTable == NULL) {
				refTable = substract->createTable(newFk->getReferencedTable()->name);
				refTable->compareResult = SQLBase::CR_LOGIC;
			}

			ForeignKey *createdFk = (*subTable)->createForeignKey(
					newFk->name,
					copyColumns(*subTable, newFk->getColumns()),
					refTable,
					copyColumns(refTable, newFk->getReferencedColumns())
			);

			createdFk->compareResult = SQLBase::CR_ADDED;
		}

		pos++;
	}
}

void Comparator::checkTableChangeStatus(Table* table) {
	if (table->compareResult != SQLBase::CR_MODIFIED)
		table->compareResult = SQLBase::CR_CHANGED;
}

void Comparator::createTableIfNeeds(Table** table, Table* origin, Database* substract) {
	// kotnrola existence
	if ((*table) == NULL) (*table) = substract->table(origin->name);

	if ((*table) == NULL) {
		(*table) = substract->createTable(origin->name);
		(*table)->compareResult = SQLBase::CR_CHANGED;

		(**table) = *origin;
	}
}

Index* Comparator::getOrCreateIndex(Table* table, std::string name, ColumnList columns) {
	Index* retVal = table->getIndex(name);

	if (retVal == NULL) {
		retVal = table->createIndex(name, columns);
		retVal->compareResult = SQLBase::CR_LOGIC;
	}

	return retVal;
}

Check* Comparator::getOrCreateCheck(Table* table, std::string name,
		std::string code) {
	Check* retVal = table->getCheck(name);

	if (retVal == NULL) {
		retVal = table->createCheck(name);
		retVal->code = code;
	}

	return retVal;
}

Database* Comparator::compare(Database* oldDatabase, Database* newDatabase) {
	Database* retVal = new Database();

	compareTriggers(oldDatabase, newDatabase, retVal);
	compareFunctions(oldDatabase, newDatabase, retVal);
	compareProcedures(oldDatabase, newDatabase, retVal);
	compareViews(oldDatabase, newDatabase, retVal);

	compareTables(oldDatabase, newDatabase, retVal);

	return retVal;
}

} /* namespace SQLRepresenter */
