/*
 * ConflictFinder.cpp
 *
 *  Created on: 7.5.2013
 *      Author: petr
 */

#include "ConflictFinder.h"

#define WRITECPARAMS conflictItem->compareResult = SQLBase::CR_CONFLICT;conflictItem->id = other->id;conflictItem->dbName = other->dbName;
#define WRITECONFLICTREC CONFLICT_SOLVE solve; solve.id = other->id; solve.useCurrent = true; solve.solved = false; conflicts[other->dbName].push_back(solve);

namespace SQLRepresenter {

ConflictFinder::ConflictFinder() {
	// TODO Auto-generated constructor stub

}

ConflictFinder::~ConflictFinder() {
	// TODO Auto-generated destructor stub
}

Database* ConflictFinder::build(const DeltaChain& chain) {
	DeltaChain::const_iterator pos = chain.begin(),
			end = chain.end();

	Database *retVal = new Database();

	// prochazeni retezem
	while (pos != end) {
		Database *delta = *pos;

		// tabulky se zpracuji zvlastni metodou
		buildTables(retVal, delta);

		// pohledy, procedury a funkce se zpracuji spolecnou metodou
		buildBase(delta->getNodes(NODE_VIEW), retVal);
		buildBase(delta->getNodes(NODE_FUNCTION), retVal);
		buildBase(delta->getNodes(NODE_PROCEDURE), retVal);

		pos++;
	}

	return retVal;
}

void ConflictFinder::buildTables BUILD_PARAMS {

	// nacteni vsech tabulek z delta balicku a jejich zpracovani
	TableList tables = delta->tables();
	TableList::iterator pos = tables.begin(), end = tables.end();

	while (pos != end) {
		Table *diffTable = *pos;

		// pokud je zmena v seznamu vyresenych konfliktu, pokracuje se na dalsi zmenu
		if (useCurrent(diffTable->dbName, diffTable->id)) {
			pos++;
			continue;
		}

		// vyhodnoceni zmeny
		if (diffTable->compareResult != SQLBase::CR_NOCOMPARE && diffTable->compareResult != SQLBase::CR_LOGIC) {
			// byla provedena zmena -> zmena se zapise
			Table *targetTable = (Table*) getOrCreate(target, diffTable->name, NODE_TABLE);
			SQLBase::COMPARE_RESULT oldResult = targetTable->compareResult;
			*targetTable = *diffTable;
			targetTable->compareResult = oldResult;
			targetTable->id = diffTable->id;
			targetTable->dbName = diffTable->dbName;

			// vyhodnoceni prepsani vysledku porovnani
			if (isHigherPriority(oldResult, diffTable->compareResult))
				targetTable->compareResult = diffTable->compareResult;

			buildTable(diffTable->childNodes(NODE_COLUMN), targetTable);
			buildIndexes(diffTable->childNodes(NODE_INDEX), targetTable);
			buildForeigns(diffTable->childNodes(NODE_FOREIGN_KEY), targetTable);
			buildTable(diffTable->childNodes(NODE_TRIGGER), targetTable);
			buildTable(diffTable->childNodes(NODE_CHECK), targetTable);
		}

		pos++;
	}

}

void ConflictFinder::buildBase(NodeList nodes, Database* target) {

	// iterace nad prvky v seznamu
	NodeList::iterator pos = nodes.begin(), end = nodes.end();

	while (pos != end) {
		// nacte se element v cilove tabulce
		SQLBase *fromDiff = (SQLBase*) *pos;

		// pokud je zmena v seznamu vyresenych konfliktu, pokracuje se na dalsi zmenu
		if (useCurrent(fromDiff->dbName, fromDiff->id)) {
			pos++;
			continue;
		}

		// kontrola stavu
		if (fromDiff->compareResult != SQLBase::CR_LOGIC && fromDiff->compareResult != SQLBase::CR_NOCOMPARE) {
			// stav je normalni zmenovy
			SQLBase *element = getOrCreate(target, fromDiff->name, fromDiff->nodeType);
			SQLBase::COMPARE_RESULT oldResult = element->compareResult;
			*element = *fromDiff;
			element->compareResult = oldResult;
			element->id = fromDiff->id;
			element->dbName = fromDiff->dbName;

			// vyhodnoti se, jestli ma novy stav vyssi prioritu
			if (isHigherPriority(element->compareResult, fromDiff->compareResult)) {
				element->compareResult = fromDiff->compareResult;
			}
		}
		pos++;
	}
}

SQLBase* ConflictFinder::getOrCreate(Database* target, string name, long int nodeType) {
	// nacteni uzlu a vyhledani
	NodeList list = target->getNodes(nodeType);
	SQLBase *element = getFromList(list, name);

	// pokud element neexistuje, pak se vytvori novy
	if (element == NULL) {
		// vyhodnoceni typu prvku
		switch (nodeType) {

		case NODE_TABLE:
			element = target->createTable(name);
			break;

		case NODE_VIEW:
			element = target->createView(name);
			break;

		case NODE_FUNCTION:
			element = target->createFunction(name, "");
			break;

		case NODE_PROCEDURE:
			element = target->createProcedure(name);
			break;

		}
	}

	return element;
}

bool ConflictFinder::isHigherPriority(SQLBase::COMPARE_RESULT oldStatus,
		SQLBase::COMPARE_RESULT newStatus) {

	// vyhodnoceni stavu
	if (newStatus == SQLBase::CR_LOGIC || newStatus == SQLBase::CR_NOCOMPARE) return false;

	// prostor pro pripadne pridani podminek

	return true;
}

SQLBase* ConflictFinder::getFromList(NodeList nodes, string name) {
	SQLBase *retVal = NULL;
	NodeList::iterator pos = nodes.begin(), end = nodes.end();

	while (pos != end && retVal == NULL) {
		SQLBase *item = (SQLBase*) *pos;
		if (item->name == name) retVal = item;

		pos++;
	}

	return retVal;
}

void ConflictFinder::buildTable(NodeList nodes, Table* target) {
	// iterace nad prvky v seznamu
	NodeList::iterator pos = nodes.begin(), end = nodes.end();

	while (pos != end) {
		// nacte se element v cilove tabulce
		SQLBase *fromDiff = (SQLBase*) *pos;

		// pokud je zmena v seznamu vyresenych konfliktu, pokracuje se na dalsi zmenu
		if (useCurrent(fromDiff->dbName, fromDiff->id)) {
			pos++;
			continue;
		}

		// kontrola stavu
		if (fromDiff->compareResult != SQLBase::CR_LOGIC && fromDiff->compareResult != SQLBase::CR_NOCOMPARE) {
			// stav je normalni zmenovy
			SQLBase *element = getOrCreate(target, fromDiff->name, fromDiff->nodeType);
			SQLBase::COMPARE_RESULT oldResult = element->compareResult;
			*element = *fromDiff;
			element->compareResult = oldResult;
			element->id = fromDiff->id;
			element->dbName = fromDiff->dbName;

			// vyhodnoti se, jestli ma novy stav vyssi prioritu
			if (isHigherPriority(element->compareResult, fromDiff->compareResult)) {
				element->compareResult = fromDiff->compareResult;
			}
		}

		pos++;
	}

}

SQLBase* ConflictFinder::getOrCreate(Table* table, string name,
		long int nodeType) {

	NodeList nodes = table->childNodes(nodeType);
	NodeList::iterator pos = nodes.begin(), end = nodes.end();

	SQLBase *retVal = getFromList(nodes, name);

	if (retVal == NULL) {
		// vytvoreni noveho prvku
		switch (nodeType) {

		case NODE_COLUMN:
			retVal = table->createColumn(name);
			break;

		case NODE_INDEX:
			retVal = table->createIndex(name, ColumnList());
			break;

		case NODE_FOREIGN_KEY:
			retVal = table->createForeignKey(name, ColumnList(), table, ColumnList());
			break;

		case NODE_TRIGGER:
			retVal = table->createTrigger(name);
			break;
		}
	}

	return retVal;
}

Database* ConflictFinder::intersectBaseItems(Database* target, NodeList first,
		NodeList second) {


	NodeList::iterator pos = first.begin(), end = first.end();

	while (pos != end) {
		SQLBase *item = (SQLBase*) *pos;
		SQLBase *other = getFromList(second, item->name);

		// zkontroluje se, jestli prvek existuje i ve druhem seznamu
		if (other != NULL) {
			// kontrola vyreseni konfliktu
			if (isConflictSolved(other->dbName, other->id)) {
				pos++;
				continue;
			}

			// prvek je v druhem seznamu - zkontroluje se operace
			if (isConflict(item, other)) {
				// hodnoty porovnani se nerovnaji - jedna se o konflikt
				SQLBase *conflictItem = getOrCreate(target, item->name, item->nodeType);
				WRITECPARAMS
				WRITECONFLICTREC
			} else {
				if (*other == *item) {
					// pokud jsou prvky shodne, nic se delat nebude
					WRITECONFLICTREC

					conflicts[other->dbName].back().solved = true;
				}
			}
		}
	}

	return target;
}

Database* ConflictFinder::intersectTables(Database* target, NodeList first,
		NodeList second) {

	NodeList::iterator pos = first.begin(), end = first.end();

	while (pos != end) {
		// kontrola jestli je tabulka pritomna i v druhem seznamu
		Table *current = (Table*) *pos;
		Table *other = (Table*) getFromList(second, current->name);

		// pocet nalezenych konfliktu a nerovnosti
		int confCount = 0;

		if (other) {
			if (isConflictSolved(other->dbName, other->id)) {
				pos++;
				continue;
			}

			// druha tabulka existuje - je zde moznost konfliktu
			if (isConflict(current, other)) {
				// doslo ke konfliktu
				SQLBase *conflictItem = getOrCreate(target, current->name, NODE_TABLE);
				WRITECPARAMS
				WRITECONFLICTREC
				confCount++;
			} else {
				if (!(*other == *current)) confCount++;
			}
		}

		// nyni se zkontroluji jednotlive prvky tabulky


		if (other == NULL) {
			confCount += intersectTableItems(target, current->name, current->childNodes(NODE_COLUMN), NodeList());
			confCount += intersectTableItems(target, current->name, current->childNodes(NODE_TRIGGER), NodeList());
			confCount += intersectTableItems(target, current->name, current->childNodes(NODE_INDEX), NodeList());
			confCount += intersectTableItems(target, current->name, current->childNodes(NODE_FOREIGN_KEY), NodeList());
		} else {
			confCount += intersectTableItems(target, current->name, current->childNodes(NODE_COLUMN), other->childNodes(NODE_COLUMN));
			confCount += intersectTableItems(target, current->name, current->childNodes(NODE_TRIGGER), other->childNodes(NODE_TRIGGER));
			confCount += intersectTableItems(target, current->name, current->childNodes(NODE_INDEX), other->childNodes(NODE_INDEX));
			confCount += intersectTableItems(target, current->name, current->childNodes(NODE_FOREIGN_KEY), other->childNodes(NODE_FOREIGN_KEY));
		}

		// pokud zadne konflikty nebyly nalezeny a vsechny prvky jsou stejne a aktualni tabulka odpovida druhe, pak se zapise do konfliktu
		if (confCount == 0 && other != NULL) {
			WRITECONFLICTREC
			conflicts[other->dbName].back().solved = true;
		}

		pos++;
	}

	return target;
}

bool ConflictFinder::isConflict(SQLBase* firstElement,
		SQLBase* secondElement) {

	bool retVal = false;
	SQLBase::COMPARE_RESULT first = firstElement->compareResult,
			second = secondElement->compareResult;

	// vyhodnoceni jestli se jedna o konflikt
	retVal = retVal || CONFLICT_COMP(CR_DELETED, CR_ADDED)
			|| CONFLICT_COMP(CR_MODIFIED, CR_DELETED)
			|| CONFLICT_COMP(CR_CHANGED, CR_DELETED)
			|| CONFLICT_COMP(CR_MODIFIED, CR_MODIFIED);

	// kontrola stejnosti
	retVal = retVal || !(*firstElement == *secondElement);

	return retVal;
}

int ConflictFinder::intersectTableItems(Database* target, string tableName, NodeList first,
		NodeList second) {

	NodeList::iterator pos = first.begin(), end = first.end();
	Table* table = NULL;

	// navratova hodnota je pocet prvku, ktere jsou v konfliktu nebo nejsou stejne
	int retVal = 0;

	while (pos != end) {
		SQLBase *current = (SQLBase*) *pos;
		SQLBase *other = getFromList(second, current->name);

		// zkontroluje se, jestli prvek existuje i ve druhem seznamu
		if (other != NULL) {
			if (isConflictSolved(other->dbName, other->id)) {
				pos++;
				continue;
			}

			// prvek je v druhem seznamu - zkontroluje se operace
			if (isConflict(current, other)) {
				// nacteni tabulky pokud je potreba
				if (table == NULL)
					table = (Table*) getOrCreate(target, tableName, NODE_TABLE);

				// hodnoty porovnani se nerovnaji - jedna se o konflikt
				SQLBase* conflictItem = getOrCreate(table, current->name, current->nodeType);
				WRITECPARAMS
				WRITECONFLICTREC

				retVal++;
			} else {
				if (*other == *current) {
					WRITECONFLICTREC
					conflicts[other->dbName].back().solved = true;
				} else {
					retVal++;
				}
			}
		}

		pos++;
	}

	return retVal;
}

Database* ConflictFinder::findConflicts(const DeltaChain &current, const  DeltaChain &other) {
	Database *retVal = new Database();

	// sestaveni seznamu zmen ze serii balicku
	Database *firstBuild = build(current),
			*secondBuild = build(other);

	// vytvoreni pruniku databazi
	intersectBaseItems(retVal, firstBuild->getNodes(NODE_FUNCTION), secondBuild->getNodes(NODE_FUNCTION));
	intersectBaseItems(retVal, firstBuild->getNodes(NODE_PROCEDURE), secondBuild->getNodes(NODE_PROCEDURE));
	intersectBaseItems(retVal, firstBuild->getNodes(NODE_VIEW), secondBuild->getNodes(NODE_VIEW));

	intersectTables(retVal, firstBuild->getNodes(NODE_TABLE), secondBuild->getNodes(NODE_TABLE));

	return retVal;
}

bool ConflictFinder::isConflictSolved(string dbName, int itemId) {
	return findSolve(dbName, itemId).solved;
}

void ConflictFinder::buildIndexes(NodeList nodes, Table* target) {
	NodeList::iterator pos = nodes.begin(), end = nodes.end();

	while (pos != end) {
		// zpracovani indexu
		Index* diffIndex = (Index*) *pos;

		if (useCurrent(diffIndex->dbName, diffIndex->id)) {
			// konflikt je vyresen a preskoci se element
			pos++;
			continue;
		}

		Index* newIndex = (Index*) getOrCreate(target, diffIndex->name, NODE_INDEX);

		// indexu se vyprazdni hrany a zapisi se nove prvky
		removeEdges(newIndex->childEdges(MATE_INDEXED));

		// zapsani obecnych hodnot
		*newIndex = *diffIndex;
		newIndex->dbName = diffIndex->dbName;
		newIndex->id = diffIndex->id;
		newIndex->compareResult = diffIndex->compareResult;

		// zapis sloupcu
		ColumnList columns = diffIndex->getColumns();
		ColumnList::iterator cPos = columns.begin(), cEnd = columns.end();

		while (cPos != cEnd) {
			Column *column = target->getColumn((*cPos)->name);

			if (column == NULL) {
				column = target->createColumn((*cPos)->name);
				column->compareResult = SQLBase::CR_LOGIC;
			}

			// sloupec existuje, zapise se do indexu
			newIndex->addChild(column, MATE_INDEXED);

			cPos++;
		}

		pos++;
	}
}

void ConflictFinder::buildForeigns(NodeList nodes, Table* target) {
	NodeList::iterator pos = nodes.begin(),
			end = nodes.end();

	while (pos != end) {
		ForeignKey *diffFk = (ForeignKey*) *pos;

		if (useCurrent(diffFk->dbName, diffFk->id)) {
			pos++;
			continue;
		}

		ForeignKey *newFk = (ForeignKey*) getOrCreate(target, diffFk->name, NODE_FOREIGN_KEY);

		// odstranneni starych hran
		removeEdges(newFk->childEdges(MATE_FOREIGN_REF));
		removeEdges(newFk->childEdges(MATE_FOREIGN_TARGET));

		// vytvoreni referencni tabulky
		Table *refTable = (Table*) getOrCreate((Database*) target->graph(), diffFk->getReferencedTable()->name, NODE_TABLE);
		newFk->addChild(refTable, MATE_FOREIGN_REF);

		// zapis mistnich sloupcu
		copyColumns(diffFk, newFk, target, MATE_FOREIGN_TARGET);
		copyColumns(diffFk, newFk, refTable, MATE_FOREIGN_REF);
	}
}

void ConflictFinder::removeEdges(EdgeList edges) {
	EdgeList::iterator pos = edges.begin(),
			end = edges.end();

	while (pos != end) {
		delete *pos;
	}
}

void ConflictFinder::copyColumns(ForeignKey* oldKey, ForeignKey* newKey,
		Table* columnSource, MATE_TYPES mate) {

	ColumnList columns = (mate == MATE_FOREIGN_TARGET) ? oldKey->getColumns() : oldKey->getReferencedColumns();
	ColumnList::iterator pos = columns.begin(), end = columns.end();

	while (pos != end) {
		SQLBase *item = columnSource->getColumn((*pos)->name);

		if (!item) {
			item = columnSource->createColumn((*pos)->name);
			item->compareResult = SQLBase::CR_LOGIC;
		}

		newKey->addChild(item, mate);

		pos++;
	}
}

CONFLICT_SOLVE ConflictFinder::findSolve(string dbName, int id) {
	Conflicts::iterator item = conflicts.find(dbName);

	if (item != conflicts.end()) {
		// prvek databaze existuje
		ConflictList::iterator pos = item->second.begin();

		while (pos != item->second.end()) {
			if (pos->id == id) return *pos;

			pos++;
		}
	}

	CONFLICT_SOLVE retVal;
	retVal.useCurrent = false;
	retVal.solved = false;
	retVal.id = id;

	return retVal;
}

bool ConflictFinder::useCurrent(string dbName, int id) {
	return findSolve(dbName, id).useCurrent;
}

} /* namespace SQLRepresenter */

