/*
 * ConflictFinder.h
 *
 *  Created on: 7.5.2013
 *      Author: petr
 */

#ifndef CONFLICTFINDER_H_
#define CONFLICTFINDER_H_

#include <vector>
#include <string>
#include "Database.h"

#define CONFLICT_COMP(FIRST, SECOND) (first == SQLBase::FIRST && second == SQLBase::SECOND) || (first == SQLBase::SECOND && second == SQLBase::FIRST)

using namespace std;

namespace SQLRepresenter {

struct CONFLICT_SOLVE {
	/**
	 * identifikator konfliktni zmeny v druhe vetvi
	 */
	int id;

	/**
	 * bude pouzita nase vetev
	 */
	bool useCurrent;

	/**
	 * prepinac, jeslti byl konflikt vyresen
	 */
	bool solved;
};

typedef vector<Database*> DeltaChain;
typedef vector<CONFLICT_SOLVE> ConflictList;
typedef map<string, ConflictList> Conflicts;

#define BUILD_PARAMS (Database *target, Database *delta)

class ConflictFinder {

	void buildTables BUILD_PARAMS;

	void buildBase(NodeList nodes, Database* target);

	void buildTable(NodeList nodes, Table* target);

	SQLBase *getOrCreate(Database* target, string name, long int nodeType);

	SQLBase *getOrCreate(Table *table, string name, long int nodeType);

	SQLBase *getFromList(NodeList nodes, string name);

	bool isHigherPriority(SQLBase::COMPARE_RESULT oldStatus, SQLBase::COMPARE_RESULT newStatus);

	bool isConflict(SQLBase* firstElement, SQLBase* secondElement);

	Database *intersectBaseItems(Database* target, NodeList first, NodeList second);

	Database *intersectTables(Database* target, NodeList first, NodeList second);

	int intersectTableItems(Database* target, string tableName, NodeList first, NodeList second);

	bool isConflictSolved(string dbName, int itemId);

	void buildIndexes(NodeList nodes, Table* target);

	void buildForeigns(NodeList nodes, Table* target);

	void removeEdges(EdgeList edges);

	void copyColumns(ForeignKey *oldKey, ForeignKey *newKey, Table* columnSource, MATE_TYPES mate);

	CONFLICT_SOLVE findSolve(string dbName, int id);

	bool useCurrent(string dbName, int id);

public:

	Conflicts conflicts;

	ConflictFinder();
	virtual ~ConflictFinder();

	Database *build(const DeltaChain &chain);

	Database* findConflicts(const DeltaChain &current, const DeltaChain &other);
};

} /* namespace SQLRepresenter */
#endif /* CONFLICTFINDER_H_ */
