/*
 * Database.cpp
 *
 *  Created on: 26.1.2013
 *      Author: petr
 */

#include "Database.h"

namespace SQLRepresenter {

Database::Database() {
	nextId = 0;
}

Database::~Database() {

}

Table* Database::createTable(string name) {
	return new Table(this, name);
}

View* Database::createView(string name) {
	return new View(this, name);
}

ForeignList Database::foreigns() {
	NodeList nodes = this->getNodes(NODE_FOREIGN_KEY);
	ForeignList retVal;

	NodeList::iterator pos = nodes.begin(), end = nodes.end();

	while (pos != end) {
		retVal.push_back((ForeignKey*) *pos);

		pos++;
	}

	return retVal;
}

TableList Database::tables() {
	NodeList nodes = this->getNodes(NODE_TABLE);
	TableList retVal;

	NodeList::iterator pos = nodes.begin(), end = nodes.end();

	while (pos != end) {
		retVal.push_back((Table*) *pos);

		pos++;
	}

	return retVal;
}

ForeignKey* Database::foreign(string name) {
	return (ForeignKey*) findByName(getNodes(NODE_FOREIGN_KEY), name);
}

Table* Database::table(string name) {
	return (Table*) findByName(getNodes(NODE_TABLE), name);
}

View* Database::view(string name) {
	return (View*) findByName(getNodes(NODE_VIEW), name);
}

SQLBase* Database::findByName(NodeList nodes, string name) {
	NodeList::iterator pos = nodes.begin(), end = nodes.end();

	while (pos != end) {
		if (((SQLBase*) *pos)->name == name) return (SQLBase*) *pos;

		pos++;
	}

	return NULL;
}

Function* Database::createFunction(string name, string returnType) {
	Function* retVal;
	retVal = new Function(this, name);

	retVal->returnType = returnType;

	return retVal;
}

Procedure* Database::createProcedure(string name) {
	return new Procedure(this, name);
}

Function* Database::function(string name) {
	return (Function*) findByName(getNodes(NODE_FUNCTION), name);
}

FunctionList Database::functions() {
	NodeList nodes = getNodes(NODE_FUNCTION);
	FunctionList retVal;

	NodeList::iterator pos = nodes.begin(), end = nodes.end();

	while (pos != end) {
		retVal.push_back((Function*) *pos);

		pos++;
	}

	return retVal;
}

Procedure* Database::procedure(string name) {
	return (Procedure*) findByName(getNodes(NODE_PROCEDURE), name);
}

ProcedureList Database::procedures() {
	NodeList nodes = getNodes(NODE_PROCEDURE);
	ProcedureList retVal;
	NodeList::iterator pos = nodes.begin(), end = nodes.end();

	while (pos != end) {
		retVal.push_back((Procedure*) *pos);

		pos++;
	}

	return retVal;
}

ViewList Database::views() {
	NodeList nodes = this->getNodes(NODE_VIEW);
	ViewList retVal;

	NodeList::iterator pos = nodes.begin(), end = nodes.end();

	while (pos != end) {
		retVal.push_back((View*) *pos);

		pos++;
	}

	return retVal;
}

SQLBase* Database::getById(int id) {
	NodeList nodes = getNodes(-1);
	NodeList::iterator pos = nodes.begin(), end = nodes.end();

	while (pos != end) {
		SQLBase *base = (SQLBase*) *pos;

		if (base->id == id) return base;

		pos++;
	}

	return NULL;
}

}
