/*
 * include.h
 *
 *  Created on: 26.1.2013
 *      Author: petr
 */

#ifndef DATABASE_REP_H_
#define DATABASE_REP_H_

#include <string>
#include <map>

#include "../ThirthParty/pugixml/pugixml.hpp"
#include "SQLBase.h"


using namespace std;
using namespace Graph;

namespace SQLRepresenter {


#ifndef PARAM_MAP
#define PARAM_MAP
struct PARAMETER {
	string value;
	bool switcher;
};

typedef map<string, PARAMETER> ParamMap;

#endif

enum MATE_TYPES {
	MATE_MAJOR_PARENT = 1,
	MATE_INDEXED = 2,
	MATE_FOREIGN_REF = 4,
	MATE_FOREIGN_TARGET = 8
};

enum NODE_TYPES {
	NODE_TABLE = 1,
	NODE_COLUMN = 2,
	NODE_VIEW = 4,
	NODE_FOREIGN_KEY = 8,
	NODE_INDEX = 16,
	NODE_CHECK = 32,
	NODE_PROCEDURE = 64,
	NODE_FUNCTION = 128,
	NODE_TRIGGER = 256,
	NODE_ALL = 511
};

enum INDEX_TYPES {
	INDEX_TYPE_KEY = 1,
	INDEX_TYPE_UNIQUE = 2,
	INDEX_TYPE_PRIMARY = 4
};

enum PROC_PARAM_TYPE {
	IN = 1,
	OUT = 2,
	INOUT = 4
};

struct PROC_PARAM {
	string name;
	string dataType;
	string defVal;
	PROC_PARAM_TYPE paramType;
	SQLBase::COMPARE_RESULT compareResult;
};

class Table;
class View;
class ForeignKey;
class Column;
class Index;
class Trigger;
class Check;

#ifndef PROCEDURE_CLASS
#define PROCEDURE_CLASS
class Procedure;
#endif

#ifndef FUNCTION_CLASS
#define FUNCTION_CLASS
class Function;
#endif

typedef map<string, PARAMETER> ParamMap;
typedef list<ForeignKey*> ForeignList;
typedef list<Table*> TableList;
typedef list<View*> ViewList;
typedef list<Column*> ColumnList;
typedef list<Index*> IndexList;
typedef list<Procedure*> ProcedureList;
typedef list<Function*> FunctionList;
typedef list<PROC_PARAM> ProcParamList;
typedef list<Trigger*> TriggerList;
typedef list<Check*> CheckList;

//////////////////////////////////

class Database : public Graph::Graph {

	SQLBase* findByName(NodeList nodes, string name);

public:

	/**
	 * id dalsiho prvku databaze
	 */
	int nextId;

	Database();
	virtual ~Database();

	/**
	 * jmeno databaze, ktere slouzi pro identifikaci
	 * neni povinne
	 * je zde z duvodu identifikace kongliktu
	 */
	string name;

	/**
	 * vytvori novou funkci
	 *
	 * @param string name jmeno funkce
	 * @param string returnType navratovy typ
	 * @return Function*
	 */
	Function* createFunction(string name, string returnType);

	/**
	 * vytvori novou proceduru
	 *
	 * @param string name jemeno procedury
	 * @return Procedure*
	 */
	Procedure* createProcedure(string name);

	/**
	 * vytvori novou tabulku
	 *
	 * @param string name jmeno nove tybulky
	 * @return Table*
	 */
	Table* createTable(string name);

	/**
	 * vytvori novy pohled
	 *
	 * @param string name jmeno pohledu
	 * @return View*
	 */
	View* createView(string name);

	/**
	 * vraci cizi klic dle jmena
	 * pokud klic neexistuje vraci NULL
	 *
	 * @param string name jmeno ciziho klice
	 * @return ForeignKey*
	 */
	ForeignKey* foreign(string name);

	/**
	 * vraci seznam cizi klicu
	 *
	 * @return ForeignList
	 */
	ForeignList foreigns();

	/**
	 * vraci funkci podle jmena
	 * pokud funkce neexistuje, vraci NULL
	 *
	 * @param string name jmeno funkce
	 * @return Function*
	 */
	Function* function(string name);

	/**
	 * vraci seznam funkce
	 *
	 * @return FunctionList
	 */
	FunctionList functions();

	/**
	 * vraci prvek dle id
	 */
	SQLBase *getById(int id);

	/**
	 * vraci proceduru podle jmena
	 * pokud procedura neexistuje, vraci NULL
	 *
	 * @param string name jmeno procedury
	 * @return Procedure*
	 */
	Procedure* procedure(string name);

	/**
	 * vraci seznam procedur
	 *
	 * @return ProcedureList
	 */
	ProcedureList procedures();

	/**
	 * vraci tabulku podle jmena
	 * pokud tabulka neexistuje, vraci NULL
	 *
	 * @param string name
	 * @return Table*
	 */
	Table* table(string name);

	/**
	 * vraci seznam tabulek
	 *
	 * @return TableList
	 */
	TableList tables();

	/**
	 * vraci pohled podle jmena
	 * pokud pohled neexistuje, vraci NULL
	 *
	 * @return View*
	 */
	View* view(string name);

	/**
	 * vraci seznam pohledu
	 *
	 * @return ViewList
	 */
	ViewList views();
};

}

#include "Check.h"
#include "Column.h"
#include "ForeignKey.h"
#include "Index.h"
#include "LoadedData.h"
#include "Table.h"
#include "Trigger.h"
#include "View.h"
#include "Procedure.h"

#endif /* INCLUDE_REP_H_ */
