/*
 * ForeignKey.cpp
 *
 *  Created on: 6.2.2013
 *      Author: petr
 */

#include "ForeignKey.h"

namespace SQLRepresenter {


SQLRepresenter::ForeignKey::ForeignKey(Database* database, string name) : SQLBase(database) {
	this->name = name;
	nodeType = NODE_FOREIGN_KEY;
}

SQLRepresenter::ForeignKey::~ForeignKey() {
}

ColumnList SQLRepresenter::ForeignKey::getColumns() {
	return edgesToColumns(childEdges(MATE_FOREIGN_TARGET));
}

ColumnList SQLRepresenter::ForeignKey::getReferencedColumns() {
	return edgesToColumns(childEdges(MATE_FOREIGN_REF));
}

Table* SQLRepresenter::ForeignKey::getReferencedTable() {
	return (Table*) childNodes(NODE_TABLE).front();
}

Table* SQLRepresenter::ForeignKey::getTable() {
	return (Table*) parentNodes(NODE_TABLE).front();
}

void ForeignKey::setColumns(ColumnList columns) {
	removeEdges(MATE_FOREIGN_TARGET, NODE_COLUMN);

	createEdges(columns, MATE_FOREIGN_TARGET);
}

void ForeignKey::setReferencedColumns(ColumnList columns) {
	removeEdges(MATE_FOREIGN_REF, NODE_COLUMN);

	createEdges(columns, MATE_FOREIGN_REF);
}

ColumnList ForeignKey::edgesToColumns(Graph::EdgeList edges) {
	ColumnList retVal;
	Graph::EdgeList::iterator pos = edges.begin(), end = edges.end();
	GraphNode *node;

	while (pos != end) {
		node = (*pos)->to();

		if (node->nodeType == NODE_COLUMN) retVal.push_back((Column*) node);

		pos++;
	}

	return retVal;
}

void ForeignKey::createEdges(ColumnList columns, MATE_TYPES mate) {
	ColumnList::iterator pos = columns.begin(), end = columns.end();

	while (pos != end) {
		addChild(*pos, mate);

		pos++;
	}
}

void ForeignKey::removeEdges(MATE_TYPES edgeTypes, NODE_TYPES nodeTypes) {
	Graph::EdgeList edges = childEdges(edgeTypes);
	Graph::EdgeList::iterator pos = edges.begin(), end = edges.end();

	while (pos != end) {
		if ((*pos)->to()->nodeType == nodeTypes) removeEdge(*pos);

		pos++;
	}
}

void ForeignKey::setReferencedTable(Table* table) {
	removeChild(getReferencedTable());

	addChild(table, MATE_FOREIGN_REF);
}

ForeignKey& ForeignKey::operator =(const ForeignKey& o) {
	onDelete = o.onDelete;
	onUpdate = o.onUpdate;

	return *this;
}

bool ForeignKey::operator ==(const ForeignKey& o) {
	// porovnani tabulek
	ForeignKey &nko = (ForeignKey &) o;

	if (!(*(getTable()) == *(nko.getTable()) && *(getReferencedTable()) == *(nko.getReferencedTable()))) return false;

	// porovnani sloupcu
	ColumnList thisLocal = getColumns(), thisRefs = getReferencedColumns(),
			oLocals = nko.getColumns(), oRefs = nko.getReferencedColumns();

	if (thisLocal.size() != oLocals.size() || thisRefs.size() != oRefs.size()) return false;

	// porovnani jednotlivych sloupcu
	ColumnList::iterator pos = thisLocal.begin(), oPos = oLocals.begin(), end = thisLocal.end();

	while (pos != end) {
		if (!(**pos == **oPos)) return false;

		pos++;
		oPos++;
	}

	pos = thisRefs.begin();
	oPos = oRefs.begin();
	end = thisRefs.end();

	while (pos != end) {
		if (!(**pos == **oPos)) return false;

		pos++;
		oPos++;
	}

	return onUpdate == o.onUpdate && onDelete == o.onDelete && *((SQLBase*) this) == (const SQLBase&) o;
}

ForeignKey& ForeignKey::operator =(ForeignKey& o) {
	onDelete = o.onDelete;
	onUpdate = o.onUpdate;

	*((SQLBase*) this) = o;

	return *this;
}

}
