/*
 * LocalAdapter.cpp
 *
 *  Created on: 18.4.2013
 *      Author: petr
 */

#include "LocalAdapter.h"

#ifndef ISCONFLICT
#define ISCONFLICT(ITERATOR) if (isConflict(*ITERATOR)) continue;
#endif

namespace SQLRepresenter {

LocalAdapter::LocalAdapter() {
	target = NULL;
}

LocalAdapter::~LocalAdapter() {
	// TODO Auto-generated destructor stub
}

Database* LocalAdapter::readData(Database* database) {
	return NULL;
}

void LocalAdapter::setConfig(pugi::xml_node config) {
}

void LocalAdapter::writeData(Database* database) {
}

void LocalAdapter::diffRoutines(Database* database) {
	// nacteni a smazani procedur a funkci
	vector<ProcedureList> procedures = sortItems<ProcedureList>(database->procedures());
	vector<FunctionList> functions = sortItems<FunctionList>(database->functions());

	ProcedureList::iterator pPos, pEnd;
	FunctionList::iterator fPos, fEnd;
	Procedure *proc;
	Function *fn;

	pPos = procedures[LIST_DELETE].begin(); pEnd = procedures[LIST_DELETE].end();
	fPos = functions[LIST_DELETE].begin(); fEnd = functions[LIST_DELETE].end();

	while (pPos != pEnd) {
		ISCONFLICT(pPos)
		proc = target->procedure((*pPos)->name);
		if (proc != NULL) delete proc;

		pPos++;
	}

	while (fPos != fEnd) {
		ISCONFLICT(fPos)
		fn = target->function((*fPos)->name);
		if (fn != NULL) delete fn;

		pPos++;
	}

	// zapis novych procedur a funkci
	pPos = procedures[LIST_ADD].begin(); pEnd = procedures[LIST_ADD].end();
	fPos = functions[LIST_ADD].begin(); fEnd = functions[LIST_ADD].end();

	Procedure *dProc;
	Function *dFn;

	while (pPos != pEnd) {
		ISCONFLICT(pPos)
		// kotrnola jestli procedure uz neexistuje
		proc = target->procedure((*pPos)->name);
		dProc = *pPos;

		if (proc == NULL) proc = target->createProcedure((*pPos)->name);
		*proc = *dProc;
		proc->compareResult = SQLBase::CR_NOCOMPARE;

		pPos++;
	}

	while (fPos != fEnd) {
		ISCONFLICT(fPos);
		dFn = *fPos;
		fn = target->function(dFn->name);

		if (fn == NULL) fn = target->createFunction(dFn->name, dFn->returnType);

		*fn = *dFn;
		fn->compareResult = SQLBase::CR_NOCOMPARE;

		fPos++;
	}
}

void LocalAdapter::diffTables(Database* database) {
	// smazani tabulek, ktere se maji smazat
	vector<TableList> sorted = sortItems<TableList>(database->tables());

	TableList::iterator pos, end;
	pos = sorted[LIST_DELETE].begin(); end = sorted[LIST_DELETE].end();
	Table *table;

	while (pos != end) {
		ISCONFLICT(pos)
		table = target->table((*pos)->name);
		if (table != NULL) delete table;

		pos++;
	}

	// vytvoreni novych tabulek (prozatim bez cizich klicu)
	pos = sorted[LIST_ADD].begin(); end = sorted[LIST_ADD].end();

	while (pos != end) {
		ISCONFLICT(pos)
		// pokud tabulka existuje, tak se smaze
		createTable(*pos);

		pos++;
	}

	// modifikace existujicich tabulek
	pos = sorted[LIST_CHANGE].begin(); end = sorted[LIST_CHANGE].end();

	while (pos != end) {
		ISCONFLICT(pos)
		diffTable(*pos);

		pos++;
	}

	// realizace cizich klicu u zmenenych tabulek
	pos = sorted[LIST_CHANGE].begin();

	while (pos != end) {
		ISCONFLICT(pos)
		diffForeigns(*pos);

		pos++;
	}

	// realizace cizich klicu u novych tabulek
	pos = sorted[LIST_ADD].begin(); end = sorted[LIST_ADD].end();
	while (pos != end) {
		ISCONFLICT(pos)
		diffForeigns(*pos);

		pos++;
	}
}

void LocalAdapter::diffViews(Database* database) {
	vector<ViewList> sorted = sortItems<ViewList>(database->views());

	// odstraneni smazanych pohledu
	ViewList::iterator pos, end;
	pos = sorted[LIST_DELETE].begin(); end = sorted[LIST_DELETE].end();
	View* view;

	while (pos != end) {
		ISCONFLICT(pos)
		view = target->view((*pos)->name);

		if (view != NULL) {
			delete view;
		}

		pos++;
	}

	// vytvoreni novych pohledu
	pos = sorted[LIST_ADD].begin(); end = sorted[LIST_ADD].end();
	View* dView;

	while (pos != end) {
		ISCONFLICT(pos)
		dView = *pos;

		// pohled by existovat nemel ale sychr je sychr
		view = target->view(dView->name);
		if (view == NULL) view = target->createView(dView->name);

		// zapis hodnot
		*view = *dView;
		view->compareResult = SQLBase::CR_NOCOMPARE;

		pos++;
	}
}

void LocalAdapter::copyColumns(Table* table, Table* dTable) {
	ColumnList columns = dTable->getColumns();
	ColumnList::iterator pos, end;

	pos = columns.begin(); end = columns.end();

	while (pos != end) {
		Column *column = table->createColumn((*pos)->name);
		*column = **pos;
		column->compareResult = SQLBase::CR_NOCOMPARE;

		pos++;
	}
}

void LocalAdapter::copyKeys(Table* table, Table* dTable) {
	IndexList indexes = dTable->getIndexes();
	IndexList::iterator pos = indexes.begin(), end = indexes.end();

	while (pos != end) {
		// kopirovani sloupcu
		ColumnList columns, dColumns((*pos)->getColumns());
		ColumnList::iterator cPos = dColumns.begin(), cEnd = dColumns.end();

		while (cPos != cEnd) {
			columns.push_back(table->getColumn((*cPos)->name));
			cPos++;
		}

		Index *index = table->createIndex((*pos)->name, columns);
		*index = **pos;
		index->compareResult = SQLBase::CR_NOCOMPARE;

		pos++;
	}
}

void LocalAdapter::diffTable(Table* dTable) {
	Table *table = target->table(dTable->name);

	if (table == NULL) return;

	// zpracovani sloupcu
	ColumnList dColumns = dTable->getColumns();
	ColumnList::iterator cPos = dColumns.begin(), cEnd = dColumns.end();

	while (cPos != cEnd) {
		ISCONFLICT(cPos)
		Column *dColumn = *cPos;
		Column *column = table->getColumn(dColumn->name);

		switch (dColumn->compareResult) {
		case SQLBase::CR_DELETED:
			if (column != NULL) delete column;
			break;

		case SQLBase::CR_ADDED:
		case SQLBase::CR_MODIFIED:
			if (column == NULL) {
				column = table->createColumn(dColumn->name);
			}

			*column = *dColumn;
			column->compareResult = SQLBase::CR_NOCOMPARE;
			break;
		}

		cPos++;
	}

	// zpracovani indexu
	IndexList indexes = dTable->getIndexes();
	IndexList::iterator iPos = indexes.begin(), iEnd = indexes.end();

	while (iPos != iEnd) {
		ISCONFLICT(iPos)
		Index *dIndex = *iPos;
		Index *index = table->getIndex(dIndex->name);

		// vyhodnoceni operace
		switch (dIndex->compareResult) {
		case SQLBase::CR_DELETED:
			if (index != NULL) delete index;
			break;

		case SQLBase::CR_ADDED:
			if (index == NULL) index = table->createIndex(dIndex->name, ColumnList());

			*index = *dIndex;
			index->compareResult = SQLBase::CR_NOCOMPARE;

			// vyprazdneni starych sloupcu
			ColumnList columns(index->getColumns());
			cPos = columns.begin();
			cEnd = columns.end();

			while (cPos != cEnd) {
				index->removeChild(*cPos);
				cPos++;
			}

			// zapis novych sloupcu
			dColumns = dIndex->getColumns();
			cPos = dColumns.begin();
			cEnd = dColumns.end();

			while (cPos != cEnd) {
				Column *column = table->getColumn((*cPos)->name);

				if (column != NULL) index->addChild(column, MATE_INDEXED);
				cPos++;
			}

			break;
		}

		iPos++;
	}

	// zpracovani triggeru
	TriggerList dTriggers = dTable->getTriggers();
	TriggerList::iterator tPos = dTriggers.begin(), tEnd = dTriggers.end();

	while (tPos != tEnd) {
		ISCONFLICT(tPos)
		Trigger *dTrigger = *tPos;
		Trigger *trigger = table->getTrigger(dTrigger->name);

		switch (dTrigger->compareResult) {
		case SQLBase::CR_DELETED:
			if (trigger != NULL) delete trigger;
			break;

		case SQLBase::CR_ADDED:
			if (trigger == NULL) trigger = table->createTrigger(dTrigger->name);
			*trigger = *dTrigger;
			trigger->compareResult = SQLBase::CR_NOCOMPARE;

			break;
		}

		tPos++;
	}
}

void LocalAdapter::createTable(Table* dTable) {
	Table *table = target->table(dTable->name);

	if (table != NULL) delete table;

	table = target->createTable(dTable->name);
	*table = *dTable;
	table->compareResult = SQLBase::CR_NOCOMPARE;

	// vygenerovani sloupcu a klicu
	copyColumns(table, dTable);
	copyKeys(table, dTable);

	// kopirovani triggeru
	TriggerList triggers = dTable->getTriggers();
	TriggerList::iterator pos = triggers.begin(), end = triggers.end();

	while (pos != end) {
		Trigger *dTrigger = *pos;
		Trigger *trigger = table->createTrigger(dTrigger->name);
		*trigger = *dTrigger;
		trigger->compareResult = SQLBase::CR_NOCOMPARE;

		pos++;
	}
}

void LocalAdapter::diffForeigns(Table* dTable) {
	Table *table = target->table(dTable->name);

	// pokud je tabulka smazana, pak neni co resit
	if (table == NULL) return;

	// roztrideni cizich klicu
	vector<ForeignList> sorted = sortItems<ForeignList>(dTable->getForeignKeys());

	// smazani starych
	ForeignList::iterator pos = sorted[LIST_DELETE].begin(), end = sorted[LIST_DELETE].end();

	while (pos != end) {
		ISCONFLICT(pos)
		ForeignKey *dFk = *pos;
		ForeignKey *fk = table->getForeignKey(dFk->name);

		if (fk != NULL) delete fk;

		pos++;
	}

	// vytvoreni novych
	pos = sorted[LIST_ADD].begin();
	end = sorted[LIST_ADD].end();

	while (pos != end) {
		ISCONFLICT(pos)
		ForeignKey *dFk = *pos;
		ForeignKey *fk = table->getForeignKey(dFk->name);

		if (fk == NULL) {
			// nacteni referencni tabulky
			Table *refTable = target->table(dFk->getReferencedTable()->name);

			if (refTable == NULL) break;

			ColumnList refColumns, columns, dColumns;
			ColumnList::iterator dPos, dEnd;

			// vytvoreni seznamu sloupcu referencni tabulky
			dColumns = dFk->getReferencedColumns();
			dPos = dColumns.begin();
			dEnd = dColumns.end();

			while (dPos != dEnd) {
				Column *column = refTable->getColumn((*dPos)->name);
				if (column == NULL) break;

				refColumns.push_back(column);

				dPos++;
			}

			// vytvoreni seznamu sloupcu teto tabulky
			dColumns = dFk->getColumns();
			dPos = dColumns.begin();
			dEnd = dColumns.end();

			while (dPos != dEnd) {
				Column *column = table->getColumn((*dPos)->name);
				if (column == NULL) break;

				columns.push_back(column);

				dPos++;
			}

			// vytvoreni ciziho klice
			fk = table->createForeignKey(dFk->name, columns, refTable, refColumns);
		}

		*fk = *dFk;
		fk->compareResult = SQLBase::CR_NOCOMPARE;

		pos++;
	}
}

void LocalAdapter::writeDiff(Database* database) {
	diffTables(database);
	diffRoutines(database);
	diffViews(database);
}

bool LocalAdapter::isConflict(SQLBase* element) {
	// vyhodnoeni, jestli je balicek v seznamu
	Conflicts::iterator conflictItem = conflicts.find(element->dbName);

	if (conflictItem == conflicts.end()) return false;

	// vyhodnoceni jestli je prvek veden v seznamu balicku
	ConflictList::iterator pos = conflictItem->second.begin(),
			end = conflictItem->second.end();

	while (pos != end) {
		if (pos->id == element->id) {
			// jedna se o hledanou zmenu
			// pokud je nastavena ze se ma pouzit puvodni vetev, vraci se TRUE (tato zmena se preskakuje), jinak FALSE
			return pos->useCurrent;
		}

		pos++;
	}

	return false;
}

}
