/*
 * MySQLAdapter.cpp
 *
 *  Created on: 22.3.2013
 *      Author: petr
 */

#include "MySQLAdapter.h"

namespace SQLRepresenter {

const char* MySQLAdapter::INFORMATION_SCHEMA = "information_schema";

const char* MySQLAdapter::COLUMN_TABLENAME = "TABLE_NAME";
const char* MySQLAdapter::COLUMN_TABLECOLLATION = "TABLE_COLLATION";
const char* MySQLAdapter::COLUMN_TABLECOMMENT = "TABLE_COMMENT";
const char* MySQLAdapter::COLUMN_TABLEENGINE = "ENGINE";

const char* MySQLAdapter::COLUMN_COLUMNCHARSET = "CHARACTER_SET_NAME";
const char* MySQLAdapter::COLUMN_COLUMNCOMMENT = "COLUMN_COMMENT";
const char* MySQLAdapter::COLUMN_COLUMNDEFAULT = "COLUMN_DEFAULT";
const char* MySQLAdapter::COLUMN_COLUMNEXTRA = "EXTRA";
const char* MySQLAdapter::COLUMN_COLUMNISNULL = "IS_NULLABLE";
const char* MySQLAdapter::COLUMN_COLUMNNAME = "COLUMN_NAME";
const char* MySQLAdapter::COLUMN_COLUMNTYPE = "COLUMN_TYPE";
const char* MySQLAdapter::COLUMN_COLUMNCOLLATION = "COLLATION_NAME";

const char* MySQLAdapter::COLUMN_INDEXNAME = "INDEX_NAME";
const char* MySQLAdapter::COLUMN_INDEXNONUNIQUE = "NON_UNIQUE";
const char* MySQLAdapter::COLUMN_INDEXCOLNAME = "COLUMN_NAME";
const char* MySQLAdapter::COLUMN_INDEXMETHOD = "INDEX_TYPE";

const char* MySQLAdapter::COLUMN_VIEWDEFINER = "DEFINER";
const char* MySQLAdapter::COLUMN_VIEWDEFINITION = "VIEW_DEFINITION";
const char* MySQLAdapter::COLUMN_VIEWNAME = "TABLE_NAME";

const char* MySQLAdapter::COLUMN_FOREIGNNAME = "CONSTRAINT_NAME";
const char* MySQLAdapter::COLUMN_FOREINGREFTABLE = "REFERENCED_TABLE_NAME";
const char* MySQLAdapter::COLUMN_FOREIGNTABLE = "TABLE_NAME";
const char* MySQLAdapter::COLUMN_FOREIGNCOLUMN = "COLUMN_NAME";
const char* MySQLAdapter::COLUMN_FOREIGNREFCOLUMN = "REFERENCED_COLUMN_NAME";
const char* MySQLAdapter::COLUMN_FOREIGNDELETE = "DELETE_RULE";
const char* MySQLAdapter::COLUMN_FOREIGNUPDATE = "UPDATE_RULE";

const char* MySQLAdapter::COLUMN_TRIGGERCODE = "ACTION_STATEMENT";
const char* MySQLAdapter::COLUMN_TRIGGERNAME = "TRIGGER_NAME";
const char* MySQLAdapter::COLUMN_TRIGGEROP = "EVENT_MANIPULATION";
const char* MySQLAdapter::COLUMN_TRIGGERTABLE = "EVENT_OBJECT_TABLE";
const char* MySQLAdapter::COLUMN_TRIGGERTIME = "ACTION_TIMING";

const char* MySQLAdapter::COLUMN_ROUTINEACCESS = "SQL_DATA_ACCESS";
const char* MySQLAdapter::COLUMN_ROUTINECOMMENT = "ROUTINE_COMMENT";
const char* MySQLAdapter::COLUMN_ROUTINEDEF = "ROUTINE_DEFINITION";
const char* MySQLAdapter::COLUMN_ROUTINEDEFINER = "DEFINER";
const char* MySQLAdapter::COLUMN_ROUTINENAME = "ROUTINE_NAME";
const char* MySQLAdapter::COLUMN_ROUTINESPECIFIC = "SPECIFIC_NAME";
const char* MySQLAdapter::COLUMN_ROUTINETYPE = "ROUTINE_TYPE";
const char* MySQLAdapter::COLUMN_ROUTINEDATATYPE = "DATA_TYPE";

const char* MySQLAdapter::PARAM_COLLATION = "mysql_collation";
const char* MySQLAdapter::PARAM_ENGINE = "mysql_engine";
const char* MySQLAdapter::PARAM_CHARSET = "mysql_charset";

const char* MySQLAdapter::BOOL_NO = "NO";
const char* MySQLAdapter::BOOL_YES = "YES";

const char* MySQLAdapter::INDEX_PRIMARY_NAME = "PRIMARY";

const char* MySQLAdapter::TRIGGER_AFTER = "AFTER";
const char* MySQLAdapter::TRIGGER_BEFORE = "BEFORE";
const char* MySQLAdapter::TRIGGER_DELETE = "DELETE";
const char* MySQLAdapter::TRIGGER_INSERT = "INSERT";
const char* MySQLAdapter::TRIGGER_UPDATE = "UPDATE";

const char* MySQLAdapter::ROUTINE_FUNCTION = "FUNCTION";
const char* MySQLAdapter::ROUTINE_PROCEDURE = "PROCEDURE";
const char* MySQLAdapter::ROUTINE_CONTAINS = "CONTAINS SQL";
const char* MySQLAdapter::ROUTINE_MODIFY = "MODIFIES SQL DATA";
const char* MySQLAdapter::ROUTINE_NOSQL = "NO SQL";
const char* MySQLAdapter::ROUTINE_READ = "READS SQL DATA";

const char* MySQLAdapter::COLUMN_PARAMMODE = "PARAMETER_MODE";
const char* MySQLAdapter::COLUMN_PARAMNAME = "PARAMETER_NAME";
const char* MySQLAdapter::COLUMN_PARAMTYPE = "DATA_TYPE";

const char* MySQLAdapter::PARAMETER_IN = "IN";
const char* MySQLAdapter::PARAMETER_INOUT = "INOUT";
const char* MySQLAdapter::PARAMETER_OUT = "OUT";

const char* MySQLAdapter::SELECT_TABLE_LIST =
		"select TABLE_NAME, ENGINE, TABLE_COLLATION, TABLE_COMMENT from TABLES where TABLE_SCHEMA like '%s' AND TABLE_TYPE like 'BASE TABLE'";
const char* MySQLAdapter::SELECT_COLUMN_LIST =
		"select COLUMN_NAME, COLUMN_DEFAULT, IS_NULLABLE, COLUMN_TYPE, CHARACTER_SET_NAME, EXTRA, COLUMN_COMMENT, COLLATION_NAME from COLUMNS where TABLE_SCHEMA like '%s' and TABLE_NAME like '%s'";
const char* MySQLAdapter::SELECT_INDEX_LIST =
		"select INDEX_NAME, NON_UNIQUE, INDEX_TYPE from STATISTICS where TABLE_SCHEMA like '%s' and TABLE_NAME like '%s' group by `INDEX_NAME`";
const char* MySQLAdapter::SELECT_INDEX_COLUMNS =
		"select COLUMN_NAME from STATISTICS where TABLE_SCHEMA like '%s' and TABLE_NAME like '%s' and INDEX_NAME like '%s' order by SEQ_IN_INDEX";
const char* MySQLAdapter::SELECT_FOREIGNKEY_LIST =
		"select UPDATE_RULE, DELETE_RULE, TC.CONSTRAINT_NAME, KCU.REFERENCED_TABLE_NAME, TC.TABLE_NAME from REFERENTIAL_CONSTRAINTS as RC, TABLE_CONSTRAINTS as TC, KEY_COLUMN_USAGE as KCU where TC.TABLE_SCHEMA LIKE '%s' and TC.CONSTRAINT_NAME like KCU.CONSTRAINT_NAME and TC.TABLE_SCHEMA like KCU.TABLE_SCHEMA and CONSTRAINT_TYPE like 'FOREIGN KEY' and RC.CONSTRAINT_SCHEMA like TC.TABLE_SCHEMA and RC.CONSTRAINT_NAME like TC.CONSTRAINT_NAME group by TC.CONSTRAINT_NAME";
const char* MySQLAdapter::SELECT_FOREIGNKEY_COLUMNS =
		"select COLUMN_NAME, REFERENCED_COLUMN_NAME from KEY_COLUMN_USAGE where TABLE_SCHEMA like '%s' and CONSTRAINT_NAME like '%s'";
const char* MySQLAdapter::SELECT_TRIGGER_LIST =
		"select TRIGGER_NAME, EVENT_MANIPULATION, ACTION_TIMING, EVENT_OBJECT_TABLE, ACTION_STATEMENT from TRIGGERS where TRIGGER_SCHEMA like '%s'";
const char* MySQLAdapter::SELECT_ROUTINE_LIST =
		"select SPECIFIC_NAME, ROUTINE_NAME, ROUTINE_TYPE, ROUTINE_DEFINITION, DATA_TYPE, SQL_DATA_ACCESS, ROUTINE_COMMENT, DEFINER from ROUTINES where ROUTINE_SCHEMA like '%s'";
const char* MySQLAdapter::SELECT_ROUTINE_PARAMS =
		"select PARAMETER_NAME, PARAMETER_MODE, DATA_TYPE from PARAMETERS where SPECIFIC_SCHEMA like '%s' and SPECIFIC_NAME like '%s' and ORDINAL_POSITION > 0 order by ORDINAL_POSITION";
const char* MySQLAdapter::SELECT_VIEW_LIST =
		"select TABLE_NAME, DEFINER, VIEW_DEFINITION from VIEWS where TABLE_SCHEMA like '%s'";

const char* MySQLAdapter::SELECT_FOREIGN =
		"select CONSTRAINT_NAME from information_schema.REFERENTIAL_CONSTRAINTS where CONSTRAINT_SCHEMA like '%s' and TABLE_NAME like '%s' and CONSTRAINT_NAME like '%s'";

const char* MySQLAdapter::DROP_TABLE = "DROP TABLE IF EXISTS `%s`";
const char* MySQLAdapter::DROP_VIEW = "DROP VIEW IF EXISTS `%s`";
const char* MySQLAdapter::DROP_TRIGGER = "DROP TRIGGER IF EXISTS `%s`";
const char* MySQLAdapter::DROP_FUNCTION = "DROP FUNCTION IF EXISTS `%s`";
const char* MySQLAdapter::DROP_PROCEDURE = "DROP PROCEDURE IF EXISTS `%s`";
const char* MySQLAdapter::DROP_COLUMN = "ALTER TABLE `%s` DROP COLUMN `%s`";
const char* MySQLAdapter::DROP_INDEX = "ALTER TABLE `%s` DROP INDEX `%s`";

const char* MySQLAdapter::ALTER_ADD_COLUMN = "ADD COLUMN ";
const char* MySQLAdapter::ALTER_ADD_INDEX = "INDEX ";

const char* MySQLAdapter::CONF_HOST = "host";
const char* MySQLAdapter::CONF_PASSWORD = "password";
const char* MySQLAdapter::CONF_DBNAME = "dbname";
const char* MySQLAdapter::CONF_USER = "user";

MySQLAdapter::MySQLAdapter() {
	// TODO Auto-generated constructor stub
	driver = new sql::mysql::MySQL_Driver();
	connection = NULL;
}

MySQLAdapter::~MySQLAdapter() {
	if (connection != NULL)
		delete connection;
	delete driver;
}

Database* MySQLAdapter::readData(Database* database) {
	connect(INFORMATION_SCHEMA);

	readTables(database);
	readViews(database);
	readForeigns(database);
	readTriggers(database);
	readRoutines(database);

	disconnect();

	return database;
}

void MySQLAdapter::setConfig(pugi::xml_node config) {
	pugi::xml_node param = config.child(NODE_PARAM);

	while (param) {
		std::string name = param.attribute(ATTR_NAME).as_string();
		std::string value = param.attribute(ATTR_VALUE).as_string();

		if (name == CONF_HOST) {
			host = value;
		} else if (name == CONF_PASSWORD) {
			password = value;
		} else if (name == CONF_USER) {
			username = value;
		} else if (name == CONF_DBNAME) {
			dbname = value;
		}

		param = param.next_sibling(NODE_PARAM);
	}
}

void MySQLAdapter::connect(const char* dbname) {
	// pokud nejake spojeni existuje, uzavre se a vytvori se nove
	if (connection != NULL)
		disconnect();

	// otevreni novehho spojeni
	connection = driver->connect(host, username, password);
	connection->setSchema(dbname);
}

void MySQLAdapter::disconnect() {
	// kontrola, jestli je spojeni uzavreno
	if (connection == NULL)
		return;

	connection->close();
	delete connection;
	connection = NULL;
}

void MySQLAdapter::readTables(Database* database) {
	char buffer[256];
	std::sprintf(buffer, SELECT_TABLE_LIST, dbname.c_str());
	sql::SQLString query(buffer);
	sql::PreparedStatement *stm = connection->prepareStatement(query);
	sql::ResultSet *result = stm->executeQuery();

	// prochazeni tabulek a generovani jejich struktury
	while (result->next()) {
		// vytvoreni tabulky
		std::string name(result->getString(COLUMN_TABLENAME));
		Table* table = new Table(database, name);

		// nastavnei komentare a dalsich parametru
		table->comment = result->getString(COLUMN_TABLECOMMENT);
		table->createParameter(PARAM_COLLATION).value = result->getString(
				COLUMN_TABLECOLLATION);

		table->createParameter(PARAM_ENGINE).value = result->getString(
				COLUMN_TABLEENGINE);

		readColumns(table);
		readIndexes(table);
	}

	delete result;
	delete stm;
}

void MySQLAdapter::readColumns(Table* table) {
	// vygenerovani dotazu
	char buffer[256];
	std::sprintf(buffer, SELECT_COLUMN_LIST, dbname.c_str(),
			table->name.c_str());

	// nacteni dat z databaze a vygenerovani sloupcu
	sql::PreparedStatement *stm = connection->prepareStatement(buffer);
	sql::ResultSet* res = stm->executeQuery();

	// zapis sloupcu
	while (res->next()) {
		Column* column = table->createColumn(res->getString(COLUMN_COLUMNNAME));
		column->comment = res->getString(COLUMN_COLUMNCOMMENT);
		column->dataType = res->getString(COLUMN_COLUMNTYPE);

		// vyhodnoceni vychozi hodnoty
		if (res->isNull(COLUMN_COLUMNDEFAULT)) {
			column->defaultUsed = false;
		} else {
			column->defaultUsed = true;
			column->defaultValue = res->getString(COLUMN_COLUMNDEFAULT);
		}

		// vyhodnoceni nullovosti
		if (res->getString(COLUMN_COLUMNISNULL) == BOOL_YES)
			column->notNull = false;

		// vyhodnoceni extra
		if (!res->isNull(COLUMN_COLUMNEXTRA))
			column->extra = res->getString(COLUMN_COLUMNEXTRA);

		// vyhodnoceni znakove sady
		if (!res->isNull(COLUMN_COLUMNCHARSET))
			column->createParameter(PARAM_CHARSET).value = res->getString(
					COLUMN_COLUMNCHARSET);

		// vyhodnoceni porovnavani
		if (!res->isNull(COLUMN_COLUMNCOLLATION))
			column->createParameter(PARAM_COLLATION).value = res->getString(
					COLUMN_COLUMNCOLLATION);
	}

	delete res;
	delete stm;
}

void MySQLAdapter::readIndexes(Table* table) {
	// nataceni seznamu indexu
	char buffer[256];
	std::sprintf(buffer, SELECT_INDEX_LIST, dbname.c_str(),
			table->name.c_str());
	sql::PreparedStatement *stm = connection->prepareStatement(buffer);
	sql::ResultSet *res = stm->executeQuery();

	while (res->next()) {
		Index *index = readIndex(table, res->getString(COLUMN_INDEXNAME));

		// vyhodnoceni typu indexu
		if (res->getInt(COLUMN_INDEXNONUNIQUE)) {
			// jedna se o klasicky index
			index->indexType = INDEX_TYPE_KEY;
		} else {
			// jedna se o unikatni klic nebo primarni klic
			if (res->getString(COLUMN_INDEXNAME) == INDEX_PRIMARY_NAME) {
				index->indexType = INDEX_TYPE_PRIMARY;
			} else {
				index->indexType = INDEX_TYPE_UNIQUE;
			}
		}

		// zapis typu indexovani
		index->indexMethod = res->getString(COLUMN_INDEXMETHOD);
	}

	delete res;
	delete stm;
}

Index* MySQLAdapter::readIndex(Table* table, std::string name) {
	char buffer[256];
	std::sprintf(buffer, SELECT_INDEX_COLUMNS, dbname.c_str(),
			table->name.c_str(), name.c_str());
	ColumnList cols;

	sql::PreparedStatement *stm = connection->prepareStatement(buffer);
	sql::ResultSet *res = stm->executeQuery();

	while (res->next())
		cols.push_back(table->getColumn(res->getString(COLUMN_INDEXCOLNAME)));

	delete res;
	delete stm;

	return table->createIndex(name, cols);
}

void MySQLAdapter::readViews(Database* database) {
	char buffer[256];
	std: sprintf(buffer, SELECT_VIEW_LIST, dbname.c_str());
	sql::PreparedStatement *stm = connection->prepareStatement(buffer);
	sql::ResultSet *res = stm->executeQuery();

	while (res->next()) {
		View *view = database->createView(res->getString(COLUMN_VIEWNAME));
		view->code = res->getString(COLUMN_VIEWDEFINITION);
	}

	delete res;
	delete stm;
}

void MySQLAdapter::readForeigns(Database* database) {
	// nacteni seznamu cizich klicu
	char buffer[500];

	std::sprintf(buffer, SELECT_FOREIGNKEY_LIST, dbname.c_str());

	sql::PreparedStatement *stm = connection->prepareStatement(buffer);
	sql::ResultSet *res = stm->executeQuery();
	ForeignKey* foreign;

	while (res->next()) {
		foreign = readForeign(
				database->table(res->getString(COLUMN_FOREIGNTABLE)),
				database->table(res->getString(COLUMN_FOREINGREFTABLE)),
				res->getString(COLUMN_FOREIGNNAME));

		// nastaveni udalosti
		foreign->onDelete = res->getString(COLUMN_FOREIGNDELETE);
		foreign->onUpdate = res->getString(COLUMN_FOREIGNUPDATE);
	}

	delete res;
	delete stm;
}

ForeignKey* MySQLAdapter::readForeign(Table* table, Table* refTable,
		std::string name) {

	char buffer[256];
	std::sprintf(buffer, SELECT_FOREIGNKEY_COLUMNS, dbname.c_str(),
			name.c_str());
	sql::PreparedStatement *stm = connection->prepareStatement(buffer);
	sql::ResultSet *res = stm->executeQuery();

	// sestaveni seznamu sloupcu
	ColumnList source, target;

	while (res->next()) {
		source.push_back(
				table->getColumn(res->getString(COLUMN_FOREIGNCOLUMN)));
		target.push_back(
				table->getColumn(res->getString(COLUMN_FOREIGNREFCOLUMN)));
	}

	delete res;
	delete stm;

	return table->createForeignKey(name, source, refTable, target);
}

void MySQLAdapter::readTriggers(Database* database) {
	char buffer[256];
	std::sprintf(buffer, SELECT_TRIGGER_LIST, dbname.c_str());
	sql::PreparedStatement *stm = connection->prepareStatement(buffer);
	sql::ResultSet *res = stm->executeQuery();

	Table* table;
	Trigger* trigger;
	std::string tmp;

	while (res->next()) {
		// vytvoreni triggeru
		table = database->table(res->getString(COLUMN_TRIGGERTABLE));
		trigger = table->createTrigger(res->getString(COLUMN_TRIGGERNAME));
		trigger->code = res->getString(COLUMN_TRIGGERCODE);
		// nastaveni operace
		tmp = res->getString(COLUMN_TRIGGEROP);

		if (tmp == TRIGGER_DELETE) {
			trigger->operation = DELETE;
		} else if (tmp == TRIGGER_INSERT) {
			trigger->operation = INSERT;
		} else if (tmp == TRIGGER_UPDATE) {
			trigger->operation = UPDATE;
		}

		// nastaveni casu
		tmp = res->getString(COLUMN_TRIGGERTIME);

		if (tmp == TRIGGER_AFTER) {
			trigger->time = AFTER;
		} else if (tmp == TRIGGER_BEFORE) {
			trigger->time = BEFORE;
		}
	}

	delete res;
	delete stm;
}

void MySQLAdapter::readRoutines(Database* database) {
	// nacteni informaci o rutinach
	char buffer[256];
	std::sprintf(buffer, SELECT_ROUTINE_LIST, dbname.c_str());
	sql::PreparedStatement *stm = connection->prepareStatement(buffer);
	sql::ResultSet *res = stm->executeQuery();

	Procedure* routine;

	while (res->next()) {
		// vyhodnoceni typu
		if (res->getString(COLUMN_ROUTINETYPE) == ROUTINE_PROCEDURE) {
			// jedna se o proceduru
			routine = new Procedure(database,
					res->getString(COLUMN_ROUTINENAME));
		} else {
			// jedna se o funkci -> musi se nastavit i datovy typ
			Function* func = new SQLRepresenter::Function(database,
					res->getString(COLUMN_ROUTINENAME));
			// func->returnType = res->getString(COLUMN_ROUTINEDATATYPE);
			func->returnType = "int";

			routine = func;
		}

		// nastaveni obecnych vlastnosti
		routine->comment = res->getString(COLUMN_ROUTINECOMMENT);
		routine->code = res->getString(COLUMN_ROUTINEDEF);

		// vyhodnoceni pristupu k datum
		std::string mode = res->getString(COLUMN_ROUTINEACCESS);

		if (mode == ROUTINE_CONTAINS) {
			routine->dataAccess = Procedure::DA_CONTAINSQL;
		} else if (mode == ROUTINE_MODIFY) {
			routine->dataAccess = Procedure::DA_MODIFYSQLDATA;
		} else if (mode == ROUTINE_NOSQL) {
			routine->dataAccess = Procedure::DA_NOSQL;
		} else if (mode == ROUTINE_READ) {
			routine->dataAccess = Procedure::DA_READSQLDATA;
		}

		readRoutinesParams(routine);
	}

	delete res;
	delete stm;
}

void MySQLAdapter::readRoutinesParams(Procedure* routine) {
	// nacteni dat
	char buffer[256];
	std::sprintf(buffer, SELECT_ROUTINE_PARAMS, dbname.c_str(),
			routine->name.c_str());
	sql::PreparedStatement *stm = connection->prepareStatement(buffer);
	sql::ResultSet *res = stm->executeQuery();

	while (res->next()) {
		PROC_PARAM param;

		param.dataType = res->getString(COLUMN_PARAMTYPE);
		param.name = res->getString(COLUMN_PARAMNAME);

		// vyhodnoceni typu
		std::string direction = res->getString(COLUMN_PARAMMODE);

		if (direction == PARAMETER_IN) {
			param.paramType = IN;
		} else if (direction == PARAMETER_OUT) {
			param.paramType = OUT;
		} else if (direction == PARAMETER_INOUT) {
			param.paramType = INOUT;
		} else {
			param.paramType = IN;
		}

		routine->getParameters().push_back(param);
	}

	delete res;
	delete stm;
}

void MySQLAdapter::writeTables(Database* database) {

	TableList tables = database->tables();
	TableList::iterator pos = tables.begin(), end = tables.end();

	while (pos != end) {
		Table *table = *pos;

		// smazani tabulky, pokud existuje
		dropTopLevelItem(DROP_TABLE, table->name.c_str());

		writeTable(table);

		pos++;
	}
}

void MySQLAdapter::writeTable(Table* table) {
	sql::Statement *stm = connection->createStatement();

	// sestaveni vytvareciho dotazu pro sloupce a indexy
	STRING_VEC columns = writeColumns(table);
	STRING_VEC indexes = writeIndexes(table);

	// slouceni seznamu do jednoho
	columns.insert(columns.end(), indexes.begin(), indexes.end());

	// sestaveni pocatku dotazu
	std::string query("CREATE TABLE `");
	query += table->name + "` (";
	query += implode(",", columns) + ")";

	if (table->isParameter(PARAM_ENGINE)) {
		query += " ENGINE=" + table->getParameter(PARAM_ENGINE).value;
	}

	if (table->isParameter(PARAM_COLLATION)
			&& table->getParameter(PARAM_COLLATION).value.size() > 0) {
		query += " collate=" + table->getParameter(PARAM_COLLATION).value;
	}

	if (table->isParameter(PARAM_CHARSET)) {
		query += " default charset " + table->getParameter(PARAM_CHARSET).value;
	}

	// odeslani dotazu pro vytvoreni tabulky
	stm->execute(query);
	delete stm;
}

std::string MySQLAdapter::implode(std::string glue,
		const MySQLAdapter::STRING_VEC& strings) {
	std::string retVal;

	if (strings.size()) {
		STRING_VEC::const_iterator pos = strings.begin(), end = strings.end();

		retVal += *pos;

		pos++;

		while (pos != end) {
			retVal += glue;
			retVal += *pos;

			pos++;
		}
	}

	return retVal;
}

MySQLAdapter::STRING_VEC MySQLAdapter::writeColumns(Table* table) {
	STRING_VEC retVal;

	ColumnList columns = table->getColumns();
	ColumnList::iterator pos = columns.begin(), end = columns.end();

	while (pos != end) {
		retVal.push_back(generateColumn(*pos));
		pos++;
	}

	return retVal;
}

std::string MySQLAdapter::generateColumn(Column* column) {
	std::string columnDef("`");
	columnDef += column->name + "` ";
	columnDef += column->dataType;

	if (column->notNull) {
		columnDef += " NOT NULL ";
	} else {
		columnDef += " NULL ";
	}

	columnDef += column->extra + " ";

	if (column->defaultUsed) {
		columnDef += " DEFAULT";
		columnDef += " " + column->defaultValue + "";
	}

	return columnDef;
}

MySQLAdapter::STRING_VEC MySQLAdapter::writeIndexes(Table* table) {
	STRING_VEC retVal;

	IndexList indexes = table->getIndexes();
	IndexList::iterator pos = indexes.begin(), end = indexes.end();

	while (pos != end) {
		retVal.push_back(generateIndex(*pos));

		pos++;
	}

	return retVal;
}

std::string MySQLAdapter::generateIndex(Index* index) {
	std::string indexDef;

	// vyhodnoceni typu indexu
	switch (index->indexType) {
	case INDEX_TYPE_KEY:
		indexDef += "INDEX (";
		break;

	case INDEX_TYPE_UNIQUE:
		indexDef += "UNIQUE (";
		break;

	case INDEX_TYPE_PRIMARY:
		indexDef += "PRIMARY KEY (";
		break;
	}

	// vytvoreni seznamu sloupcu a jejich zapis
	ColumnList columns = index->getColumns();
	ColumnList::iterator cPos = columns.begin(), cEnd = columns.end();
	STRING_VEC columnNames;

	while (cPos != cEnd) {
		columnNames.push_back((*cPos)->name);
		cPos++;
	}

	indexDef += implode(",", columnNames) + ")";

	return indexDef;
}

void MySQLAdapter::writeViews(Database* database) {
	ViewList views = database->views();
	ViewList::iterator pos = views.begin(), end = views.end();

	while (pos != end) {
		// samazani pohledu
		View* view = *pos;
		dropTopLevelItem(DROP_VIEW, view->name.c_str());

		createView(view);

		pos++;
	}
}

void MySQLAdapter::writeTriggers(Database* database) {
	NodeList triggers = database->getNodes(NODE_TRIGGER);
	NodeList::iterator pos = triggers.begin(), end = triggers.end();
	sql::Statement *stm;

	while (pos != end) {
		Trigger* trigger = (Trigger*) *pos;

		// smazani stareho triggeru
		dropTopLevelItem(DROP_TRIGGER, trigger->name.c_str());

		// sestaveni dotazu pro vytvoreni triggeru
		Table* table = trigger->getTable();

		std::string query("CREATE TRIGGER `");
		query += trigger->name + "` ";

		// vyhodncoeni casu
		switch (trigger->time) {
		case AFTER:
			query += TRIGGER_AFTER;
			break;

		case BEFORE:
		default:
			query += TRIGGER_BEFORE;
			break;
		}

		query += " ";

		// vyhodncoeni udalosti
		switch (trigger->operation) {
		case UPDATE:
			query += TRIGGER_UPDATE;
			break;

		case DELETE:
			query += TRIGGER_DELETE;
			break;

		case INSERT:
		default:
			query += TRIGGER_INSERT;
			break;
		}

		// dalsi informace
		query += " ON `" + table->name + "` FOR EACH ROW ";

		// telo triggeru
		query += trigger->code;

		// odelsani dat na server
		stm = connection->createStatement();
		stm->execute(query);
		delete stm;

		pos++;
	}
}

void MySQLAdapter::writeProcedures(Database* database) {
	ProcedureList procedures = database->procedures();
	ProcedureList::iterator pos = procedures.begin(), end = procedures.end();

	while (pos != end) {
		// smazani stare procedury, pokud existuje
		dropTopLevelItem(DROP_PROCEDURE, (*pos)->name.c_str());

		// zapis rutiny
		writeRoutine(*pos, true);

		pos++;
	}
}

void MySQLAdapter::writeFunctions(Database* database) {
	FunctionList functions = database->functions();
	FunctionList::iterator pos = functions.begin(), end = functions.end();

	while (pos != end) {
		// smazani stare procedury, pokud existuje
		dropTopLevelItem(DROP_FUNCTION, (*pos)->name.c_str());

		// zapis rutiny
		writeRoutine(*pos, false);

		pos++;
	}
}

void MySQLAdapter::writeRoutine(Procedure* routine, bool isProcedure) {
	// sestaveni pocatku
	std::string query("CREATE ");

	if (isProcedure) {
		query += "PROCEDURE ";
	} else {
		query += " FUNCTION ";
	}

	query += routine->name + " (";

	// zapis parametru
	ProcParamList params = routine->getParameters();
	ProcParamList::iterator pos = params.begin(), end = params.end();

	STRING_VEC stringParams;

	while (pos != end) {
		std::string parameter;

		// vyhodnoceni smeru
		if (isProcedure) {
			switch (pos->paramType) {
			case IN:
				parameter += PARAMETER_IN;
				break;

			case OUT:
				parameter += PARAMETER_OUT;
				break;

			case INOUT:
				parameter += PARAMETER_INOUT;
				break;

			default:
				parameter += PARAMETER_IN;
				break;
			}
		}

		// zapis jmena a datoveho typu
		parameter += " " + pos->name + " " + pos->dataType;
		stringParams.push_back(parameter);

		pos++;
	}

	query += implode(",", stringParams) + ")";

	if (!isProcedure) {
		// jedna se o funkci -> zapise se navratovy typ
		query += " RETURNS " + ((Function*) routine)->returnType + " ";
	}

	// zapsani charakteristik
	switch (routine->dataAccess) {
	case Procedure::DA_CONTAINSQL:
		query += ROUTINE_CONTAINS;
		break;

	case Procedure::DA_MODIFYSQLDATA:
		query += ROUTINE_MODIFY;
		break;

	case Procedure::DA_NOSQL:
		query += ROUTINE_NOSQL;
		break;

	case Procedure::DA_READSQLDATA:
		query += ROUTINE_READ;
		break;

	default:
		query += ROUTINE_READ;
		break;
	}

	query += " " + routine->code;

	// odeslani dat
	sql::Statement *stm = connection->createStatement();
	stm->execute(query);
	delete stm;
}

void MySQLAdapter::writeForeigns(Database* database) {
	// nacteni klicu z reprezentace
	NodeList foreigns = database->getNodes(NODE_FOREIGN_KEY);
	NodeList::iterator pos = foreigns.begin(), end = foreigns.end();

	while (pos != end) {
		// nacteni dulezitych dat
		writeForeign((ForeignKey*) *pos);

		pos++;
	}
}

void MySQLAdapter::writeForeign(ForeignKey* foreign) {
	sql::Statement *stm;

	Table *table = foreign->getTable(), *refTable =
			foreign->getReferencedTable();
	ColumnList columns = foreign->getColumns(), refColumns =
			foreign->getReferencedColumns();

	std::string query("ALTER TABLE `");
	query += table->name + "` ADD FOREIGN KEY `" + foreign->name + "` (";

	// sestaveni seznamu mistnich sloupcu
	STRING_VEC columnNames;
	ColumnList::iterator cPos = columns.begin(), cEnd = columns.end();

	while (cPos != cEnd) {
		columnNames.push_back((*cPos)->name);

		cPos++;
	}

	query += implode(",", columnNames) + ") REFERENCES " + refTable->name
			+ " (";

	// zapis seznamu referencnich sloupcu
	cPos = refColumns.begin();
	cEnd = refColumns.end();
	columnNames.clear();

	while (cPos != cEnd) {
		columnNames.push_back((*cPos)->name);

		cPos++;
	}

	// zapis udalosti
	query += implode(",", columnNames) + ") ON DELETE " + foreign->onDelete
			+ " ON UPDATE " + foreign->onUpdate;

	stm = connection->createStatement();
	stm->execute(query);
	delete stm;
}

void MySQLAdapter::dropTopLevelItem(const char* templateStr, const char* name) {
	char buffer[256];
	std::sprintf(buffer, templateStr, name);

	sql::Statement *stm = connection->createStatement();
	stm->execute(buffer);
	delete stm;
}

void MySQLAdapter::writeData(Database* database) {
	connect(dbname.c_str());

	// vypnuti kontroly dat
	sql::Statement *stm = connection->createStatement();
	stm->execute("SET foreign_key_checks = 0");

	writeTables(database);
	writeViews(database);
	writeTriggers(database);
	writeProcedures(database);
	writeFunctions(database);
	writeForeigns(database);

	// zapnuti kontroly dat
	stm->execute("SET foreign_key_checks = 1");
	delete stm;

	disconnect();
}

void MySQLAdapter::diffTables(Database* database) {

	// nejprve je nutne znicit stare cizi klice
	NodeList nodes = database->getNodes(NODE_FOREIGN_KEY);
	ForeignList foreigns = *(reinterpret_cast<ForeignList*>(&nodes));
	std::vector<ForeignList> sortFk = sortItems<ForeignList>(foreigns);

	sql::Statement *stm = connection->createStatement();

	ForeignList::iterator fkPos, fkEnd;
	fkPos = sortFk[LIST_DELETE].begin();
	fkEnd = sortFk[LIST_DELETE].end();

	while (fkPos != fkEnd) {
		dropForeign(*fkPos);
		fkPos++;
	}

	// smazani tabulek, ktere byly urceny ke smazani
	std::vector<TableList> sortedTables = sortItems<TableList>(
			database->tables());
	TableList::iterator tPos, tEnd;
	tPos = sortedTables[LIST_DELETE].begin();
	tEnd = sortedTables[LIST_DELETE].end();

	while (tPos != tEnd) {
		dropTopLevelItem(DROP_TABLE, (*tPos)->name.c_str());
		tPos++;
	}

	// vytvoreni novych tabulek
	tPos = sortedTables[LIST_ADD].begin();
	tEnd = sortedTables[LIST_ADD].end();

	while (tPos != tEnd) {
		writeTable(*tPos);
		tPos++;
	}

	//zmena stavajicich tabulek
	tPos = sortedTables[LIST_CHANGE].begin();
	tEnd = sortedTables[LIST_CHANGE].end();

	while (tPos != tEnd) {
		Table* table = *tPos;

		// zniceni indexu
		std::vector<IndexList> sortIndexes = sortItems<IndexList>(
				table->getIndexes());
		IndexList::iterator iPos, iEnd;
		iPos = sortIndexes[LIST_DELETE].begin();
		iEnd = sortIndexes[LIST_DELETE].end();

		while (iPos != iEnd) {
			dropTableItem(DROP_INDEX, table->name.c_str(),
					(*iPos)->name.c_str());
			iPos++;
		}

		// zniceni sloupcu
		std::vector<ColumnList> sortColumns = sortItems<ColumnList>(
				table->getColumns());
		ColumnList::iterator cPos, cEnd;
		cPos = sortColumns[LIST_DELETE].begin();
		cEnd = sortColumns[LIST_DELETE].end();

		while (cPos != cEnd) {
			dropTableItem(DROP_COLUMN, table->name.c_str(),
					(*cPos)->name.c_str());
			cPos++;
		}

		// vytvoreni sloupcu
		cPos = sortColumns[LIST_ADD].begin();
		cEnd = sortColumns[LIST_ADD].end();

		STRING_VEC items;

		while (cPos != cEnd) {
			std::string subQ(ALTER_ADD_COLUMN);
			subQ += generateColumn(*cPos);

			items.push_back(subQ);
			cPos++;
		}

		// vytvoreni indexu
		iPos = sortIndexes[LIST_ADD].begin();
		iEnd = sortIndexes[LIST_ADD].end();

		while (iPos != iEnd) {
			std::string subQ(ALTER_ADD_INDEX);
			subQ += generateIndex(*iPos);
			items.push_back(subQ);
			iPos++;
		}

		if (items.size()) {
			std::string query = "ALTER TABLE `" + table->name + "` ";
			query += implode(", ", items);

			stm->execute(query.c_str());
		}

		tPos++;
	}

	// zapis cizich klicu
	for (fkPos = sortFk[LIST_ADD].begin(), fkEnd = sortFk[LIST_ADD].end();
			fkPos != fkEnd; fkPos++) {
		writeForeign(*fkPos);
	}

	delete stm;
}

void MySQLAdapter::dropForeign(ForeignKey* fk) {
	// kontrola, jeslti FK stale existuje
	Table* table = fk->getTable();

	char buffer[1024];
	std::sprintf(buffer, SELECT_FOREIGN, dbname.c_str(), table->name.c_str(),
			fk->name.c_str());
	sql::Statement* stm = connection->createStatement();
	sql::ResultSet *res = stm->executeQuery(buffer);

	if (res->rowsCount() > 0) {
		// FK existuje - smaze se
		std::string query = "ALTER TABLE `" + table->name
				+ "` DROP FOREIGN KEY `" + fk->name + "`";
		stm->execute(query.c_str());
	}

	delete res;
	delete stm;
}

bool MySQLAdapter::isTopLevelItemExists(const char* templateStr,
		const char* itemName) {
	char buffer[1024];
	std::sprintf(buffer, templateStr, dbname.c_str(), itemName);
	sql::Statement *stm = connection->createStatement();
	sql::ResultSet *res = stm->executeQuery(buffer);

	// vyhodnoceni existence
	bool retVal = (res->rowsCount() > 0);

	delete res;
	delete stm;

	return retVal;
}

void MySQLAdapter::diffProcedures(Database* database) {
	diffRoutine<ProcedureList>(database->procedures(), true, DROP_PROCEDURE);
}

void MySQLAdapter::diffFunctions(Database* database) {
	diffRoutine<FunctionList>(database->functions(), false, DROP_FUNCTION);
}

void MySQLAdapter::diffViews(Database* database) {
	vector<ViewList> sorted = sortItems<ViewList>(database->views());

	ViewList::iterator pos, end;
	pos = sorted[LIST_DELETE].begin();
	end = sorted[LIST_DELETE].end();

	// smazani starych
	while (pos != end) {
		dropTopLevelItem(DROP_VIEW, (*pos)->name.c_str());
		pos++;
	}

	// vytvoreni novych
	pos = sorted[LIST_ADD].begin();
	end = sorted[LIST_ADD].end();

	while (pos != end) {
		createView(*pos);
		pos++;
	}
}

void MySQLAdapter::createView(View* view) {
	sql::Statement *stm = connection->createStatement();
	std::string query("CREATE VIEW `");

	query += view->name + "` AS ";
	query += view->code;

	stm->execute(query.c_str());
	delete stm;

}

void MySQLAdapter::dropTableItem(const char* templateStr, const char* tableName,
		const char* itemName) {

	char buffer[1024];
	std::sprintf(buffer, templateStr, tableName, itemName);

	sql::Statement *stm = connection->createStatement();
	stm->execute(buffer);

	delete stm;
}

void MySQLAdapter::writeDiff(Database* database) {
	connect(dbname.c_str());

	diffTables(database);
	diffFunctions(database);
	diffProcedures(database);
	diffViews(database);

	disconnect();
}

} /* namespace SQLRepresenter */
