/*
 * MySQLAdapter.h
 *
 *  Created on: 22.3.2013
 *      Author: petr
 */

#ifndef MYSQLADAPTER_H_
#define MYSQLADAPTER_H_

#include "Adapter.h"
#include <string>
#include <vector>
#include <cstdio>
#include <cstring>
#include <cstdarg>
#include <mysql_connection.h>
#include <mysql_driver.h>
#include <cppconn/prepared_statement.h>
#include <cppconn/resultset.h>

#include <iostream>

namespace SQLRepresenter {

class MySQLAdapter: public Adapter {

	typedef std::vector<std::string> STRING_VEC;

	sql::mysql::MySQL_Driver *driver;

	sql::Connection *connection;

	void connect(const char* dbname);

	void disconnect();

	void readTables(Database* database);

	void readViews(Database* database);

	void readColumns(Table* table);

	void readIndexes(Table* table);

	Index* readIndex(Table* table, std::string name);

	void readForeigns(Database* database);

	ForeignKey* readForeign(Table* table, Table* refTable, std::string name);

	void readTriggers(Database* database);

	void readRoutines(Database* database);

	void readRoutinesParams(Procedure* routine);

	void writeTables(Database* database);

	void writeTable(Table* table);

	std::string implode(std::string glue, const STRING_VEC& strings);

	STRING_VEC writeColumns(Table *table);

	std::string generateColumn(Column* column);

	STRING_VEC writeIndexes(Table *table);

	std::string generateIndex(Index *index);

	void writeViews(Database* database);

	void createView(View* view);

	void writeTriggers(Database* database);

	void writeProcedures(Database* database);

	void writeFunctions(Database* database);

	void writeRoutine(Procedure* routine, bool isProcedure);

	void writeForeigns(Database* database);

	void writeForeign(ForeignKey* foreign);

	void diffTables(Database* database);

	void diffProcedures(Database* database);

	void diffFunctions(Database* database);

	void diffViews(Database* database);

	template<typename listType>
	void diffRoutine(listType routines, bool isProcedure,
			const char* dropTemplate);

	void dropForeign(ForeignKey* fk);

	/**
	 * odesle na server dotaz pro smazani prvku
	 */
	void dropTopLevelItem(const char* templateStr, const char* name);

	/**
	 * zkontroluje, jestli prvek nejvyssi urovne
	 */
	bool isTopLevelItemExists(const char* templateStr, const char* itemName);

	void dropTableItem(const char* templateStr, const char* tableName, const char* itemName);

public:

	static const char* INFORMATION_SCHEMA;

	static const char* COLUMN_TABLENAME;
	static const char* COLUMN_TABLEENGINE;
	static const char* COLUMN_TABLECOLLATION;
	static const char* COLUMN_TABLECOMMENT;

	static const char* COLUMN_COLUMNNAME;
	static const char* COLUMN_COLUMNTYPE;
	static const char* COLUMN_COLUMNDEFAULT;
	static const char* COLUMN_COLUMNISNULL;
	static const char* COLUMN_COLUMNCHARSET;
	static const char* COLUMN_COLUMNEXTRA;
	static const char* COLUMN_COLUMNCOMMENT;
	static const char* COLUMN_COLUMNCOLLATION;

	static const char* COLUMN_INDEXNAME;
	static const char* COLUMN_INDEXNONUNIQUE;
	static const char* COLUMN_INDEXCOLNAME;
	static const char* COLUMN_INDEXMETHOD;

	static const char* COLUMN_VIEWNAME;
	static const char* COLUMN_VIEWDEFINER;
	static const char* COLUMN_VIEWDEFINITION;

	static const char* COLUMN_FOREIGNNAME;
	static const char* COLUMN_FOREINGREFTABLE;
	static const char* COLUMN_FOREIGNTABLE;
	static const char* COLUMN_FOREIGNCOLUMN;
	static const char* COLUMN_FOREIGNREFCOLUMN;
	static const char* COLUMN_FOREIGNDELETE;
	static const char* COLUMN_FOREIGNUPDATE;

	static const char* COLUMN_TRIGGERNAME;
	static const char* COLUMN_TRIGGERTABLE;
	static const char* COLUMN_TRIGGERTIME;
	static const char* COLUMN_TRIGGEROP;
	static const char* COLUMN_TRIGGERCODE;

	static const char* COLUMN_ROUTINESPECIFIC;
	static const char* COLUMN_ROUTINENAME;
	static const char* COLUMN_ROUTINETYPE;
	static const char* COLUMN_ROUTINEDEF;
	static const char* COLUMN_ROUTINEACCESS;
	static const char* COLUMN_ROUTINECOMMENT;
	static const char* COLUMN_ROUTINEDEFINER;
	static const char* COLUMN_ROUTINEDATATYPE;

	static const char* COLUMN_PARAMNAME;
	static const char* COLUMN_PARAMMODE;
	static const char* COLUMN_PARAMTYPE;

	static const char* PARAM_ENGINE;
	static const char* PARAM_COLLATION;
	static const char* PARAM_CHARSET;

	static const char* SELECT_TABLE_LIST;
	static const char* SELECT_COLUMN_LIST;
	static const char* SELECT_INDEX_LIST;
	static const char* SELECT_INDEX_COLUMNS;
	static const char* SELECT_VIEW_LIST;
	static const char* SELECT_FOREIGNKEY_LIST;
	static const char* SELECT_FOREIGNKEY_COLUMNS;
	static const char* SELECT_TRIGGER_LIST;
	static const char* SELECT_ROUTINE_LIST;
	static const char* SELECT_ROUTINE_PARAMS;

	static const char* SELECT_FOREIGN;

	static const char* BOOL_YES;
	static const char* BOOL_NO;

	static const char* INDEX_PRIMARY_NAME;

	static const char* TRIGGER_BEFORE;
	static const char* TRIGGER_AFTER;
	static const char* TRIGGER_UPDATE;
	static const char* TRIGGER_DELETE;
	static const char* TRIGGER_INSERT;

	static const char* ROUTINE_FUNCTION;
	static const char* ROUTINE_PROCEDURE;
	static const char* ROUTINE_MODIFY;
	static const char* ROUTINE_READ;
	static const char* ROUTINE_CONTAINS;
	static const char* ROUTINE_NOSQL;

	static const char* PARAMETER_IN;
	static const char* PARAMETER_OUT;
	static const char* PARAMETER_INOUT;

	static const char* DROP_TABLE;
	static const char* DROP_VIEW;
	static const char* DROP_TRIGGER;
	static const char* DROP_PROCEDURE;
	static const char* DROP_FUNCTION;
	static const char* DROP_COLUMN;
	static const char* DROP_INDEX;

	static const char* ALTER_ADD_COLUMN;
	static const char* ALTER_ADD_INDEX;

	const static char* CONF_HOST;
	const static char* CONF_DBNAME;
	const static char* CONF_USER;
	const static char* CONF_PASSWORD;

	std::string dbname;

	std::string username;

	std::string password;

	std::string host;

	MySQLAdapter();
	virtual ~MySQLAdapter();

	virtual Database* readData(Database* database);

	virtual void setConfig(pugi::xml_node config);

	virtual void writeData(Database* database);

	virtual void writeDiff(Database* database);
};

} /* namespace SQLRepresenter */

template<typename listType>
inline void SQLRepresenter::MySQLAdapter::diffRoutine(listType routines,
		bool isProcedure, const char* dropTemplate) {

	std::vector<listType> sorted = sortItems<listType>(routines);

	// smazani starych procedur
	typename listType::iterator pos, end;

	pos = sorted[Adapter::LIST_DELETE].begin();
	end = sorted[Adapter::LIST_DELETE].end();

	while (pos != end) {
		dropTopLevelItem(dropTemplate, (*pos)->name.c_str());

		pos++;
	}

	// zapis novych procedur
	pos = sorted[Adapter::LIST_ADD].begin();
	end = sorted[Adapter::LIST_ADD].end();

	while (pos != end) {
		writeRoutine(*pos, isProcedure);

		pos++;
	}
}

#endif /* MYSQLADAPTER_H_ */
