/*
 * SQLBase.h
 *
 *  Created on: 6.2.2013
 *      Author: petr
 */

#ifndef SQLBASE_H_
#define SQLBASE_H_

#include <string>
#include <map>
#include "../Graph/Graph.h"

using namespace std;

namespace SQLRepresenter {


#ifndef PARAM_MAP
#define PARAM_MAP
struct PARAMETER {
	string value;
	bool switcher;
};

typedef map<string, PARAMETER> ParamMap;

#endif

#ifndef DATABASE_H_

class Database;

#endif



class SQLBase : public Graph::GraphNode {

	ParamMap params;

protected:

	void dropItem(Graph::NodeList items, string name);

	SQLBase* findItem(Graph::NodeList items, string name);

public:

	/**
	 * identifikacni cislo prvku v databzi - je zde z duvodu identifikace konfliktu
	 */
	int id;

	/**
	 * jmeno databaze ze ktere zmena pochazi
	 */
	string dbName;

	enum COMPARE_RESULT {
		CR_NOCOMPARE = 0,		// vychozi hodnota informujici, ze se nejedna o delta prvek
		CR_ADDED,				// hodnota pro elementy, ktere byly nove pridany
		CR_CHANGED,				// hodnota pro elementy, jejichz vnitrek byl zmenen
		CR_MODIFIED,			// s touto hodnotou je pocitano do budoucna a v teto verzi neni pouzita
		CR_DELETED,				// hodnota pro elementy, tkre byly smazany
		CR_LOGIC,				// hodnota pro elementy, ktere existuji pouze jako "virtualni" (aby jine elementy mohli vzniknout bez poruseni integrity),
		CR_CONFLICT				// hodnota oznacujici, ze prvek je konfliktni (neobjevuje se po pouziti tridy Comparator, ale ConflictFinder)
	};

	COMPARE_RESULT compareResult;

	string name;

	string comment;

	SQLBase(Database* database);
	virtual ~SQLBase();

	PARAMETER& createParameter(string name);

	bool deleteParameter(string name);

	PARAMETER& getParameter(string name);

	const ParamMap& getParameters();

	SQLBase* getParent();

	bool isParameter(string name);

	bool operator==(const SQLBase& o);

	SQLBase& operator=(const SQLBase& o);
};

}

#include "Database.h"


#endif /* SQLBASE_H_ */
