/*
 * Table.h
 *
 *  Created on: 6.2.2013
 *      Author: petr
 */

#ifndef TABLE_H_
#define TABLE_H_

#include "Database.h"

namespace SQLRepresenter {

#ifndef CLASS_CHECK
class Check;
#endif

class Table : public SQLBase {

public:
	Table(Database* database, string name);
	virtual ~Table();

	/**
	 *
	 * @property string typ datoveho uloziste
	 */
	string storageType;

	/**
	 * vytvori novy sloupec
	 *
	 * @param string name jmeno sloupce
	 * @return Column*
	 */
	Column* createColumn(string name);

	/**
	 * vytvori cizi klic
	 *
	 * @param string name jmeno klice
	 * @param ColumnList columns mistni sloupce
	 * @param Table* refTable referencni tabulka
	 * @param ColumnList refColumns referencni sloupce
	 */
	ForeignKey* createForeignKey(string name, ColumnList columns, Table* refTable, ColumnList refColumns);

	/**
	 * vytvori novy kontrolni kod
	 *
	 * @param string name jmeno kontrolniho kodu
	 * @return Check*
	 */
	Check* createCheck(string name);

	/**
	 * vytvori novy index
	 *
	 * @param string name jmeno indexu
	 * @return Index*
	 */
	Index* createIndex(string name, ColumnList columns);

	/**
	 * vytvori novy trigger
	 *
	 * @param string name jmeno triggeru
	 * @return Trigger*
	 */
	Trigger* createTrigger(string name);

	/**
	 * smaze sloupec dle jmena
	 *
	 */
	void dropColumn(string name);

	/**
	 * smaze cizi klic dle jmena
	 *
	 */
	void dropForeignKey(string name);

	/**
	 * smaze kontrolni kod dle jmena
	 *
	 */
	void dropCheck(string name);

	/**
	 * smaze index dle jmena
	 *
	 */
	void dropIndex(string name);

	/**
	 * smaze trigger dle jmena
	 *
	 */
	void dropTrigger(string name);

	/**
	 * vraci sloupec dle jmena
	 * pokud neni nalezen, vraci NULL
	 *
	 * @param string name jmeno sloupce
	 * @return Column*
	 */
	Column* getColumn(string name);

	/**
	 * vraci seznam sloupcu
	 *
	 * @return ColumnList
	 */
	ColumnList getColumns();

	/**
	 * vraci cizi klic dle jmena
	 * pokud neni nalezen, vraci NULL
	 *
	 * @param string name jmeno ciziho klice
	 * @return ForeignKey*
	 */
	ForeignKey* getForeignKey(string name);

	/**
	 * vraci seznam cizich klicu
	 *
	 * @return ForeignList
	 */
	ForeignList getForeignKeys();

	/**
	 * vraci kontrolni kod dle jmena
	 * pokud neni nalezen, vraci NULL
	 *
	 * @param string name jmeno kontrolniho kodu
	 * @return Check*
	 */
	Check* getCheck(string name);

	/**
	 * vraci seznam kontrolnich kodu
	 *
	 * @return CheckList
	 */
	CheckList getChecks();

	/**
	 * vraci index dle jmena
	 * pokud neni nalezen, vraci NULL
	 *
	 * @param string name jmeno indexu
	 * @return Index*
	 */
	Index* getIndex(string name);

	/**
	 * vraci seznam indexu
	 *
	 * @return IndexList
	 */
	IndexList getIndexes();

	/**
	 * vraci primarni klic
	 *
	 * @return Index*
	 */
	Index* getPrimary();

	/**
	 * vraci trigger dle jmena
	 * pokud neni nalezen, vraci NULL
	 *
	 * @param string name jmeno triggeru
	 * @return Trigger*
	 */
	Trigger* getTrigger(string name);

	/**
	 * vraci seznam triggeru
	 *
	 * @return TriggerList
	 */
	TriggerList getTriggers();

	/**
	 * vraci TRUE, pokud sloupec s danym jmenem existuje
	 *
	 * @return bool
	 */
	bool hasColumn(string name);

	/**
	 * vraci TRUE, pokud cizi klic s danym jmenem existuje
	 *
	 * @return bool
	 */
	bool hasForeignKey(string name);

	/**
	 * vraci TRUE, pokud kontrolni kod s danym jmenem existuje
	 *
	 * @return bool
	 */
	bool hasCheck(string name);

	/**
	 * vraci TRUE, pokud index s danym jmenem existuje
	 *
	 * @return bool
	 */
	bool hasIndex(string name);

	/**
	 * vraci TRUE, pokud trigger s danym jmenem existuje
	 *
	 * @return bool
	 */
	bool hasTrigger(string name);

	Table& operator =(const Table& o);

	bool operator ==(const Table& o);
};

}


#endif /* TABLE_H_ */
