/*
 * XMLAdapter.cpp
 *
 *  Created on: 6.2.2013
 *      Author: petr
 */

#include "XMLAdapter.h"

namespace SQLRepresenter {

const pugi::char_t* XMLAdapter::ELEMENT_TABLE = "table";
const pugi::char_t* XMLAdapter::ELEMENT_COLUMN = "column";
const pugi::char_t* XMLAdapter::ELEMENT_KEY = "key";
const pugi::char_t* XMLAdapter::ELEMENT_TARCOLUMN = "tarcolumn";
const pugi::char_t* XMLAdapter::ELEMENT_REFCOLUMN = "refcolumn";
const pugi::char_t* XMLAdapter::ELEMENT_CHECK = "check";
const pugi::char_t* XMLAdapter::ELEMENT_CDATA = "code";
const pugi::char_t* XMLAdapter::ELEMENT_VIEW = "view";
const pugi::char_t* XMLAdapter::ELEMENT_FOREIGN = "foreignkey";
const pugi::char_t* XMLAdapter::ELEMENT_PARAMETER = "parameter";
const pugi::char_t* XMLAdapter::ELEMENT_PROCEDURE = "procedure";
const pugi::char_t* XMLAdapter::ELEMENT_FUNCTION = "function";
const pugi::char_t* XMLAdapter::ELEMENT_PROCPARAM = "procparam";
const pugi::char_t* XMLAdapter::ELEMENT_ROOT = "database";
const pugi::char_t* XMLAdapter::ELEMENT_TRIGGER = "trigger";

const pugi::char_t* XMLAdapter::ATTRIBUTE_NAME = "name";
const pugi::char_t* XMLAdapter::ATTRIBUTE_ID = "elementid";
const pugi::char_t* XMLAdapter::ATTRIBUTE_STORAGE = "storage";
const pugi::char_t* XMLAdapter::ATTRIBUTE_CHARSET = "charset";
const pugi::char_t* XMLAdapter::ATTRIBUTE_COLLATE = "collate";
const pugi::char_t* XMLAdapter::ATTRIBUTE_EXTRA = "extra";
const pugi::char_t* XMLAdapter::ATTRIBUTE_NOTNULL = "notnull";
const pugi::char_t* XMLAdapter::ATTRIBUTE_DATATYPE = "datatype";
const pugi::char_t* XMLAdapter::ATTRIBUTE_DEFVAL = "default";
const pugi::char_t* XMLAdapter::ATTRIBUTE_INDEXTYPE = "type";
const pugi::char_t* XMLAdapter::ATTRIBUTE_INDEXMETHOD = "method";
const pugi::char_t* XMLAdapter::ATTRIBUTE_COMMENT = "comment";
const pugi::char_t* XMLAdapter::ATTRIBUTE_TABLE = "table";
const pugi::char_t* XMLAdapter::ATTRIBUTE_REFTABLE = "reftable";
const pugi::char_t* XMLAdapter::ATTRIBUTE_ONDELETE = "ondelete";
const pugi::char_t* XMLAdapter::ATTRIBUTE_ONUPDATE = "onupdate";
const pugi::char_t* XMLAdapter::ATTRIBUTE_DIRECTION = "direction";
const pugi::char_t* XMLAdapter::ATTRIBUTE_TIME = "time";
const pugi::char_t* XMLAdapter::ATTRIBUTE_OPERATION = "operation";
const pugi::char_t* XMLAdapter::ATTRIBUTE_VALUE = "value";
const pugi::char_t* XMLAdapter::ATTRIBUTE_ACCESSMODE = "access";
const pugi::char_t* XMLAdapter::ATTRIBUTE_COMPARESTATUS = "compare";

const pugi::char_t* XMLAdapter::DIRECTION_IN = "in";
const pugi::char_t* XMLAdapter::DIRECTION_OUT = "out";
const pugi::char_t* XMLAdapter::DIRECTION_INOUT = "inout";

const pugi::char_t* XMLAdapter::TRIGGER_AFTER = "after";
const pugi::char_t* XMLAdapter::TRIGGER_BEFORE = "before";
const pugi::char_t* XMLAdapter::TRIGGER_DELETE = "delete";
const pugi::char_t* XMLAdapter::TRIGGER_INSERT = "insert";
const pugi::char_t* XMLAdapter::TRIGGER_UPDATE = "update";

const pugi::char_t* XMLAdapter::BOOL_FALSE = "false";
const pugi::char_t* XMLAdapter::BOOL_TRUE = "true";

const pugi::char_t* XMLAdapter::INDEX_KEY = "key";
const pugi::char_t* XMLAdapter::INDEX_PRIMARY = "primary";
const pugi::char_t* XMLAdapter::INDEX_UNIQUE = "unique";

const pugi::char_t* XMLAdapter::ROUTINE_CONTAINSSQL = "CONTAINS SQL";
const pugi::char_t* XMLAdapter::ROUTINE_MODIFYSQL = "MODIFY SQL";
const pugi::char_t* XMLAdapter::ROUTINE_NOSQL = "NO SQL";
const pugi::char_t* XMLAdapter::ROUTINE_READSQL = "READ SQL";

const pugi::char_t* XMLAdapter::CONFIG_FILENAME = "filename";

const pugi::char_t* XMLAdapter::COMPARE_ADD = "add";
const pugi::char_t* XMLAdapter::COMPARE_CHANGE = "change";
const pugi::char_t* XMLAdapter::COMPARE_DELETED = "deleted";
const pugi::char_t* XMLAdapter::COMPARE_LOGIC = "logic";
const pugi::char_t* XMLAdapter::COMPARE_MODIFY = "modify";
const pugi::char_t* XMLAdapter::COMPARE_NO = "no";

XMLAdapter::XMLAdapter() {
	useString = false;
}

XMLAdapter::~XMLAdapter() {
}

Database* XMLAdapter::readData(Database* database) {
	if (useString) {
		document.load(file.c_str());
	} else {
		document.load_file(file.c_str());
	}

	pugi::xml_node root = document.root().child("database");

	// nacitani tabulek
	loadTables(root, database);
	loadViews(root, database);
	loadProcedures(root, database);
	loadFunctions(root, database);
	loadForeignKeys(root, database);

	return database;
}

Database* XMLAdapter::loadTables(pugi::xml_node &root, Database* database) {
	// iterace nad tabulkami
	pugi::xml_node tableNode = root.child(ELEMENT_TABLE);
	Table* table;
	string name;
	pugi::xml_attribute attr;

	while (tableNode) {
		name = tableNode.attribute(ATTRIBUTE_NAME).as_string();
		table = database->createTable(name);
		setStandardParams(tableNode, table);

		// nastaveni dalsich dat
		loadColumns(tableNode, table);
		loadKeys(tableNode, table);
		loadTriggers(tableNode, table);
		loadChecks(tableNode, table);

		readParameters(tableNode, table);

		tableNode = tableNode.next_sibling(ELEMENT_TABLE);
	}

	return database;
}

Table* XMLAdapter::loadColumns(pugi::xml_node &node, Table* table) {
	pugi::xml_node columnNode = node.child(ELEMENT_COLUMN);
	pugi::xml_attribute attr;
	Column* column;

	while (columnNode) {
		column = table->createColumn(
				columnNode.attribute(ATTRIBUTE_NAME).as_string());

		setStandardParams(columnNode, column);

		// nastavenu povinnych atributu
		column->dataType = columnNode.attribute(ATTRIBUTE_DATATYPE).as_string();

		// vyhodnoceni nepovinnych atributu
		attr = columnNode.attribute(ATTRIBUTE_EXTRA);
		if (attr)
			column->extra = attr.as_string();

		attr = columnNode.attribute(ATTRIBUTE_NOTNULL);
		if (attr)
			column->notNull = attr.as_bool();

		attr = columnNode.attribute(ATTRIBUTE_DEFVAL);
		if (attr) {
			column->defaultUsed = true;
			column->defaultValue = attr.as_string();
		}

		columnNode = columnNode.next_sibling(ELEMENT_COLUMN);
	}

	return table;
}

void XMLAdapter::setParameter(pugi::xml_node &node, const pugi::char_t* name,
		SQLBase* elemenet) {
	// nejprve je nutne nacist atribut a zkontrolovat jestli existuje
	pugi::xml_attribute attr = node.attribute(name);

	if (attr) {
		// atribut existuje, zapise se do prvku
		PARAMETER &param = elemenet->createParameter(name);
		param.switcher = false;
		param.value = attr.as_string();
	}
}

Table* XMLAdapter::loadKeys(pugi::xml_node &node, Table* table) {
	pugi::xml_node key = node.child(ELEMENT_KEY);
	Index* index;

	while (key) {
		// aby bylo mozne klic vytvorit, musime nejprve nacist
		ColumnList columns;
		Column* column;

		pugi::xml_node columnNode = key.child(ELEMENT_TARCOLUMN);

		while (columnNode) {
			// jeste nebyly projduty vsechny sloupce
			column = table->getColumn(
					columnNode.attribute(ATTRIBUTE_NAME).as_string());
			columns.push_back(column);

			columnNode = columnNode.next_sibling(ELEMENT_TARCOLUMN);
		}

		index = table->createIndex(key.attribute(ATTRIBUTE_NAME).as_string(),
				columns);

		// vyhodnoceni typu indexu
		string indexType = key.attribute(ATTRIBUTE_INDEXTYPE).as_string();

		if (indexType == INDEX_PRIMARY) {
			index->indexType = INDEX_TYPE_PRIMARY;
		} else if (indexType == INDEX_UNIQUE) {
			index->indexType = INDEX_TYPE_UNIQUE;
		} else {
			index->indexType = INDEX_TYPE_KEY;
		}

		pugi::xml_attribute attr = key.attribute(ATTRIBUTE_INDEXMETHOD);

		if (attr)
			index->indexMethod = attr.as_string();

		// vyhodnoceni typu porovnani
		setStandardParams(key, index);

		key = key.next_sibling(ELEMENT_KEY);
	}

	return table;
}

Table* XMLAdapter::loadChecks(pugi::xml_node &node, Table* table) {
	pugi::xml_node checkNode = node.child(ELEMENT_CHECK);
	Check* check;

	while (checkNode) {
		// existuje dalsi zaznam typu CHECK
		check = table->createCheck(checkNode.attribute(ATTRIBUTE_NAME).as_string());
		setStandardParams(checkNode, check);
		check->code = checkNode.child(ELEMENT_CDATA).text().as_string();

		checkNode = checkNode.next_sibling(ELEMENT_CHECK);
	}

	return table;
}

Database* XMLAdapter::loadViews(pugi::xml_node &root, Database* database) {

	pugi::xml_node viewNode = root.child(ELEMENT_VIEW);
	View* view;

	while (viewNode) {
		view = database->createView(
				viewNode.attribute(ATTRIBUTE_NAME).as_string());

		setStandardParams(viewNode, view);
		view->code = viewNode.child(ELEMENT_CDATA).text().as_string();

		// nacitani dodatecnych atributu
		readParameters(viewNode, view);

		readParameters(viewNode, view);

		viewNode = viewNode.next_sibling(ELEMENT_VIEW);
	}

	return database;
}

void XMLAdapter::setStandardParams(pugi::xml_node &node, SQLBase* element) {
	pugi::xml_attribute attr;

	attr = node.attribute(ATTRIBUTE_NAME);
	if (attr)
		element->name = attr.as_string();

	attr = node.attribute(ATTRIBUTE_COMMENT);
	if (attr)
		element->comment = attr.as_string();

	// vyhodnoceni porovnani
	attr = node.attribute(ATTRIBUTE_COMPARESTATUS);
	if (attr) {
		string status = attr.as_string();

		if (status == COMPARE_ADD) element->compareResult = SQLBase::CR_ADDED;
		else if (status == COMPARE_CHANGE) element->compareResult = SQLBase::CR_CHANGED;
		else if (status == COMPARE_DELETED) element->compareResult = SQLBase::CR_DELETED;
		else if (status == COMPARE_LOGIC) element->compareResult = SQLBase::CR_LOGIC;
		else if (status == COMPARE_MODIFY) element->compareResult = SQLBase::CR_MODIFIED;
		else if (status == COMPARE_NO) element->compareResult = SQLBase::CR_NOCOMPARE;
	}

	// nastaveni identifikacniho cisla
	attr = node.attribute(ATTRIBUTE_ID);
	if (attr) {
		element->id = attr.as_int();
	}
}

Database* XMLAdapter::loadForeignKeys(pugi::xml_node &root, Database* database) {
	pugi::xml_node fkNode = root.child(ELEMENT_FOREIGN);
	ForeignKey* fk;
	Table *refTable, *table;

	while (fkNode) {
		table = database->table(fkNode.attribute(ATTRIBUTE_TABLE).as_string());

		refTable = database->table(
				fkNode.attribute(ATTRIBUTE_REFTABLE).as_string());

		// sestaveni seznamu sloupcu z teto tabulky
		ColumnList columns, refColumns;

		pugi::xml_node column = fkNode.child(ELEMENT_TARCOLUMN);

		while (column) {
			columns.push_back(
					table->getColumn(
							column.attribute(ATTRIBUTE_NAME).as_string()));

			column = column.next_sibling(ELEMENT_TARCOLUMN);
		}

		// nacteni referencnich sloupcu
		column = fkNode.child(ELEMENT_REFCOLUMN);

		while (column) {
			refColumns.push_back(
					refTable->getColumn(
							column.attribute(ATTRIBUTE_NAME).as_string()));

			column = column.next_sibling(ELEMENT_REFCOLUMN);
		}

		fk = table->createForeignKey(
				fkNode.attribute(ATTRIBUTE_NAME).as_string(), columns, refTable,
				refColumns);

		setStandardParams(fkNode, fk);

		// nastaveni dalsich hodnot
		fk->onDelete = fkNode.attribute(ATTRIBUTE_ONDELETE).as_string("");
		fk->onUpdate = fkNode.attribute(ATTRIBUTE_ONUPDATE).as_string("");

		fkNode = fkNode.next_sibling(ELEMENT_FOREIGN);
	}

	return database;
}

SQLBase* XMLAdapter::readParameters(pugi::xml_node &node, SQLBase* element) {
	// nacteni parametru a priprava docasneho atributu
	pugi::xml_node param = node.child(ELEMENT_PARAMETER);
	pugi::xml_attribute attr;

	while (param) {
		// dokud je k dispozici XML element parametru, probiha nacitani
		PARAMETER &localParam = element->createParameter(
				param.attribute(ATTRIBUTE_NAME).as_string());

		attr = param.attribute(ATTRIBUTE_VALUE);

		if (attr) {
			localParam.value = attr.as_string();
		} else {
			// pokud hodnota parametru neexistuje, pak se jedna o prepinac
			localParam.switcher = true;
		}

		// nacteni dalsiho parametru, aby nedoslo k zacykleni
		param = param.next_sibling(ELEMENT_PARAMETER);
	}

	return element;
}

Database* XMLAdapter::loadProcedures(pugi::xml_node &root, Database* database) {
	pugi::xml_node procedureNode = root.child(ELEMENT_PROCEDURE);
	Procedure* procedure;

	while (procedureNode) {
		procedure = database->createProcedure(
				procedureNode.attribute(ATTRIBUTE_NAME).as_string());
		setStandardParams(procedureNode, procedure);

		readCommonRoutineData(procedureNode, procedure);

		procedureNode = procedureNode.next_sibling(ELEMENT_PROCEDURE);
	}

	return database;
}

Database* XMLAdapter::loadFunctions(pugi::xml_node &root, Database* database) {
	pugi::xml_node functionNode = root.child(ELEMENT_FUNCTION);
	Function* function;

	while (functionNode) {
		function = database->createFunction(functionNode.attribute(ATTRIBUTE_NAME).as_string(), functionNode.attribute(ATTRIBUTE_DATATYPE).as_string());
		setStandardParams(functionNode, function);

		readCommonRoutineData(functionNode, function);

		functionNode = functionNode.next_sibling(ELEMENT_FUNCTION);
	}

	return database;
}

Procedure* XMLAdapter::readProcedureParams(pugi::xml_node &node,
		Procedure* procedure) {

	pugi::xml_node procParamNode = node.child(ELEMENT_PROCPARAM);

	while (procParamNode) {
		// prochazime uzly, dokud existuje dalsi uzel parametru procedury
		PROC_PARAM& procParam = procedure->addParameter(procParamNode.attribute(ATTRIBUTE_NAME).as_string(), procParamNode.attribute(ATTRIBUTE_DATATYPE).as_string());

		// zkontrolujeme, jestli existuje atribut smeru pouziti parametru procedury
		pugi::xml_attribute attr = procParamNode.attribute(ATTRIBUTE_DIRECTION);
		procParam.paramType = IN;	// vychozi nastaveni smeru

		if (attr) {
			// atribut existuje
			string attrVal = attr.as_string();

			if (attrVal == DIRECTION_OUT) {
				procParam.paramType = OUT;
			} else if (attrVal == DIRECTION_INOUT) {
				procParam.paramType = INOUT;
			}
		}

		procParamNode = procParamNode.next_sibling(ELEMENT_PROCPARAM);
	}

	return procedure;
}

Table* XMLAdapter::loadTriggers(pugi::xml_node& node, Table* table) {
	pugi::xml_node triggerNode = node.child(ELEMENT_TRIGGER);

	while (triggerNode) {
		// prochazime dokud existuje dalsi element triggeru
		Trigger* trigger = table->createTrigger(node.attribute(ATTRIBUTE_NAME).as_string());

		// nastaveni triggeru
		setStandardParams(triggerNode, trigger);
		trigger->code = triggerNode.child(ELEMENT_CDATA).text().as_string();

		// nastaveni casu
		string time = triggerNode.attribute(ATTRIBUTE_TIME).as_string();

		if (time == TRIGGER_AFTER)
			trigger->time = AFTER;

		string operation = triggerNode.attribute(ATTRIBUTE_OPERATION).as_string();

		if (operation == TRIGGER_DELETE) {
			trigger->operation = DELETE;
		} else if (operation == TRIGGER_UPDATE) {
			trigger->operation = UPDATE;
		}

		triggerNode = triggerNode.next_sibling(ELEMENT_TRIGGER);
	}

	return table;
}

pugi::xml_node XMLAdapter::saveTables(pugi::xml_node& root,
		Database* database) {

	TableList tables = database->tables();
	TableList::iterator pos = tables.begin(), end = tables.end();

	while (pos != end) {
		// dosud nebyly zpracovany vsechny tabulky databaze
		pugi::xml_node tableNode = root.append_child(ELEMENT_TABLE);
		writeStandardParams(tableNode, *pos);
		writeParameters(tableNode, *pos);

		// zapis podrizenych elementu
		saveColumns(tableNode, *pos);
		saveKeys(tableNode, *pos);
		saveChecks(tableNode, *pos);
		saveForeignKeys(root, *pos);
		saveTriggers(tableNode, *pos);

		pos++;
	}

	return root;
}

pugi::xml_node XMLAdapter::saveViews(pugi::xml_node& root, Database* database) {
	ViewList views = database->views();
	ViewList::iterator pos = views.begin(), end = views.end();

	while (pos != end) {
		// nacteni zakladnich dat o pohledu
		pugi::xml_node viewNode = root.append_child(ELEMENT_VIEW);
		writeStandardParams(viewNode, *pos);

		viewNode.append_child(ELEMENT_CDATA).text().set((*pos)->code.c_str());

		// nacteni specialnich parametru
		writeParameters(viewNode, *pos);

		pos++;
	}

	return root;
}

pugi::xml_node XMLAdapter::saveProcedures(pugi::xml_node& root,
		Database* database) {

	// iterace nad procedurami
	ProcedureList procedures = database->procedures();
	ProcedureList::iterator pos = procedures.begin(), end = procedures.end();

	while (pos != end) {
		// vytvoreni zakladniho elementu
		pugi::xml_node procedureNode = root.append_child(ELEMENT_PROCEDURE);
		writeStandardParams(procedureNode, (*pos));

		// zapiseme standadni parametry spolecne pro procedury i funkce
		writeCommonRoutineData(procedureNode, *pos);

		pos++;
	}

	return root;
}

pugi::xml_node XMLAdapter::saveFunctions(pugi::xml_node& root,
		Database* database) {

	// iterace nad procedurami
	FunctionList functions = database->functions();
	FunctionList::iterator pos = functions.begin(), end = functions.end();

	while (pos != end) {
		// vytvoreni zakladniho elementu
		pugi::xml_node functionNode = root.append_child(ELEMENT_FUNCTION);
		writeStandardParams(functionNode, (*pos));
		functionNode.append_attribute(ATTRIBUTE_DATATYPE).set_value((*pos)->returnType.c_str());

		// zapisi se data spolecne proceduram i funkcim
		writeCommonRoutineData(functionNode, *pos);

		pos++;
	}

	return root;
}

pugi::xml_node XMLAdapter::writeStandardParams(pugi::xml_node& node,
		SQLBase* element) {

	node.append_attribute(ATTRIBUTE_NAME).set_value(element->name.c_str());
	node.append_attribute(ATTRIBUTE_COMMENT).set_value(element->comment.c_str());

	pugi::xml_attribute compare = node.append_attribute(ATTRIBUTE_COMPARESTATUS);

	switch (element->compareResult) {
	case SQLBase::CR_ADDED:
		compare.set_value(COMPARE_ADD);
		break;

	case SQLBase::CR_CHANGED:
		compare.set_value(COMPARE_CHANGE);
		break;

	case SQLBase::CR_DELETED:
		compare.set_value(COMPARE_DELETED);
		break;

	case SQLBase::CR_LOGIC:
		compare.set_value(COMPARE_LOGIC);
		break;

	case SQLBase::CR_MODIFIED:
		compare.set_value(COMPARE_MODIFY);
		break;

	case SQLBase::CR_NOCOMPARE:
	default:
		compare.set_value(COMPARE_NO);
		break;
	}

	// zapis identifikacniho cisla
	node.append_attribute(ATTRIBUTE_ID).set_value(element->id);

	return node;
}

pugi::xml_node XMLAdapter::writeProcedureParams(pugi::xml_node& node,
		Procedure* procedure) {

	// zapis parametru procedury
	ProcParamList params = procedure->getParameters();
	ProcParamList::iterator pos = params.begin(), end = params.end();

	while (pos != end) {
		pugi::xml_node paramNode = node.append_child(ELEMENT_PROCPARAM);

		paramNode.append_attribute(ATTRIBUTE_NAME).set_value(pos->name.c_str());
		paramNode.append_attribute(ATTRIBUTE_DATATYPE).set_value(pos->dataType.c_str());

		pugi::xml_attribute direction = paramNode.append_attribute(ATTRIBUTE_DIRECTION);

		switch (pos->paramType) {
		case IN:

			direction.set_value(DIRECTION_IN);
			break;

		case OUT:
			direction.set_value(DIRECTION_OUT);
			break;

		case INOUT:
			direction.set_value(DIRECTION_INOUT);
			break;
		}

		pos++;
	}

	return node;
}

pugi::xml_node XMLAdapter::writeParameters(pugi::xml_node& node,
		SQLBase* element) {

	const ParamMap &params = element->getParameters();
	ParamMap::const_iterator pos = params.begin(), end = params.end();

	while (pos != end) {
		// stale jeste exituje parametr k prochazeni
		pugi::xml_node paramNode = node.append_child(ELEMENT_PARAMETER);

		// nastavei jmena
		paramNode.append_attribute(ATTRIBUTE_NAME).set_value(pos->first.c_str());

		// vyhodnoceni prepinace
		if (!pos->second.switcher) {
			paramNode.append_attribute(ATTRIBUTE_VALUE).set_value(pos->second.value.c_str());
		}

		pos++;
	}

	return node;
}

pugi::xml_node XMLAdapter::saveColumns(pugi::xml_node& tableNode,
		Table* table) {

	ColumnList columns = table->getColumns();
	ColumnList::iterator pos = columns.begin(), end = columns.end();

	while (pos != end) {
		// jeste nebyly projduty vsechny sloupce
		Column* column = *pos;
		pugi::xml_node columnNode = tableNode.append_child(ELEMENT_COLUMN);
		writeStandardParams(columnNode, column);
		writeParameters(columnNode, column);

		// zapis dalsich atributu
		columnNode.append_attribute(ATTRIBUTE_DATATYPE).set_value(column->dataType.c_str());
		columnNode.append_attribute(ATTRIBUTE_EXTRA).set_value(column->extra.c_str());

		// vyhodnoceni nullovosti
		pugi::xml_attribute notNull = columnNode.append_attribute(ATTRIBUTE_NOTNULL);
		if (column->notNull) {
			notNull.set_value(BOOL_TRUE);
		} else {
			notNull.set_value(BOOL_FALSE);
		}

		// vyhodnoceni defaultni hodnoty
		if (column->defaultUsed) {
			columnNode.append_attribute(ATTRIBUTE_DEFVAL).set_value(column->defaultValue.c_str());
		}

		pos++;
	}

	return tableNode;
}

pugi::xml_node XMLAdapter::saveKeys(pugi::xml_node& tableNode, Table* table) {
	IndexList keys = table->getIndexes();
	IndexList::iterator pos = keys.begin(), end = keys.end();

	while (pos != end) {
		// jeste nebyly zpracovany vsechny indexy
		pugi::xml_node keyNode = tableNode.append_child(ELEMENT_KEY);
		writeStandardParams(keyNode, *pos);
		writeParameters(keyNode, *pos);

		keyNode.append_attribute(ATTRIBUTE_INDEXMETHOD).set_value((*pos)->indexMethod.c_str());

		// zapis typu indexu
		pugi::xml_attribute indexType = keyNode.append_attribute(ATTRIBUTE_INDEXTYPE);

		switch ((*pos)->indexType) {
		case INDEX_TYPE_PRIMARY:
			indexType.set_value(INDEX_PRIMARY);
			break;

		case INDEX_TYPE_UNIQUE:
			indexType.set_value(INDEX_UNIQUE);
			break;

		default:
			indexType.set_value(INDEX_KEY);
			break;
		}

		// zapis sloupcu
		ColumnList columns = (*pos)->getColumns();
		ColumnList::iterator columnPos = columns.begin(), columnEnd = columns.end();

		while (columnPos != columnEnd) {
			// jeste nebyly zapsany vsechny sloupce
			keyNode.append_child(ELEMENT_TARCOLUMN).append_attribute(ATTRIBUTE_NAME)
					.set_value((*columnPos)->name.c_str());

			columnPos++;
		}

		pos++;
	}

	return tableNode;
}

pugi::xml_node XMLAdapter::saveChecks(pugi::xml_node& tableNode, Table* table) {
	CheckList checks = table->getChecks();
	CheckList::iterator pos = checks.begin(), end = checks.end();

	while (pos != end) {
		// jeste nebyly zpracovany vsechny kontroly
		pugi::xml_node checkNode = tableNode.append_child(ELEMENT_CHECK);
		writeParameters(checkNode, *pos);
		writeStandardParams(checkNode, *pos);

		// zapis kodu
		checkNode.append_child(ELEMENT_CDATA).text().set((*pos)->code.c_str());

		pos++;
	}

	return tableNode;
}

pugi::xml_node XMLAdapter::saveForeignKeys(pugi::xml_node& root,
		Table* table) {

	ForeignList foreigns = table->getForeignKeys();
	ForeignList::iterator pos = foreigns.begin(), end = foreigns.end();

	while (pos != end) {
		// jeste nebyly provedeny vsechny klice
		pugi::xml_node foreignNode = root.append_child(ELEMENT_FOREIGN);

		ForeignKey* foreign = *pos;
		writeParameters(foreignNode, foreign);
		writeStandardParams(foreignNode, foreign);

		// zapis dalsich atrbutu
		foreignNode.append_attribute(ATTRIBUTE_TABLE).set_value(table->name.c_str());
		foreignNode.append_attribute(ATTRIBUTE_REFTABLE).set_value(foreign->getReferencedTable()->name.c_str());
		foreignNode.append_attribute(ATTRIBUTE_ONDELETE).set_value(foreign->onDelete.c_str());
		foreignNode.append_attribute(ATTRIBUTE_ONUPDATE).set_value(foreign->onUpdate.c_str());

		// zapis referencovanych sloupcu
		ColumnList columns = foreign->getReferencedColumns();
		ColumnList::iterator columnPos = columns.begin(), columnEnd = columns.end();

		while (columnPos != columnEnd) {
			foreignNode.append_child(ELEMENT_REFCOLUMN).append_attribute(ATTRIBUTE_NAME).set_value((*columnPos)->name.c_str());

			columnPos++;
		}

		// zapis mistnich sloupcu
		columns = foreign->getColumns();
		columnPos = columns.begin();
		columnEnd = columns.end();

		while (columnPos != columnEnd) {
			foreignNode.append_child(ELEMENT_TARCOLUMN).append_attribute(ATTRIBUTE_NAME).set_value((*columnPos)->name.c_str());

			columnPos++;
		}

		pos++;
	}

	return root;
}

pugi::xml_node XMLAdapter::saveTriggers(pugi::xml_node& tableNode,
		Table* table) {

	TriggerList triggers = table->getTriggers();
	TriggerList::iterator pos = triggers.begin(), end = triggers.end();

	while (pos != end) {
		Trigger *trigger = *pos;
		pugi::xml_node triggerNode = tableNode.append_child(ELEMENT_TRIGGER);
		writeStandardParams(triggerNode, trigger);
		writeParameters(triggerNode, trigger);

		// nastaveni casu
		pugi::xml_attribute time = triggerNode.append_attribute(ATTRIBUTE_TIME);

		switch (trigger->time) {
		case AFTER:
			time.set_value(TRIGGER_AFTER);
			break;

		case BEFORE:
			time.set_value(TRIGGER_BEFORE);
			break;
		}

		// nastaveni operace
		pugi::xml_attribute operation = triggerNode.append_attribute(ATTRIBUTE_OPERATION);

		switch(trigger->operation) {
		case DELETE:
			operation.set_value(TRIGGER_DELETE);
			break;

		case UPDATE:
			operation.set_value(TRIGGER_UPDATE);
			break;

		case INSERT:
			operation.set_value(TRIGGER_INSERT);
			break;
		}

		// zapis kodu
		triggerNode.append_child(ELEMENT_CDATA).text().set(trigger->code.c_str());

		pos++;
	}

	return tableNode;
}

void XMLAdapter::setConfig(pugi::xml_node config) {
	pugi::xml_node file = config.child(CONFIG_FILENAME);

	if (file) {
		this->file = file.text().as_string();
	}
}

pugi::xml_node XMLAdapter::writeCommonRoutineData(pugi::xml_node& node,
		Procedure* routine) {

	// zapiseme vykonny kod
	node.append_child(ELEMENT_CDATA).text().set(routine->code.c_str());

	// zapis pristupoveho modu
	pugi::xml_attribute mode = node.append_attribute(ATTRIBUTE_ACCESSMODE);

	switch (routine->dataAccess) {
	case Procedure::DA_CONTAINSQL:
		mode.set_value(ROUTINE_CONTAINSSQL);
		break;

	case Procedure::DA_MODIFYSQLDATA:
		mode.set_value(ROUTINE_MODIFYSQL);
		break;

	case Procedure::DA_NOSQL:
		mode.set_value(ROUTINE_NOSQL);
		break;

	case Procedure::DA_READSQLDATA:
		mode.set_value(ROUTINE_READSQL);
		break;
	}

	// zapis parametru
	writeProcedureParams(node, routine);

	return node;
}

Procedure* XMLAdapter::readCommonRoutineData(pugi::xml_node& node,
		Procedure* routine) {

	readParameters(node, routine);
	routine->code = node.child(ELEMENT_CDATA).text().as_string();
	readProcedureParams(node, routine);

	// vyhodnoceni pristupu k parametrum
	std::string access(node.attribute(ATTRIBUTE_ACCESSMODE).as_string());

	if (access == ROUTINE_CONTAINSSQL) {
		routine->dataAccess = Procedure::DA_CONTAINSQL;
	} else if (access == ROUTINE_MODIFYSQL) {
		routine->dataAccess = Procedure::DA_MODIFYSQLDATA;
	} else if (access == ROUTINE_NOSQL) {
		routine->dataAccess = Procedure::DA_NOSQL;
	} else if (access == ROUTINE_READSQL){
		routine->dataAccess = Procedure::DA_READSQLDATA;
	}

	return routine;
}

void XMLAdapter::writeData(Database* database) {
	document.reset();
	ostringstream buffer;

	// vytvoreni korenoveho uzlu
	pugi::xml_node root = document.append_child(ELEMENT_ROOT);

	saveTables(root, database);
	saveViews(root, database);
	saveFunctions(root, database);
	saveProcedures(root, database);

	if (useString) {
		std::stringstream str;
		document.save(str);
		file = str.str();
	} else {
		document.save_file(file.c_str());
	}
}

void XMLAdapter::writeDiff(Database* database) {
	// v tomto pripade se jedna o normalni zapis do souboru
	writeData(database);
}

}
