/*
 * XMLAdapter.h
 *
 *  Created on: 6.2.2013
 *      Author: petr
 */

#ifndef XMLADAPTER_H_
#define XMLADAPTER_H_

#include <string>
#include <ios>
#include <sstream>
#include <iostream>
#include "Adapter.h"
#include "../ThirthParty/pugixml/pugixml.hpp"

using namespace std;

namespace SQLRepresenter {

class XMLAdapter : Adapter {

	/**
	 * dokument XML
	 */
	pugi::xml_document document;

	/**
	 * nacte tabulky a vraci databazi s tabulkami
	 *
	 * @param pugi::xml_node root korenovy uzel XML dokumentu
	 * @param Database* database ukazatel na graf databaze
	 * @return Database*
	 */
	Database* loadTables(pugi::xml_node &root, Database* database);

	/**
	 * nacte pohledy databaze
	 *
	 * @param pugi::xml_node root korenovy element XML dokumentu
	 * @param Database* database reprezentace databaze
	 * @return Database*
	 */
	Database* loadViews(pugi::xml_node &root, Database* database);

	/**
	 * nacte cizi klice databaze
	 *
	 * @param pugi::xml_node root korenovy uzel XML dokumentu
	 * @param Table* table tabulka, do ktere se bude zapisovat
	 * @return Database*
	 */
	Database* loadForeignKeys(pugi::xml_node &root, Database* database);

	/**
	 * nacte sloupce tabulky
	 *
	 * @param pugi::xml_node node uzel XML dokumentu vyjadrujici tabulku
	 * @param Table* table reprezentace tabulky, do ktere budou sloupce zapisovany
	 * @return Table*
	 */
	Table* loadColumns(pugi::xml_node &node, Table* table);

	/**
	 * nacte klice (indexy) tabulky
	 *
	 * @param pugi::xml_node node uzel XML dokumentu reprezentujici tabulku
	 * @param Table* table reprezentace tabulky, do ktere budou indexy zapisovany
	 * @return Table*
	 */
	Table* loadKeys(pugi::xml_node &node, Table* table);

	/**
	 * nacte kontrolni kody dat
	 *
	 * @param pugi::xml_node node uzel s tabulkou
	 * @param Table* table reprezentace tabulky, do ktere se bude zapisovat
	 * @return Table*
	 */
	Table* loadChecks(pugi::xml_node &node, Table* table);

	/**
	 * nacte triggery
	 *
	 * @param pugi::xml_node &node uzel ze ktereho se bude nacitat
	 * @param Table* table tabulka do ktere se bude zapisovat
	 * @return Table*
	 */
	Table* loadTriggers(pugi::xml_node &node, Table* table);

	/**
	 * nacte procedury
	 *
	 * @param pugi::xml_node root korenove element
	 * @param Database* database databaze, do ktere se bude zapisovat
	 * @return Database*
	 */
	Database* loadProcedures(pugi::xml_node &root, Database* database);

	/**
	 * nacte funkce
	 *
	 * @param pugi::xml_node root korenovy element xml dokumentu
	 * @param Database* database databaze do ktere se bude zapisovat
	 * @return Database*
	 */
	Database* loadFunctions(pugi::xml_node &root, Database* database);

	/**
	 * zapise tabulky databaze do XML uzlu
	 *
	 * @param pugi::xml_node root uzel do ktereho se bude zapisivat
	 * @param Database* database databaze, ktera je zdrojem dat
	 * @return pugi::xml_node
	 */
	pugi::xml_node saveTables(pugi::xml_node &root, Database* database);

	/**
	 * zapise pohledy do XML uzlu
	 *
	 * @param pugi::xml_node root uzel do ktereho se bude zapisovat
	 * @param Database* database databaze, ktera je zdrojem dat
	 * @return pugi::xml_node
	 */
	pugi::xml_node saveViews(pugi::xml_node &root, Database* database);

	/**
	 * zapise procedury do XML uzlu
	 *
	 * @param pugi::xml_node root uzel do ktereho se bude zapisovat
	 * @param Database* database databaze, ktera je zdrojem dat
	 * @return pugi::xml_node
	 */
	pugi::xml_node saveProcedures(pugi::xml_node &root, Database* database);

	/**
	 * zapise funkce do XML uzlu
	 *
	 * @param pugi::xml_node root uzel do ktereho se bude zapisovat
	 * @param Database* database databaze, ktera je zdrojem dat
	 * @return pugi::xml_node
	 */
	pugi::xml_node saveFunctions(pugi::xml_node &root, Database* database);

	/**
	 * zapise sloupce do XML uzlu
	 *
	 * @param pugi::xml_node tableNode uzel do ktereho se bude zapisovat
	 * @param Database* database databaze, ktera je zdrojem dat
	 * @return pugi::xml_node
	 */
	pugi::xml_node saveColumns(pugi::xml_node &tableNode, Table* table);

	/**
	 * zapise indexy do XML uzlu
	 *
	 * @param pugi::xml_node tableNode uzel do ktereho se bude zapisovat
	 * @param Database* database databaze, ktera je zdrojem dat
	 * @return pugi::xml_node
	 */
	pugi::xml_node saveKeys(pugi::xml_node &tableNode, Table* table);

	/**
	 * zapise kontrolni kody do XML uzlu
	 *
	 * @param pugi::xml_node tableNode uzel do ktereho se bude zapisovat
	 * @param Database* database databaze, ktera je zdrojem dat
	 * @return pugi::xml_node
	 */
	pugi::xml_node saveChecks(pugi::xml_node &tableNode, Table* table);

	/**
	 * zapise cizi klice do XML uzlu
	 *
	 * @param pugi::xml_node root uzel do ktereho se bude zapisovat
	 * @param Database* database databaze, ktera je zdrojem dat
	 * @return pugi::xml_node
	 */
	pugi::xml_node saveForeignKeys(pugi::xml_node &root, Table* table);

	/**
	 * zapise triggery do XML uzlu
	 *
	 * @param pugi::xml_node tableNode uzel tabulky
	 * @param Table* table tabulka, ktera je zdrojem dat
	 * @return pugi::xml_node
	 */
	pugi::xml_node saveTriggers(pugi::xml_node &tableNode, Table* table);

protected:

	/**
	 * zkontroluje, zda existuje atribut XML uzlu
	 * pokud ano, vytvori parametr SQL elementu s nazvem atributu a s jeho hodnotou
	 *
	 * @param pugi::xml_node node uzel XML
	 * @param const pugi::char_t* name jmeno atributu
	 * @param SQLBase* element uzel grafu reprezentaci SQL struktury
	 */
	void setParameter(pugi::xml_node &node, const pugi::char_t* name, SQLBase* elemenet);

	/**
	 * nastavi standardni parametry (jmeno a komentar)
	 * pokud nejsou atributy v elementu, pak nejsou nastaveny
	 *
	 * @param pugi::xml_node node uzel XML s nastavenim
	 * @param SQLBase* element prvek reprezentace SQL
	 */
	void setStandardParams(pugi::xml_node &node, SQLBase* element);

	/**
	 * nacte parametry z elementu parameter
	 *
	 * @param pugi::xml_node node matersky uzel
	 * @param SQLBase* element SQL element, do ktereho se budou parametry zapisovat
	 * @return SQLBase*
	 */
	SQLBase* readParameters(pugi::xml_node &node, SQLBase* element);

	/**
	 * nacte a zapise parametry procedury
	 *
	 * @param pugi::xml_node node uzel procedury
	 * @param Procedure* procedure
	 * @return Procedure*
	 */
	Procedure* readProcedureParams(pugi::xml_node &node, Procedure* procedure);

	/**
	 * nacte a zapise standardni data procedur a funkci
	 *
	 * @param pugi::xml_node &node uzel ze ktereho se bude nacitat
	 * @param Procedure* routine rutina
	 * @return Procedure*
	 */
	Procedure* readCommonRoutineData(pugi::xml_node &node, Procedure* routine);

	/**
	 * zapise standardni parametry do uzlu
	 *
	 * @param pugi::xml_node node uzel do ktereho se bude zapisovat
	 * @param SQLBase* element prvek, ktery je zdrojem dat
	 * @return pugi::xml_node
	 */
	pugi::xml_node writeStandardParams(pugi::xml_node &node, SQLBase *element);

	/**
	 * zapise parametry volani procedury do XML uzlu
	 *
	 * @param pugi::xml_node node uzel
	 * @param Procedure* procedure procedura
	 * @return pugi::xml_node
	 */
	pugi::xml_node writeProcedureParams(pugi::xml_node &node, Procedure* procedure);

	/**
	 * zapise do uzli nestandardni parametry
	 *
	 * @param puxi::xml_node node cilovy uzel
	 * @pram SQLBase* element zdroj dat
	 * @return pugi::xml_node
	 */
	pugi::xml_node writeParameters(pugi::xml_node &node, SQLBase* element);

	/**
	 * zapise obecna data rutiny
	 *
	 * @param pugi::xml_node &node uzel kam se bude zapisovat
	 * @param Procedure* routine rutina s daty
	 * @return pugi::xml_node
	 */
	pugi::xml_node writeCommonRoutineData(pugi::xml_node &node, Procedure* routine);

public:

	/**
	 * jmeno souboru pro cteni nebo zapis
	 */
	std::string file;

	/**
	 * pokud je TRUE nebude se zapisovat do souboru, ale do retezce file
	 */
	bool useString;

	const static pugi::char_t* ELEMENT_TABLE;
	const static pugi::char_t* ELEMENT_COLUMN;
	const static pugi::char_t* ELEMENT_KEY;
	const static pugi::char_t* ELEMENT_TARCOLUMN;
	const static pugi::char_t* ELEMENT_REFCOLUMN;
	const static pugi::char_t* ELEMENT_CHECK;
	const static pugi::char_t* ELEMENT_CDATA;
	const static pugi::char_t* ELEMENT_VIEW;
	const static pugi::char_t* ELEMENT_FOREIGN;
	const static pugi::char_t* ELEMENT_PARAMETER;
	const static pugi::char_t* ELEMENT_PROCEDURE;
	const static pugi::char_t* ELEMENT_FUNCTION;
	const static pugi::char_t* ELEMENT_TRIGGER;
	const static pugi::char_t* ELEMENT_PROCPARAM;
	const static pugi::char_t* ELEMENT_ROOT;

	const static pugi::char_t* ATTRIBUTE_NAME;
	const static pugi::char_t* ATTRIBUTE_ID;
	const static pugi::char_t* ATTRIBUTE_STORAGE;
	const static pugi::char_t* ATTRIBUTE_CHARSET;
	const static pugi::char_t* ATTRIBUTE_COLLATE;
	const static pugi::char_t* ATTRIBUTE_DATATYPE;
	const static pugi::char_t* ATTRIBUTE_EXTRA;
	const static pugi::char_t* ATTRIBUTE_NOTNULL;
	const static pugi::char_t* ATTRIBUTE_DEFVAL;
	const static pugi::char_t* ATTRIBUTE_INDEXTYPE;
	const static pugi::char_t* ATTRIBUTE_INDEXMETHOD;
	const static pugi::char_t* ATTRIBUTE_COMMENT;
	const static pugi::char_t* ATTRIBUTE_TABLE;
	const static pugi::char_t* ATTRIBUTE_REFTABLE;
	const static pugi::char_t* ATTRIBUTE_ONDELETE;
	const static pugi::char_t* ATTRIBUTE_ONUPDATE;
	const static pugi::char_t* ATTRIBUTE_VALUE;
	const static pugi::char_t* ATTRIBUTE_DIRECTION;
	const static pugi::char_t* ATTRIBUTE_TIME;
	const static pugi::char_t* ATTRIBUTE_OPERATION;
	const static pugi::char_t* ATTRIBUTE_ACCESSMODE;
	const static pugi::char_t* ATTRIBUTE_COMPARESTATUS;

	const static pugi::char_t* DIRECTION_IN;
	const static pugi::char_t* DIRECTION_OUT;
	const static pugi::char_t* DIRECTION_INOUT;

	const static pugi::char_t* TRIGGER_BEFORE;
	const static pugi::char_t* TRIGGER_AFTER;
	const static pugi::char_t* TRIGGER_INSERT;
	const static pugi::char_t* TRIGGER_UPDATE;
	const static pugi::char_t* TRIGGER_DELETE;

	const static pugi::char_t* BOOL_TRUE;
	const static pugi::char_t* BOOL_FALSE;

	const static pugi::char_t* INDEX_KEY;
	const static pugi::char_t* INDEX_UNIQUE;
	const static pugi::char_t* INDEX_PRIMARY;

	const static pugi::char_t* ROUTINE_NOSQL;
	const static pugi::char_t* ROUTINE_CONTAINSSQL;
	const static pugi::char_t* ROUTINE_READSQL;
	const static pugi::char_t* ROUTINE_MODIFYSQL;

	const static pugi::char_t* CONFIG_FILENAME;

	const static pugi::char_t* COMPARE_NO;
	const static pugi::char_t* COMPARE_ADD;
	const static pugi::char_t* COMPARE_CHANGE;
	const static pugi::char_t* COMPARE_MODIFY;
	const static pugi::char_t* COMPARE_DELETED;
	const static pugi::char_t* COMPARE_LOGIC;

	XMLAdapter();

	virtual ~XMLAdapter();

	virtual Database* readData(Database* database);

	virtual void setConfig(pugi::xml_node config);

	virtual void writeData(Database* database);

	virtual void writeDiff(Database* database);

};

}


#endif /* XMLADAPTER_H_ */
