/*
 * Configurator.cpp
 *
 *  Created on: 15.4.2013
 *      Author: petr
 */

#include "Configurator.h"

const char* Configuration::NODE_DB = "db";
const char* Configuration::NODE_PATH = "path";
const char* Configuration::NODE_ROOT = "config";
const char* Configuration::NODE_USERINFO = "user";
const char* Configuration::NODE_ACTIVE = "active";

const char* Configuration::ATTR_EMAIL = "email";
const char* Configuration::ATTR_USERNAME = "name";
const char* Configuration::ATTR_ADAPTER = "adapter";

const char* Configuration::ADAPTER_MYSQL = "mysql";

Configuration::Configuration() {
	// vygenerovani uzlu
	xml_node root = document.append_child(NODE_ROOT);

	user = root.append_child(NODE_USERINFO);
	dbconfig = root.append_child(NODE_DB);
	dbconfig.append_attribute(ATTR_ADAPTER);

	path = root.append_child(NODE_PATH);
	active = root.append_child(NODE_ACTIVE);

	user.append_attribute(ATTR_USERNAME).set_value("");
	user.append_attribute(ATTR_EMAIL).set_value("");
}

Configuration::~Configuration() {
	// TODO Auto-generated destructor stub
}

SQLRepresenter::Adapter* Configuration::createAdapter() {
	// vyhodnoceni adapteru
	SQLRepresenter::Adapter *retVal = NULL;

	string dbAdapter = getDbAdapter();

	if (getDbAdapter() == ADAPTER_MYSQL) {
		retVal = new SQLRepresenter::MySQLAdapter();
		retVal->setConfig(getDbConfig());
	}

	return retVal;
}

void Configuration::exportConfig(string name) {
	string path = generateFileName(name);

	document.save_file(path.c_str());
}

string Configuration::getDbAdapter() {
	return dbconfig.attribute(ATTR_ADAPTER).as_string();
}

xml_node Configuration::getDbConfig() {
	return dbconfig;
}

string Configuration::getDbParam(string name) {
	/// prvni uzel parametru
	xml_node node = findDbParam(name);

	return node ? node.attribute(SQLRepresenter::Adapter::ATTR_VALUE).as_string() : "";
}

string Configuration::getEmail() {
	return string(user.attribute(ATTR_EMAIL).as_string());
}

string Configuration::getPath() {
	return path.text().as_string();
}

string Configuration::getUserName() {
	return string(user.attribute(ATTR_USERNAME).as_string());
}

void Configuration::setAdapter(const char *adapter) {
	dbconfig.attribute(ATTR_ADAPTER).set_value(adapter);
}

void Configuration::setDbParam(string name, string value) {
	xml_node node = findDbParam(name);

	if (!node) {
		node = dbconfig.append_child(SQLRepresenter::Adapter::NODE_PARAM);
		node.append_attribute(SQLRepresenter::Adapter::ATTR_NAME).set_value(name.c_str());
		node.append_attribute(SQLRepresenter::Adapter::ATTR_VALUE);
	}

	node.attribute(SQLRepresenter::Adapter::ATTR_VALUE).set_value(value.c_str());
}

void Configuration::setEmail(string email) {
	user.attribute(ATTR_EMAIL).set_value(email.c_str());
}

void Configuration::setPath(string path) {
	this->path.text().set(path.c_str());
}

string Configuration::generateFileName(string name) {
	return "./" + name + ".xml";
}

void Configuration::importConfig(string name) {
	string path = generateFileName(name);

	document.load_file(path.c_str());

	xml_node root = document.child(NODE_ROOT);
	user = root.child(NODE_USERINFO);
	dbconfig = root.child(NODE_DB);
	this->path = root.child(NODE_PATH);
	active = root.child(NODE_ACTIVE);
}

string Configuration::getActive() {
	return active.text().as_string();
}

void Configuration::setActive(string uuid) {
	active.text().set(uuid.c_str());
}

Configuration::Configuration(const Configuration& o) : Configuration() {
	user.attribute(ATTR_USERNAME).set_value(o.user.attribute(ATTR_USERNAME).value());
	user.attribute(ATTR_EMAIL).set_value(o.user.attribute(ATTR_EMAIL).value());
	path.text().set(o.path.text().as_string());
	active.text().set(o.active.text().as_string());
	dbconfig.attribute(ATTR_ADAPTER).set_value(o.dbconfig.attribute(ATTR_ADAPTER).as_string());

	// prepis parametru databaze
	xml_node child = o.dbconfig.child(SQLRepresenter::Adapter::NODE_PARAM);

	while (child) {
		xml_node thisChild = dbconfig.append_child(SQLRepresenter::Adapter::NODE_PARAM);

		thisChild.append_attribute(SQLRepresenter::Adapter::ATTR_NAME).set_value(child.attribute(SQLRepresenter::Adapter::ATTR_NAME).as_string());
		thisChild.append_attribute(SQLRepresenter::Adapter::ATTR_VALUE).set_value(child.attribute(SQLRepresenter::Adapter::ATTR_VALUE).as_string());

		child = child.next_sibling(SQLRepresenter::Adapter::NODE_PARAM);
	}
}

void Configuration::setUserName(string name) {
	user.attribute(ATTR_USERNAME).set_value(name.c_str());
}

xml_node Configuration::findDbParam(string name) {
	xml_node retVal = dbconfig.child(SQLRepresenter::Adapter::NODE_PARAM);

	while (retVal && name != retVal.attribute(SQLRepresenter::Adapter::ATTR_NAME).as_string()) retVal = retVal.next_sibling(SQLRepresenter::Adapter::NODE_PARAM);

	return retVal;
}

void createConfiguration(string name) {
	char tmp[1024];
	Configuration conf;

	// jmeno
	cout << "Your name:      ";
	cin.getline(tmp, 1024);
	conf.setUserName(tmp);

	// email
	cout << "Your email:     ";
	cin.getline(tmp, 1024);
	conf.setEmail(tmp);

	// cesta k souborum
	cout << "Path to files:  ";
	cin.getline(tmp, 1024);
	conf.setPath(tmp);

	// zjisteni DB adapteru
	cout << "Choose DB adapter:" << endl;
	cout << "1. MySQL" << endl;

	cout << endl;
	char dbIndex;
	cin >> dbIndex;

	switch (dbIndex) {
	case '1':
		conf.setAdapter(Configuration::ADAPTER_MYSQL);
		createMySQLConfiguration(conf);
		break;

	default:
		cout << "Invalid adapter. Ending" << endl;
		return;
	}

	conf.exportConfig(name);
}

void createMySQLConfiguration(Configuration& config) {
	char tmp[1024];

	cin.ignore();

	// adresa server
	cout << "Server address: ";
	cin.getline(tmp, 1024);

	config.setDbParam(SQLRepresenter::MySQLAdapter::CONF_HOST, tmp);

	// port
	cout << "Database name:  ";
	cin.getline(tmp, 1024);

	config.setDbParam(SQLRepresenter::MySQLAdapter::CONF_DBNAME, tmp);

	// uzivatel
	cout << "User name:      ";
	cin.getline(tmp, 1024);

	config.setDbParam(SQLRepresenter::MySQLAdapter::CONF_USER, tmp);

	// adresa serverz
	cout << "Password:       ";
	cin.getline(tmp, 1024);

	config.setDbParam(SQLRepresenter::MySQLAdapter::CONF_PASSWORD, tmp);
}

void showConfiguration(string name) {
	Configuration conf;
	conf.importConfig(name);

	// vypis informaci
	cout << "Name:               " << conf.getUserName() << endl;
	cout << "Email:              " << conf.getEmail() << endl;
	cout << "Active package:     " << conf.getActive() << endl;
	cout << "File path:          " << conf.getPath() << endl << endl;
	cout << "Adapter:            " << conf.getDbAdapter() << endl;
	cout << "=========" << endl << endl;

	showMySQLConfiguration(conf);
}

void showMySQLConfiguration(Configuration &conf) {
	cout << "Host:               " << conf.getDbParam(SQLRepresenter::MySQLAdapter::CONF_HOST) << endl;
	cout << "Database name:      " << conf.getDbParam(SQLRepresenter::MySQLAdapter::CONF_DBNAME) << endl;
	cout << "Username:           " << conf.getDbParam(SQLRepresenter::MySQLAdapter::CONF_USER) << endl;
	cout << "Password:           " << conf.getDbParam(SQLRepresenter::MySQLAdapter::CONF_PASSWORD) << endl;
}
