/*
 * DiffPackage.cpp
 *
 *  Created on: 17.4.2013
 *      Author: petr
 */

#include "DiffPackage.h"

const char *DiffPackage::NODE_DAY = "day";
const char *DiffPackage::NODE_EMAIL = "email";
const char *DiffPackage::NODE_HOUR = "hour";
const char *DiffPackage::NODE_MINUTE = "minute";
const char *DiffPackage::NODE_MONTH = "month";
const char *DiffPackage::NODE_PARENT = "parent";
const char *DiffPackage::NODE_PARENTS = "parents";
const char *DiffPackage::NODE_ROOT = "meta";
const char *DiffPackage::NODE_SECOND = "second";
const char *DiffPackage::NODE_TIME = "time";
const char *DiffPackage::NODE_USER = "user";
const char *DiffPackage::NODE_YEAR = "year";
const char *DiffPackage::NODE_BRANCH = "branch";
const char *DiffPackage::NODE_CONFLICTS = "conflicts";
const char *DiffPackage::NODE_CONFLICT = "conflict";

const char *DiffPackage::ATTR_DB = "db";
const char *DiffPackage::ATTR_ID = "id";
const char *DiffPackage::ATTR_METHOD = "method";

const char *DiffPackage::METHOD_CURRENT = "current";
const char *DiffPackage::METHOD_OTHER = "other";

const char *DiffPackage::EXTENSION = ".zip";

const char *DiffPackage::META_FILE = "meta.xml";
const char *DiffPackage::DATA_FILE = "diff.xml";

DiffPackage::DiffIndex DiffPackage::pool = DiffPackage::DiffIndex();
DiffPackage::BranchList DiffPackage::branches = DiffPackage::BranchList();

DiffPackage::DiffPackage() {
	diffDb = NULL;
	readOnly = false;

}

DiffPackage::~DiffPackage() {
	// TODO Auto-generated destructor stub
}

SQLRepresenter::Database* DiffPackage::getDatabase() {
	return diffDb;
}

DiffPackage::META_INFO DiffPackage::getMetaInfo() {
	return metaInfo;
}

DiffPackage::META_INFO& DiffPackage::meta() {
	return metaInfo;
}

void DiffPackage::setCurrentTime() {
	time_t currTime = time(NULL);
	metaInfo.time = *gmtime(&currTime);
}

string DiffPackage::generateMetaXML() {
	xml_document document;
	stringstream buffer;

	// zapis informaci o uzivatlei
	xml_node root = document.append_child(NODE_ROOT);
	root.append_child(NODE_USER).text().set(metaInfo.name.c_str());
	root.append_child(NODE_EMAIL).text().set(metaInfo.email.c_str());

	// zapis casu
	xml_node time = root.append_child(NODE_TIME);
	time.append_child(NODE_YEAR).text().set(metaInfo.time.tm_year);
	time.append_child(NODE_MONTH).text().set(metaInfo.time.tm_mon);
	time.append_child(NODE_DAY).text().set(metaInfo.time.tm_mday);
	time.append_child(NODE_HOUR).text().set(metaInfo.time.tm_hour);
	time.append_child(NODE_MINUTE).text().set(metaInfo.time.tm_min);
	time.append_child(NODE_SECOND).text().set(metaInfo.time.tm_sec);

	// zapis vetve
	root.append_child(NODE_BRANCH).text().set(metaInfo.branch.c_str());

	// zapis predku balicku
	xml_node parents = root.append_child(NODE_PARENTS);

	PackageUuids::iterator pos = metaInfo.parents.begin(), end = metaInfo.parents.end();

	while (pos != end) {
		parents.append_child(NODE_PARENT).text().set(pos->c_str());

		pos++;
	}

	xml_node conflicts = root.append_child(NODE_CONFLICTS);

	// zapis konfliktu
	SQLRepresenter::Conflicts::iterator cPos = this->conflicts.begin(),
			cEnd = this->conflicts.end();

	while (cPos != cEnd) {
		SQLRepresenter::ConflictList::iterator tPos = cPos->second.begin(),
				tEnd = cPos->second.end();

		while (tPos != tEnd) {
			xml_node conflict = conflicts.append_child(NODE_CONFLICT);

			// zapis informaci o reseni konfliktu
			conflict.append_attribute(ATTR_ID).set_value(tPos->id);
			conflict.append_attribute(ATTR_DB).set_value(cPos->first.c_str());
			conflict.append_attribute(ATTR_METHOD).set_value(tPos->useCurrent ? METHOD_CURRENT : METHOD_OTHER);

			tPos++;
		}

		cPos++;
	}

	// vygenerovani XML do bufferu
	document.save(buffer);

	return buffer.str();
}

void DiffPackage::setDatabase(SQLRepresenter::Database* database) {
	diffDb = database;
}

string DiffPackage::calculateSHA1(string filename) {
	// nacteni souboru
	string retVal;

	const char *file = filename.c_str();
	CryptoPP::SHA1 hash;

	CryptoPP::FileSource((file), true,
			new CryptoPP::HashFilter(hash,
					new CryptoPP::HexEncoder(new CryptoPP::StringSink(retVal), false)
	));

	return retVal;
}

void DiffPackage::loadPool(Configuration config) {
	// faze 0 -> zniceni starych instanci
	DiffIndex::iterator pos = DiffPackage::pool.begin(), end = DiffPackage::pool.end();

	while (pos != end) {
		delete pos->second;
		pos++;
	}

	DiffPackage::pool.clear();

	// prochazeni adresarem
	path p(config.getPath());

	if (!is_directory(p)) return;

	directory_iterator directory(p), endDir;

	while (directory != endDir) {
		directory_entry entry = *directory;

		if (is_regular_file(entry.status())) {
			// polozka je souborem, zkontroluje se, jestli je ZIP
			path entryPath = entry.path();
			string ext = entryPath.extension().string();
			if (entryPath.extension().string() == EXTENSION) {
				if (entryPath.stem().string().size() == 40) {
					// pravdepodobne se jedna o spravny soubor - provede se nacteni a zapis do poolu
					if (!loadFile(entryPath)) {
						cout << "Invalid file entry" << endl;
						throw exception();
					}
				}
			}
		}

		directory++;
	}


	// prochazeni adresarem a hledani balicku
	/** @todo NACITANI ADRESARE */

	// faze 2 - vytvoreni vazeb a sestaveni seznamu vetvi
	pos = DiffPackage::pool.begin(); end = DiffPackage::pool.end();

	while (pos != end) {
		DiffPackage *current = pos->second;

		// prohledani metainformace a zapis predku a potomku
		PackageUuids::iterator pPos = current->metaInfo.parents.begin(),
				pEnd = current->metaInfo.parents.end();

		while (pPos != pEnd) {
			DiffPackage *target = pool[*pPos];
			target->children.push_back(current);
			current->parents.push_back(target);

			pPos++;
		}

		// zaneseni id vetve
		writeBranchId(current);

		pos++;
	}
}

bool DiffPackage::loadFile(path source) {
	// otevreni a nacteni dat
	zip *zipFile = zip_open(source.c_str(), ZIP_SOURCE_OPEN, NULL);

	if (zipFile == NULL) return false;

	// nacteni meta informaci
	struct zip_stat stat;
	if (zip_stat(zipFile, META_FILE, ZIP_STAT_SIZE, &stat)) return false;

	// alokace bufferu a nactani dat
	allocator<char> alloc;
	char *buffer = alloc.allocate(stat.size);

	zip_file *content = zip_fopen(zipFile, META_FILE, ZIP_SOURCE_OPEN);
	if (content == NULL) return false;
	if (zip_fread(content, buffer, stat.size) == -1) return false;

	zip_fclose(content);

	xml_document metaXml;
	metaXml.load(buffer, stat.size);

	// nacteni databaze
	if (zip_stat(zipFile, DATA_FILE, ZIP_STAT_SIZE, &stat)) return false;
	buffer = alloc.allocate(stat.size, buffer);
	content = zip_fopen(zipFile, DATA_FILE, ZIP_SOURCE_OPEN);
	if (content == NULL) return false;

	if (zip_fread(content, buffer, stat.size) == -1) return false;

	// vytvoreni balicku
	DiffPackage *package = new DiffPackage();
	package->name = source.stem().string();
	package->readOnly = true;

	// nacteni dat databaze
	SQLRepresenter::XMLAdapter xml;
	xml.file = buffer;
	xml.useString = true;
	SQLRepresenter::Database *db = new SQLRepresenter::Database();
	db->name = package->name;
	xml.readData(db);

	package->diffDb = db;

	// nactnei metadat
	xml_node root = metaXml.child(NODE_ROOT), node;
	META_INFO &info = package->metaInfo;

	info.name = root.child(NODE_USER).text().get();
	info.email = root.child(NODE_EMAIL).text().get();
	info.branch = root.child(NODE_BRANCH).text().get();

	// nacteni predku
	xml_node parents(root.child(NODE_PARENTS));
	xml_node parent = parents.child(NODE_PARENT);

	while (parent) {
		info.parents.push_back(parent.text().get());
		parent = parent.next_sibling();
	}

	// nacteni casu
	xml_node times = root.child(NODE_TIME);

	if (times) {
		// uzel byl nazelen a zpracuji se prvky
		info.time.tm_hour = times.child(NODE_HOUR).text().as_int();
		info.time.tm_min = times.child(NODE_MINUTE).text().as_int();
		info.time.tm_sec = times.child(NODE_SECOND).text().as_int();
		info.time.tm_year = times.child(NODE_YEAR).text().as_int();
		info.time.tm_mon = times.child(NODE_MONTH).text().as_int();
		info.time.tm_mday = times.child(NODE_DAY).text().as_int();
	} else {
		// uzel nebyl nalezen a zapisi se nuly
		info.time.tm_hour = 0;
		info.time.tm_min = 0;
		info.time.tm_sec = 0;
		info.time.tm_year = 0;
		info.time.tm_mon = 0;
		info.time.tm_mday = 0;
	}

	// zpracovani konfliktu
	xml_node conflictItem = root.child(NODE_CONFLICTS).child(NODE_CONFLICT);

	while (conflictItem) {
		SQLRepresenter::CONFLICT_SOLVE solve;
		solve.id = conflictItem.attribute(ATTR_ID).as_int();
		solve.solved = true;

		string method = conflictItem.attribute(ATTR_METHOD).as_string();

		if (method == METHOD_CURRENT) {
			solve.useCurrent = true;
		} else {
			solve.useCurrent = false;
		}

		conflictItem = conflictItem.next_sibling();

		package->conflicts[conflictItem.attribute(ATTR_DB).as_string()].push_back(solve);
	}


	// zapis do poolu
	pool[package->name] = package;

	return true;
}

string DiffPackage::writeFile(string target) {
	if (readOnly) return name;

	string tmpName = target + "tmp.zip";

	zip* zipFile = zip_open(tmpName.c_str(), ZIP_CREATE, NULL);

	// zapis metainformaci
	string meta = generateMetaXML();
	zip_source *metaSource = zip_source_buffer(zipFile, meta.c_str(),
			meta.size(), 0);
	zip_add(zipFile, META_FILE, metaSource);

	// zapis databaze
	SQLRepresenter::XMLAdapter xmlAdapter;
	xmlAdapter.useString = true;
	xmlAdapter.writeData(diffDb);

	zip_source *dataSource = zip_source_buffer(zipFile, xmlAdapter.file.c_str(),
			xmlAdapter.file.size(), 0);
	zip_add(zipFile, DATA_FILE, dataSource);

	zip_close(zipFile);

	// vypocet kontrolniho souctu a prejmenovani souboru
	string sha = calculateSHA1(tmpName);

	string fileName = target + sha + EXTENSION;

	boost::filesystem3::rename(tmpName.c_str(), fileName.c_str());

	name = sha;

	return sha;
}

string DiffPackage::generateBranchName() {
	// vygenerovani prvotniho nastrelu z casu
	string retVal;
	time_t currTime = time(NULL);
	char buffer[17];
	buffer[16] = 0;

	BranchList::iterator beg = branches.begin(), end = branches.end();

	do {
		sprintf(buffer, "%lX", currTime);
		retVal = buffer;

		currTime++;
	} while (branches.find(retVal) != end);

	return retVal;
}

DiffPackage* DiffPackage::createPackage(DiffPackage* parent) {
	DiffPackage *retVal = new DiffPackage();

	retVal->setCurrentTime();

	if (parent == NULL) {
		retVal->metaInfo.branch = generateBranchName();
	} else {
		// balicek ma predka
		retVal->metaInfo.parents.push_back(parent->name);
		retVal->metaInfo.branch = parent->metaInfo.branch;
		retVal->parents.push_back(parent);
		parent->children.push_back(retVal);
	}

	return retVal;
}

void DiffPackage::writeBranchId(DiffPackage *package) {
	string name = package->metaInfo.branch;

	if (branches.find(name) == branches.end()) {
		// vetev jeste neni zapsana
		branches[name] = package;
	} else {
		// vete je zapsana, zkontroluje se potreba aktualizace
		time_t currTime, newTime;
		currTime = mktime(&branches[name]->metaInfo.time);
		newTime = mktime(&package->metaInfo.time);

		if (currTime < newTime) {
			branches[name] = package;
		}
	}
}

DiffPackage* DiffPackage::getPackage(string name) {
	DiffPackage *retVal;
	DiffIndex::iterator pkg = pool.find(name);

	retVal = (pkg == pool.end()) ? NULL : pkg->second;

	return retVal;
}

SQLRepresenter::Database* DiffPackage::assembleDatabase() {
	SQLRepresenter::Database *retVal = new SQLRepresenter::Database();
	doAssemble(retVal, conflicts);

	return retVal;
}

void DiffPackage::doAssemble(SQLRepresenter::Database* db, SQLRepresenter::Conflicts conflicts) {
	Packages::iterator pos = parents.begin(),
			end = parents.end();

	// vyhodnoceni poctu predku
	SQLRepresenter::LocalAdapter local;
	local.target = db;
	local.conflicts = conflicts;

	switch (parents.size()) {
	case 1:
	case 2:
		// z podstaty vytvareni balicku je zajisteno, ze prvni predek je predek z nasi vetve
		parents[0]->doAssemble(db, conflicts);

	case 0:
		local.writeDiff(diffDb);
		break;
	}
}

DiffPackage::Packages DiffPackage::getMergeables() {
	Packages retVal;

	// postupne se prochazeji jednotlive vetve a hleda se slouceni HEADu
	BranchList::iterator pos = branches.begin(), end = branches.end();

	while (pos != end) {
		// pokud dany head neni primo v historii, pak se prida do navratoveho seznamu
		if (pos->second->metaInfo.branch != metaInfo.branch && !inHistory(pos->second)) retVal.push_back(pos->second);

		pos++;
	}

	return retVal;
}

bool DiffPackage::inHistory(DiffPackage* package) {

	// rekurzivni prochazeni predku a hledani daneho balicku
	Packages::iterator pos = parents.begin(), end = parents.end();
	bool retVal = false;

	while (pos != end && !retVal) {
		DiffPackage *current = *pos;

		if (current->name == package->name) {
			// jedna se o predka
			retVal = true;
		} else {
			retVal = current->inHistory(package);
		}

		pos++;
	}

	return retVal;
}

void DiffPackage::getDeltaChain(SQLRepresenter::DeltaChain& current,
		SQLRepresenter::DeltaChain& other, string branch) {

	// vyhledani prvniho merge s druhou vetvi
	Packages *fullPath = findFirstPackage(branch, NULL);
	DiffPackage *otherPackage = branches[branch];
	Packages thisPath, otherPath;

	// vyhodnoceni jeslti byla cesta nalezena
	if (fullPath == NULL) {
		// cesta nebyla nalezena - pouziji se cele vetve
		getPath(thisPath, NULL);
		otherPackage->getPath(otherPath, NULL);
	} else {
		// cesta byla nalezena - pouziji se jen potrebne casti vetvi
		otherPackage->getPath(otherPath, fullPath->front());

		// sestaveni lokalni cesty
		Packages::reverse_iterator pos = fullPath->rbegin(), end = fullPath->rend();

		do {
			thisPath.push_back(*pos);

			pos++;
		} while (pos != end && (*pos)->metaInfo.branch == metaInfo.branch);

		// elementy v thisPath jsou momentalne obracene -> musi se zmenit jejich poradi
		reverse(thisPath.begin(), thisPath.end());

		// likvidace alokovane pameti
		delete fullPath;
	}

	generateDeltaChain(current, thisPath);
	generateDeltaChain(other, otherPath);
}

DiffPackage::Packages* DiffPackage::findFirstPackage(string branch,
		DiffPackage* currentPackage) {

	// pokud pokud je tento balicek starsi nez posledne nalezeny, nic sedelta nebude
	if (currentPackage != NULL) {
		time_t currTime = mktime(&currentPackage->metaInfo.time),
				thisTime = mktime(&metaInfo.time);

		if (currTime >= thisTime) return NULL;
	}

	DiffPackage::Packages *retVal = NULL;

	// kontrola jestli vyhovuje vetev tohoto balicku
	if (metaInfo.branch == branch) {
		// vetev vyhovuje - vytvori se navratova hodnota
		retVal = new Packages();
		retVal->push_back(this);

		return retVal;
	}

	// tento balicek neni soucasti vetve - zkontroluji se predci
	Packages::iterator pos = parents.begin(), end = parents.end();

	while (pos != end) {
		DiffPackage *parent = *pos;
		Packages* currentPath = NULL;

		currentPath = parent->findFirstPackage(branch, currentPackage);

		// pokud bylo neco nalezeno - byl nalezen novy aktualnejsi balicek
		if (currentPath != NULL) {
			// zniceni stare navratove hodnoty pokud je to potreba
			if (retVal != NULL) delete retVal;

			retVal = currentPath;

			// nastavei noveho nejnovejsiho balicku
			currentPackage = retVal->front();
		}

		pos++;
	}

	return retVal;
}

void DiffPackage::getPath(Packages& path, DiffPackage* limit) {
	// vyhodnoceni, jestli se ma pokracovat
	if (limit != NULL && name == limit->name) {
		return;
	}

	// prochazeni predku a nalezeni toho ze sve vetve
	Packages::iterator pos = parents.begin(),
			end = parents.end();

	while (pos != end) {
		// vyhodnoceni vetve
		DiffPackage *package = *pos;

		if (package->metaInfo.branch == this->metaInfo.branch) {
			// spravny predek byl nalezen
			package->getPath(path, limit);
			break;
		}

		pos++;
	}

	path.push_back(this);
}

void DiffPackage::generateDeltaChain(SQLRepresenter::DeltaChain& chain,
		Packages packages) {

	Packages::iterator pos = packages.begin(), end = packages.end();

	while (pos != end) {
		chain.push_back((*pos)->diffDb);

		pos++;
	}
}

void DiffPackage::addParent(DiffPackage* parent) {
	if (readOnly) return;

	// kontrola, jestli rodic existuje
	Packages::iterator pos = parents.begin(), end = parents.end();

	while (pos != end) {
		if ((*pos)->name == parent->name) return;

		pos++;
	}

	// zapis predka
	parents.push_back(parent);
	metaInfo.parents.push_back(parent->name);
}

string DiffPackage::getName() {
	return name;
}

DiffPackage& DiffPackage::operator =(const DiffPackage& o) {
	diffDb = o.diffDb;
	metaInfo = o.metaInfo;
	readOnly = o.readOnly;
	parents = o.parents;
	children = o.children;

	return *this;
}
