/*
 * diff.cpp
 *
 *  Created on: 17.4.2013
 *      Author: petr
 */


#include "diff.h"

void createDiff(string config) {
	// nacteni konfigurace a adapteru
	Configuration conf;
	conf.importConfig(config);
	SQLRepresenter::Adapter *adapter = conf.createAdapter();

	// nacteni poolu
	DiffPackage::loadPool(conf);

	// kontrola, jestli je uz vytvoren nejaky diff balicek
	string active = conf.getActive();

	if (!active.size()) {
		// zadny balicek zatim nebyl vytvoren -> vytvori se novy balicek
		createFirstDiff(adapter, conf);
		conf.exportConfig(config);
		return;
	}

	// zarazeni noveho balickui za existujici balicek
	DiffPackage *current = DiffPackage::getPackage(active);

	if (current == NULL) return;

	// sestaveni databaze
	SQLRepresenter::Database *oldDb = current->assembleDatabase(),
			*newDb = new SQLRepresenter::Database(),
			*diffDb;

	adapter->readData(newDb);

	// porovnani databazi
	SQLRepresenter::Comparator comparator;
	diffDb = comparator.compare(oldDb, newDb);

	// kotnrola, jeslti se neco zmenilo
	if (diffDb->getNodes(NODE_TABLE | NODE_VIEW | NODE_TRIGGER | NODE_PROCEDURE | NODE_FUNCTION).size() == 0) {
		// nic se nezmenilo
		cout << "No changes found" << endl;
		return;
	}

	// vytvoreni noveho balicku
	DiffPackage &pkg = *DiffPackage::createPackage(current);
	createDiffPackage(conf, diffDb, current);

	conf.exportConfig(config);

	cout << "Diff package created." << endl;
	cout << "Its name is " << conf.getActive() << endl;
}

void createFirstDiff(SQLRepresenter::Adapter *adapter, Configuration &config) {
	SQLRepresenter::Database* local = new SQLRepresenter::Database();
	SQLRepresenter::Database* substract;

	substract = compareDb(adapter, local);

	createDiffPackage(config, substract, NULL);
}

void createDiffPackage(Configuration &config, SQLRepresenter::Database* delta, DiffPackage *parent) {
	DiffPackage &pkg = *DiffPackage::createPackage(parent);
	pkg.setCurrentTime();
	pkg.setDatabase(delta);

	pkg.meta().email = config.getEmail();
	pkg.meta().name = config.getUserName();

	string newActive = pkg.writeFile(config.getPath());
	config.setActive(newActive);
}

SQLRepresenter::Database* compareDb(SQLRepresenter::Adapter *adapter, SQLRepresenter::Database* localDb) {
	SQLRepresenter::Database* serverDb = new SQLRepresenter::Database();
	adapter->readData(serverDb);

	SQLRepresenter::Comparator comparator;

	return comparator.compare(localDb, serverDb);
}
