/*
 * merge.cpp
 *
 *  Created on: 9.5.2013
 *      Author: petr
 */

#include "merge.h"

#define SOLVETABLEC(NODE_TYPE) solveTableConflicts(conflictTable->childNodes(NODE_TYPE), currentTable->childNodes(NODE_TYPE), otherTable->childNodes(NODE_TYPE), conflicts)

void doMerge(string config) {
	// nacteni konfigurace a balicku
	Configuration conf;
	conf.importConfig(config);

	DiffPackage::loadPool(conf);

	// nacteni aktivniho balicku
	DiffPackage *current = DiffPackage::getPackage(conf.getActive());

	if (current == NULL) {
		cout << "No active package found" << endl;
		return;
	}

	// nacteni seznamu vetvi, ktere je mozne sloucit a vypsani jejich seznamu
	DiffPackage::Packages posibles = current->getMergeables();

	// kontrola jeslti je nejaky balicek dostupny
	if (posibles.size() == 0) {
		cout << "No branches for merging found" << endl;
		return;
	}

	cout << "0 - end" << endl;
	DiffPackage::Packages::iterator pos = posibles.begin(), end = posibles.end();

	unsigned int index = 1;

	while (pos != end) {
		// nacteni metadat a sestaveni casu
		DiffPackage::META_INFO info = (*pos)->meta();
		char buffer[30];

		strftime(buffer, 80, "%d. %m %Y %H:%M:%S", &info.time);

		cout << index++ << " - " << info.name << " (" << info.email << "; " << buffer << endl;

		pos++;
	}

	cout << endl;
	cout << "Select branch to merge: ";
	cin >> index;

	// kontrola indexu
	if (index == 0) {
		cout << "No branch selected... ending" << endl;
		return;
	}

	if (index < 0 || index > posibles.size()) {
		cout << "Invalid branch index. Ending" << endl;
		return;
	}

	// sestaveni seznamu po prvni merge
	DiffPackage *other = posibles[index - 1];

	SQLRepresenter::DeltaChain currChain, otherChain;
	current->getDeltaChain(currChain, otherChain, other->getMetaInfo().branch);

	SQLRepresenter::ConflictFinder finder;
	Conflicts conflicts;
	finder.conflicts = conflicts;

	SQLRepresenter::Database *conflict = finder.findConflicts(currChain, otherChain);

	// sestaveni databazi
	SQLRepresenter::Database *currentDatabase = current->assembleDatabase(),
			*otherDatabase = other->assembleDatabase();

	// kontrola jestli doslo ke konfliktu
	while (conflict->getNodes(SQLRepresenter::NODE_ALL).size()) {
		// dokud dochazi ke konfliktu, provadi se vyhodnocovani
		solveConflicts(finder.conflicts, currentDatabase);

		delete conflict;

		conflict = finder.findConflicts(currChain, otherChain);
	}

	delete conflict;

	// vytvoreni balicku
	DiffPackage &package = *DiffPackage::createPackage(current);
	package.addParent(other);

	package.conflicts = conflicts;
	SQLRepresenter::Database* diff = finder.build(otherChain);
	package.setDatabase(diff);

	// zapis do MySQL
	Adapter* adapter = conf.createAdapter();
	adapter->writeDiff(diff);
	delete adapter;

	// ulozeni dat
	string newActive = package.writeFile(conf.getPath());

	cout << "Branches merged. Ending" << endl;

	conf.setActive(newActive);
	conf.exportConfig(config);
}

SQLBase *getByName(NodeList nodes, string name) {
	NodeList::iterator pos = nodes.begin(), end = nodes.end();

	while (pos != end) {
		SQLBase *node = (SQLBase *) *pos;
		if (node->name == name) return node;

		pos++;
	}

	return NULL;
}

int solveConflicts(Conflicts &conflicts, SQLRepresenter::Database *currentDb) {
	// prochazeni konfliktu a zapis dat
	Conflicts::iterator pos = conflicts.begin(), end = conflicts.end();
	int retVal = 0;

	while (pos != end) {
		string dbName = pos->first;
		ConflictList::iterator cPos = pos->second.begin(), cEnd = pos->second.end();
		DiffPackage *otherPackage = DiffPackage::getPackage(dbName);

		while (cPos != cEnd) {
			// kontrola vyreseni
			if (cPos->solved) {
				cPos++;
				continue;
			}

			// nacteni zmeny a stavajiciho stavu
			SQLBase *change = otherPackage->getDatabase()->getById(cPos->id);
			SQLBase *current = NULL;
			NodeList currentNodes;
			Table* changeTable = NULL;

			// musime vyhodnotit, zda je jedna o tabulkovou zmenu nebo o ostatni
			if (change->nodeType & (NODE_VIEW | NODE_TABLE | NODE_FUNCTION | NODE_PROCEDURE)) {
				// zakladni prvek
				currentNodes = currentDb->getNodes(change->nodeType);
			} else {
				// prvek uvnitr tabulky - musime nacist tabulku
				changeTable = (Table*) change->parentNodes(NODE_TABLE).front();
				Table* currentTable = currentDb->table(changeTable->name);

				if (currentTable != NULL) {
					currentNodes = currentTable->childNodes(change->nodeType);
				}
			}

			current = getByName(currentNodes, change->name);

			int answer = askConflictQuestion(change->nodeType, current, change, changeTable);

			cPos->solved = true;
			cPos->useCurrent = (answer == 1) ? true : false;

			cPos++;
			retVal++;
		}

		pos++;
	}

	return retVal;
}

int askConflictQuestion(long int nodeType, SQLBase* current,
		SQLBase* other, SQLBase *table) {

	// vyhodoceni tabulky
	if (table != NULL) {
		cout << "Table - " << table->name << " -> ";
	}

	// vypsani dotazu
	cout << "Conflict on ";
	switch (nodeType) {
	case NODE_PROCEDURE:
		cout << "procedure:";
		break;

	case NODE_FUNCTION:
		cout << "function:";
		break;

	case NODE_VIEW:
		cout << "view:";
		break;

	case NODE_TABLE:
		cout << "table:";
		break;

	case NODE_COLUMN:
		cout << "column:";
		break;

	case NODE_TRIGGER:
		cout << "trigger:";
		break;

	case NODE_INDEX:
		cout << "index:";
		break;

	case NODE_FOREIGN_KEY:
		cout << "foreign key:";
		break;
	}

	cout << endl;
	int retVal;

	do {
		cout << "1 - THIS BRANCH     " << printOp(current) << endl;
		cout << "2 - OTHER BRANCH    " << printOp(other) << endl << endl;
		cout << "Your choose: ";

		cin >> retVal;
	} while (retVal != 1 && retVal != 2);

	return retVal;

}

string printOp(SQLBase* item) {

	if (item == NULL) {
		return "DELETED";
	}

	switch (item->compareResult) {
	case SQLBase::CR_ADDED:
		return "ADDED";
		break;

	case SQLBase::CR_CHANGED:
		return "CHANGED";
		break;

	case SQLBase::CR_DELETED:
		return "DELETED";
		break;

	case SQLBase::CR_MODIFIED:
		return "MODIFIED";
		break;
	}

	return "";
}
